/*  allest.hpp

    Class definitions for all estimators.

    This file is part of Xtremes for Windows,
    Copyright (C) 1995 Xtremes Group, Siegen.
*/

#ifndef ALLEST_HPP
#define ALLEST_HPP



#ifndef TERMINAL
#include "estcen.hpp"
#else
#include "estimate.hpp"
#endif
#include "jensest.hpp"

/********************************************************************
			GP/GP I estimators
********************************************************************/

class THillEstimator : public TUnivariateGPDEstimator {
    public:
	THillEstimator (BOOL AddList = FALSE);
	virtual TEstResult Estimate (int k, TSample *);
	virtual TEstimatorData *GetParameter ();
	virtual TEstimator *Duplicate ();
    protected:
	virtual int GetMinK ();
	virtual void CalcEstimator (Array<double> &alpha, int &kmax, TSample *);

	virtual TEstMode GetEstimatorMode ();
    private:
	TEstimatorData Data;
};


class TMEstimatorData : public TEstimatorData {
    public:
	double rho1, rho2, q;
	int MFunc;
};


class TMEstimator : public TUnivariateGPDEstimator {
    typedef TUnivariateGPDEstimator inherited;
    public:
	TMEstimator (BOOL AddMenu = FALSE);
	virtual TEstResult Estimate (int k, TSample *);
	virtual TEstimatorData *GetParameter ();
	virtual TEstimator *Duplicate ();
    protected:
	virtual TEstMode GetEstimatorMode ();
#ifndef TERMINAL
	virtual void NotifyInitDialog ();
	virtual void EvaluateAdditionalParameters ();
#endif
    private:
	void CalcMFunction (int Nr, TUnivariateSample &Data, int n, int k, double a,
			    double rho, double &f, double &fs);
	TMEstimatorData Data;
#ifndef TERMINAL
	TDoubleField Rho1;
#endif
};


class TMLEEstimator : public TUnivariateGPDEstimator {
    public:
	TMLEEstimator (BOOL AddList = FALSE);
	virtual TEstResult Estimate (int k, TSample *);
	virtual TEstimatorData *GetParameter ();
	virtual TEstimator *Duplicate ();
    protected:
	virtual int GetMinK ();
        virtual double GetExponent ();
	virtual void CalcEstimator (Array<double> &alpha, int &kmax, TSample *);
	virtual TEstMode GetEstimatorMode ();
    private:
	void FindStart (double &bstart, double &tstart, int type,
			TUnivariateSample *p, double s, int k ,int n);
	int FindZero (double &alpha, double &sigma, double &mu,
		      TUnivariateSample *p, double s, int k, int n);
	double bstart, tstart;
	TEstimatorData Data;
};


class TMomentEstimator : public TUnivariateGPDEstimator {
    public:
	TMomentEstimator (BOOL AddMenu = FALSE);
	virtual TEstResult Estimate (int k, TSample *);
	virtual TEstimatorData *GetParameter ();
	virtual TEstimator *Duplicate ();
    protected:
	virtual int GetMinK ();
	virtual void CalcEstimator (Array<double> &alpha, int &kmax, TSample *);
	virtual TEstMode GetEstimatorMode ();
    private:
	TEstimatorData Data;
	double BetaEst (TUnivariateSample &, int k);
};


class TSlopeEstimator : public TUnivariateGPDEstimator {
    public:
	TSlopeEstimator (BOOL AddMenu = FALSE);
	virtual TEstResult Estimate (int k, TSample *);
	virtual TEstimatorData *GetParameter ();
	virtual TEstimator *Duplicate ();
    protected:
	virtual int GetMinK ();
	virtual void CalcEstimator (Array<double> &alpha, int &kmax, TSample *);
	virtual TEstMode GetEstimatorMode ();
    private:
	TEstimatorData Data;
};

class TLMomentGPEstimator : public TUnivariateGPDEstimator {
    public:
	TLMomentGPEstimator (BOOL AddMenu = FALSE);
	virtual TEstResult Estimate (int k, TSample *);
	virtual TEstimatorData *GetParameter ();
	virtual TEstimator *Duplicate ();
    protected:
	virtual int GetMinK ();
	virtual void CalcEstimator (Array<double> &alpha, int &kmax, TSample *);
	virtual TEstMode GetEstimatorMode ();
    private:
	TEstimatorData Data;
};

class TDreesPickandsEstimator : public TUnivariateGPDEstimator {
    public:
	TDreesPickandsEstimator (BOOL AddMenu = FALSE);
	virtual TEstResult Estimate (int k, TSample *);
	virtual TEstimatorData *GetParameter ();
	virtual TEstimator *Duplicate ();
    protected:
	virtual int GetMinK ();
	virtual void CalcEstimator (Array<double> &alpha, int &kmax, TSample *);
	virtual TEstMode GetEstimatorMode ();
    private:
	double DQEst (TUnivariateSample &s, int k, BOOL EstAlphaRec);
	double betadach (TUnivariateSample &s, int k, BOOL EstAlphaRec);
	int rnd (double x);
	double BetaEst (int k, TUnivariateSample &s);
	TEstimatorData Data;
};

class TExponentialEstimator : public TUnivariateGPDEstimator {
    public:
	TExponentialEstimator (BOOL AddMenu = FALSE);
	virtual TEstResult Estimate (int k, TSample *);
	virtual TEstimatorData *GetParameter ();
	virtual TEstimator *Duplicate ();
    protected:
	virtual TEstMode GetEstimatorMode ();
    private:
	TEstimatorData Data;
};

class TGroupedMLEstimator : public TGroupedGPDEstimator {
    public:
	TGroupedMLEstimator (BOOL AddList = FALSE);
	virtual TEstResult Estimate (int k, TSample *);
	virtual TEstimator *Duplicate ();
	virtual TEstimatorData *GetParameter ();
    protected:
	TEstMode GetEstimatorMode ();
    private:
	void evalphi (double &phi, double &dphi, double &d2phi, double a, int i, TGroupedSample *, int Offset);
	void evalg (double &g, double &dg, double a, TGroupedSample *, int Offset, int NoG);
	double alphaGroup (double a, TGroupedSample *, int Offset, int NoG);
	double sigmaGroup (TGroupedSample *, int Offset, double size1, double size2, double a);
	TEstimatorData Data;
};

class TGroupedGPMLEstimator : public TGroupedGPDEstimator {
    public:
	TGroupedGPMLEstimator (BOOL AddList = FALSE);
	virtual TEstResult Estimate (int k, TSample *);
	virtual TEstimator *Duplicate ();
	virtual TEstimatorData *GetParameter ();
    protected:
	TEstMode GetEstimatorMode ();
    private:
	void SetValues (double *grenz, int *anz, int nr, int anzdat);
	BOOL iterate(double& gamma, double& eta);
        TEstimatorData Data;

        // The following functions are used by iterate:

	double phi(int i, double gamma, double eta);
	double dgphi(int i, double gamma, double eta);
	double dephi(int i, double gamma, double eta);
	double dgdgphi(int i, double gamma, double eta);
	double dedgphi(int i, double gamma, double eta);
	double dgdephi(int i, double gamma, double eta);
	double dedephi(int i, double gamma, double eta);
	double likeli(double gamma, double eta);
	double dglikeli(double gamma, double eta);
	double delikeli(double gamma, double eta);
	double dgdglikeli(double gamma, double eta);
	double dedglikeli(double gamma, double eta);
	double dgdelikeli(double gamma, double eta);
	double dedelikeli(double gamma, double eta);
	double iterateGrad(double& gamma, double& eta, double gradgamma,
			  double gradeta, double value);
	BOOL InRange(double gamma, double eta);
	double *u;   // Grenzwerte
	int *n;      // n[i] = | [ u[i], u[i+1] ) |
	int m;       // Anzahl Grenzen
	int anzdata; // Anzahl gezogener Ereignisse
	int k;       // Anzahl Excedenten ueber u[0]
};

#ifndef TERMINAL
#include "parbayes.hpp"
#endif

/********************************************************************
	       GP/GP I estimators for censored data
********************************************************************/

#ifndef TERMINAL

class THillCenEstimator : public TCensoredGPDEstimator {
    public:
	THillCenEstimator (BOOL AddList = FALSE);
	virtual TEstResult Estimate (int k, TSample *);
	virtual TEstimatorData *GetParameter ();
	virtual TEstimator *Duplicate ();
    protected:
	virtual TEstMode GetEstimatorMode ();
    private:
	TEstimatorData Data;
};

class TMLECenEstimator : public TCensoredGPDEstimator {
    public:
	TMLECenEstimator (BOOL AddList = FALSE);
	virtual TEstResult Estimate (int k, TSample *);
	virtual TEstimatorData *GetParameter ();
	virtual TEstimator *Duplicate ();
    protected:
	virtual TEstMode GetEstimatorMode ();
    private:
	void FindStart (double &bstart, double &tstart, int type,
			TCensoredSample *p, double s, int k ,int n);
	int FindZero (double &alpha, double &sigma, double &mu,
		      TCensoredSample *p, double s, int k, int n);
	TEstimatorData Data;
	double bstart, tstart;
};

#endif

/********************************************************************
			EV/EV I estimators
********************************************************************/

#include "evdest.hpp"

/********************************************************************
			Gaussian estimators
********************************************************************/

#include "normest.hpp"

/********************************************************************
			Poisson estimators
********************************************************************/

#ifndef TERMINAL
#include "poissest.hpp"
#endif

/********************************************************************
			Negative binomial estimators
********************************************************************/

#ifndef TERMINAL
#include "negbin.hpp"
#endif

/********************************************************************
		Global pointers to estimators
********************************************************************/

extern TEstimator *HillGP1, *HillCen, *MEGP1, *MLEGP, *MLECen, *MomentGP,
	          *SlopeGP, *DreesPickandsGP, *LMomentGP, *BayesEstPareto,
		  *MLEEV1, *MLEEV, *RDEEV, *MDEEV, *MDEGroupedEV,
		  *MLEGaussian, *LSGaussian, *MHDEGaussian, *MDEGaussian, *MLET, *MLENCT,
		  *LSEGaussian, *MLEGumbel, *MLEExponential, *QuickGaussian
#ifdef USE_STABLE
                  , *MLEStable, *KKWStable
#endif
		  ;

void BuildEstimators ();

#endif
