/*  drees.cpp

    Implemenation of Drees-Pickands estimator.

    This file is part of Xtremes for Windows,
    Copyright (C) 1994 Xtremes Group, Siegen.
*/

// b, m, s

#include <math.h>
#include "allest.hpp"
#include "mathfunc.hpp"
#include "stddef.hpp"

#ifndef TERMINAL
#include "resource.h"
#endif

#define LOG2 0.6931472

TDreesPickandsEstimator::TDreesPickandsEstimator (BOOL AddMenu)
#ifdef TERMINAL
  : TUnivariateGPDEstimator ("Drees-Pickands (GP)") {
#else
  : TUnivariateGPDEstimator ("Drees-Pickands (GP)", IDM_DreesPickands, IDD_Estimator, AddMenu) {
#endif
    Data.SetMode (BETA_MODE);
}

double TDreesPickandsEstimator::DQEst (TUnivariateSample &s, int k, BOOL EstAlphaRec) {
    int x2, m = k >> 2;
    double h1, h2;
    int size = s.GetDataSize ();
    do {
        x2 = size - (m << 1);
        h1 = s[size - m] - s[x2];
        h2 = s[x2] - s[size - (m << 2)];
        m++;
    } while ((h1 == 0.0 || h2 == 0.0) && m <= size / 4);
    double h3 = h2 == 0.0 ? MYMAXDOUBLE : log (h1 / h2) / LOG2;
    return EstAlphaRec ?
                h3 :
                h3 == 0.0 ? MYMAXDOUBLE : 1.0 / h3;
}

double TDreesPickandsEstimator::betadach (TUnivariateSample &s, int k, BOOL EstAlphaRec) {
    return DQEst (s, (k << 2), EstAlphaRec);
}


int TDreesPickandsEstimator::rnd (double x) {
    const double EPS = 1e-6;
    return (int) floor (x + 1 - EPS);
//    return (int) floor (x) + 1;
}

double TDreesPickandsEstimator::BetaEst (int k, TUnivariateSample &s) {
    if (k < 4) return DQEst (s, k, TRUE);
    int m = k / 4,
        l = max ((int) floor (log (k) / LOG2) - 1,1);
    double *c = new double [l],
           b = 0.0;
    for (int j = 0; j < 3; j++) {
        if (b < -0.5) b = -(1.0 + b);
        double Pwr2b = pow (2.0, b + 1.0);
        int i;
        for (i = 0; i < l - 1; i++)
            c [i] = (b != 0.0 ?
                 (Pwr2b - 1.0) / (Pwr2b * 0.5 - 1.0) *
                   (1.0 - pow (2.0, -(i+1) * b)) * pow (2.0, -i) :
                 (i+1) * pow (2.0, -i)
                ) / 4.0;
        c [l-1] = b != 0.0 ?
                 pow (2.0, -l) / (Pwr2b * 0.5 - 1.0) *
                   (Pwr2b - 1.0 - pow (2.0, (1-l)*b)) :
                 (l+1) * pow (2.0, -l);
	b = 0.0;
        for (i = 0; i < l; i++)
            b += c[i] * betadach (s, rnd (m * pow (2, -i)), TRUE);
    }
    delete c;
    return b;
}

TEstMode TDreesPickandsEstimator::GetEstimatorMode () {
    return BETA_MODE;
}

TEstResult TDreesPickandsEstimator::Estimate (int k, TSample *q) {
    TUnivariateSample *p = (TUnivariateSample *) q;
    if (k < 4) return EST_EXT;
    if (k > p->GetDataSize ()) return EST_PAR;
    RawParameters [0] = BetaEst (k, *p);
    TransformModels (k, p);
    return EST_OK;
}

TEstimatorData *TDreesPickandsEstimator::GetParameter () {
    return &Data;
}

TEstimator *TDreesPickandsEstimator::Duplicate () {
    TDreesPickandsEstimator *p = new TDreesPickandsEstimator;
    p->Data = Data;
    return p;
}

int TDreesPickandsEstimator::GetMinK () {
    return 4;
}

void TDreesPickandsEstimator::CalcEstimator (Array<double> &alpha, int &kmax, TSample *q) {
    TUnivariateSample &p = *(TUnivariateSample *) q;
    kmax = p.GetDataSize ();
    int k = GetMinK ();
    while (k <= kmax) {
        Estimate (k, q);
        alpha [k++] = RawParameters [0];
    }
}
