/*  errors.cpp
 *
 *  Error messages of XPL compiler.
 *
 *  This file is part of Xtremes for Windows,
 *  Copyright (C) 1998 Xtremes Group, Siegen.
 */

#include <stdio.h>
#include <stdlib.h>

#include "errors.hpp"
#include "string.hpp"
#include "compiler.hpp"

TErrorClass::TErrorClass (TErrorType e, int l, String Desc, String File) {
    ErrorType = e;
    ErrorLine = l;
    Description = Desc;
    FileName = File;
}
 
String TErrorClass::GetErrorMessage () {
    String msg;
    switch (ErrorType) {
        case SYNTAX_ERROR:
            msg = "Syntax error"; break;
        case IDENTIFIER_NOT_FOUND:
            msg = "Identifier not declared"; break;
        case STATEMENT_EXPECTED:
            msg = "Statement expected"; break;
        case IDENTIFIER_EXPECTED:
            msg = "Identifier expected"; break;
        case CONST_EXPECTED:
            msg = "Constant expected"; break;
        case INVALID_TYPE:
            msg = "Unknown type"; break;
        case STRING_VAR_EXPECTED:
            msg = "String variable expected"; break;
	case NUMERIC_VAR_EXPECTED:
            msg = "Numeric variable expected"; break;
        case PARAMETERTYPE_NOT_ALLOWED:
            msg = "Parameter type is not allowed"; break;
        case RETURN_ONLY_IN_FUNCTION:
            msg = "Return is only allowed in a function body"; break;
        case ARRAY_EXPECTED:
            msg = "Array expected"; break;
	case COMPONENT_NOT_FOUND:
	    msg = "Component name is not part of record"; break;
	case INVALID_USE_OF_SYMBOL:
	    msg = "Invalid use of symbol"; break;
        case INCOMPATIBLE_TYPES:
            msg = "Incompatible types"; break;
        case FUNCTIONCALL_EXPECTED:
            msg = "Function call expected"; break;
	case IDENTIFIER_ALREADY_DECLARED:
	    msg = "Identifier declared twice"; break;
        case PROCEDURECALL_EXPECTED:
            msg = "Procedure call expected"; break;
        case INTEGERCONSTANTS_EXPECTED:
            msg = "Integer constants expected"; break;
	case POINTER_REQUIRED:
	    msg = "Pointer type required in new statement"; break;
        case VARIABLE_EXPECTED:
            msg = "Variable expected"; break;
        case INTEGER_VAR_EXPECTED:
            msg = "Integer variable expected"; break;
        case MODE_REQUIRED:
            msg = "Estimation model required"; break;
        case DIVISION_BY_ZERO:
            msg = "Division by zero"; break;
        case POSITION_FOUND:
	    msg = "Run time error position found"; break;
	case MEMORY_FULL:
            msg = "Memory full"; break;
	case INTERNAL_ERROR:
	    msg = "Internal error"; break;
	case FILE_NOT_FOUND:
            msg = "Source file of unit not found"; break;
        default:
            msg = "No error message available for error"; break;
    }
    msg = FileName & form (50, ":%d: ", ErrorLine) & msg;
    if (Description.Len ()) msg = msg & ": "  & String (Description);
    return msg;
}   

TErrorClass *TErrorList::GetFirstError () {
    return (TErrorClass *) GetFirst ();
}

TErrorClass *TErrorList::GetNextError () {
    return (TErrorClass *) GetNext ();
}

void ErrorMessage (TErrorType e, int Line, String Description, String FileName) {
    if (FileName == String ("")) FileName = Scanner->GetFile ();
    ErrorList.Append (new TErrorClass (e, Line, Description, FileName));
    Compiler->IncErrorCount ();
}
    
TErrorList ErrorList;
