/*  errors.hpp
 *
 *  Error messages of XPL compiler.
 *
 *  This file is part of XPL
 *  Copyright (C) 1998 Xtremes Group, Siegen.
 */

#ifndef ERRORS_HPP
#define ERRORS_HPP

#include "string.hpp"
#include "list.hpp"

enum TErrorType {
	      SYNTAX_ERROR, IDENTIFIER_NOT_FOUND, STATEMENT_EXPECTED,
              IDENTIFIER_EXPECTED, CONST_EXPECTED, INVALID_TYPE,
              STRING_VAR_EXPECTED, NUMERIC_VAR_EXPECTED,
              PARAMETERTYPE_NOT_ALLOWED, RETURN_ONLY_IN_FUNCTION,
	      ARRAY_EXPECTED, INCOMPATIBLE_TYPES, FUNCTIONCALL_EXPECTED,
              PROCEDURECALL_EXPECTED, INTEGERCONSTANTS_EXPECTED,
	      COMPONENT_NOT_FOUND, IDENTIFIER_ALREADY_DECLARED,
              VARIABLE_EXPECTED, INTEGER_VAR_EXPECTED, MODE_REQUIRED,
	      DIVISION_BY_ZERO, INVALID_USE_OF_SYMBOL, POINTER_REQUIRED,
	      FILE_NOT_FOUND, MEMORY_FULL, INTERNAL_ERROR,
              POSITION_FOUND };


class TErrorClass : public TObject {
    public:
	TErrorClass (TErrorType ErrorType, int ErrorLine, String Desc, String File);
	TErrorType ErrorType;
	int        ErrorLine;
	String     Description;
	String     FileName;
	String GetErrorMessage ();
};

class TErrorList : public TList {
    public:
	TErrorClass *GetFirstError (),
                    *GetNextError ();
};

extern TErrorList ErrorList;

void ErrorMessage (TErrorType e, int Line, String Description = "", String FileName = "");

#endif

