/*  evdest.hpp

    Estimators for extreme value distributions.

    This file is part of Xtremes for Windows,
    Copyright (C) 1994 Xtremes Group, Siegen.
*/

#ifndef EVDEST_HPP
#define EVDEST_HPP

#include "estimate.hpp"

class TDiffRatioEstimator : public TEVDEstimator {
    public:
	TDiffRatioEstimator (BOOL AddList = FALSE);
	TEstimator *Duplicate ();

	virtual TEstResult Estimate (int k, TSample *);
	virtual TEstimatorData *GetParameter ();
    protected:
	double RDEEstimate (int i, double n, TUnivariateSample &Sample);
	virtual TEstMode GetEstimatorMode ();
	TEstimatorData Data;
};

class TFrechetMLEstimator : public TEVDEstimator {
    public:
	TFrechetMLEstimator (BOOL AddList = FALSE);
	TEstimator *Duplicate ();

	virtual TEstResult Estimate (int k, TSample *);
	virtual TEstimatorData *GetParameter ();
    protected:
	virtual TEstMode GetEstimatorMode ();		   
	TEstimatorData Data;
    private:
	TEstResult DoEstimation (TSample *);
	TUnivariateSample *Sample;
	double h0 (double, double sign);
	double h1 (double, double sign);
	double h2 (double, double sign);
};

class TGumbelMLEstimator : public TGumbelEstimator {
    public:
	TGumbelMLEstimator (BOOL AddList = FALSE);
	TEstimator *Duplicate ();
	virtual TEstResult Estimate (int k, TSample *);
    private:
	TUnivariateSample *Sample;
	double h0 (double);
	double h1 (double);
	double h2 (double);
};

class TEVDMLEstimator : public TEVDEstimator {
    public:
	TEVDMLEstimator (BOOL AddList = FALSE);
	TEstimator *Duplicate ();
	virtual TEstResult Estimate (int k, TSample *);
	virtual TEstimatorData *GetParameter ();
    protected:
	virtual TEstMode GetEstimatorMode ();		
	TEstimatorData Data;
    private:
	double xmin, xmax;
	TEstResult DoEstimation (TSample *);
	BOOL IsRangeValid (double beta, double sigma, double mu);
	double l (double beta, double sigma, double mu);	// Log-Likelihood
	double dldb (double beta, double sigma, double mu);	// Derivatives
	double dlds (double beta, double sigma, double mu);
	double dldm (double beta, double sigma, double mu);
	void CalcDL (double beta, double sigma, double mu,
		     double &dldb, double &dlds, double &dldm); 
	void CalcDl (double beta, double sigma, double mu, double a [3][3],
	             double &dldbeta, double &dldsigma, double &dldmu);	// Hessian matrix
	BOOL Iter (double &b, double &s, double &m);
	double RDEEstimate (int i, double n, TUnivariateSample &Sample);
	void InitEstimator ();
	void InitEstimator2 (double &b, double &s, double &m);
};

#endif
