/*  expest.cpp

    Implemenation of ML-estimator for Exponential distribution.

    This file is part of Xtremes for Windows,
    Copyright (C) 1995 Xtremes Group, Siegen.
*/

#include <math.h>
#include "allest.hpp"
#include "mathfunc.hpp"
#include "stddef.hpp"

#ifndef TERMINAL
#include "resource.h"
#endif

#define LOG2 0.6931472

// b,m,s

TExponentialEstimator::TExponentialEstimator (BOOL AddMenu)
#ifndef TERMINAL
  : TUnivariateGPDEstimator ("MLE(GP 0)", IDM_MLEGPD3, IDD_Estimator, AddMenu) {
#else
  : TUnivariateGPDEstimator ("MLE(GP 0)") {
#endif
    Data.SetMode (BETA_MODE);
}


TEstMode TExponentialEstimator::GetEstimatorMode () {
    return FULL_BETA_MODE;
}

TEstResult TExponentialEstimator::Estimate (int k, TSample *q) {
    TUnivariateSample *p = (TUnivariateSample *) q;
    int n = p->GetDataSize ();
    if (k < 2) return EST_EXT;
    if (k > n) return EST_PAR;
    double sigma = 0.0, t = (*p) [n - k];
    for (int i = n - k + 1; i < n; i++)
        sigma += ((*p) [i] - t) / (k - 1);
    RawParameters [0] = 0.0;
    RawParameters [2] = sigma;
    RawParameters [1] = t + sigma * log ((double) k / (double) n);
    TransformModels (k, p);
    return EST_OK;
}

TEstimatorData *TExponentialEstimator::GetParameter () {
    return &Data;
}

TEstimator *TExponentialEstimator::Duplicate () {
    TExponentialEstimator *p = new TExponentialEstimator;
    p->Data = Data;
    return p;
}
