/*  hill.cpp

    Implemenation of Hill estimator.

    This file is part of Xtremes for Windows,
    Copyright (C) 1994 Xtremes Group, Siegen.
*/

// b,m,s

#include <math.h>
#include "allest.hpp"
#include "mathfunc.hpp"

#ifndef TERMINAL
#include "resource.h"
#endif

THillEstimator::THillEstimator (BOOL AddList)
#ifdef TERMINAL
 : TUnivariateGPDEstimator ("Hill(GP 1)") {
#else
 : TUnivariateGPDEstimator ("Hill(GP 1)", IDM_Hill, IDD_Estimator, AddList) {
#endif
    Data.SetMode (ALPHA_MODE);
}

TEstMode THillEstimator::GetEstimatorMode () {
    return ALPHA_MODE;
}

TEstResult THillEstimator::Estimate (int k, TSample *q) {
    TUnivariateSample *p = (TUnivariateSample *) q;
    int n = p->GetDataSize ();
    if (k < 3) return EST_EXT;
    if (k > n) return EST_PAR;

    double sum = 0.0, alpha;
    if ((*p) [n - 1] <= 0.0) {
	for (int i = 1; i < k; i++)
	    sum -= log (-(*p) [n - i]);
	alpha = - (sum / (k - 1) + log (-(*p) [n - k]));
    } else if ((*p) [n - k] <= 0.0)
	return EST_NEG;
    else {
	for (int i = 1; i < k; i++)
	    sum += log ((*p) [n - i]);
	alpha = sum / (k - 1) - log ((*p) [n - k]);
    }

    RawParameters [0] = (alpha != 0.0) ? 1.0 / alpha : MYMAXDOUBLE;
    TransformModels (k, p);
    return EST_OK;
}

TEstimatorData *THillEstimator::GetParameter () {
    return &Data;
}

TEstimator *THillEstimator::Duplicate () {
    THillEstimator *p = new THillEstimator;
    p->Data = Data;
    return p;
}

int THillEstimator::GetMinK () {
    return 2;
}

void THillEstimator::CalcEstimator (Array<double> &alpha, int &kmax, TSample *q) {
    TUnivariateSample &p = *(TUnivariateSample *) q;
    int k = 2;
    alpha [1] = 0.0;
    int n = p.GetDataSize ();
    if (p [n - 1] < 0.0) {
	while (k <= n) {
	    alpha [k] = alpha [k - 1] - log (-p [n - k + 1]);
	    k++;
	}
	kmax = k - 1;
	for (k = 2; k <= kmax; k++)
	    alpha [k] = - (1.0 / (alpha [k] / (k - 1) + log (-p [n - k])));
    } else {
	while (k <= n && p [n - k] > 0.0) {
	    alpha [k] = alpha [k - 1] + log (p [n - k + 1]);
	    k++;
	}
	kmax = k - 1;
	for (k = 2; k <= kmax; k++)
	    alpha [k] = 1.0 / (alpha [k] / (k - 1) - log (p [n - k]));
    }
}
