/*  mdest.cpp */

// b,m,s

#include <math.h>
#include <stdlib.h>

#include "allest.hpp"
#include "mathfunc.hpp"
#include "stddef.hpp"

#ifndef TERMINAL
  #include "resource.h"
#endif


const double EPSILON = 1e-6;


// Hilfsfunktionen fr zweiparametrige Schtzer

const int m0 = 2;

static int IsPositiveDefinit (double a [2][2]) {
    return a [0][0] > 0.0 &&
	   a [0][0] * a [1][1] - a [0][1] * a [1][0] > 0.0;
}

static int IsNegativeDefinit (double a [2][2]) {
    return a [0][0] < 0.0 &&
           a [0][0] * a [1][1] - a [0][1] * a [1][0] < 0.0;
}

static void Invert (double a [m0][m0]) {
    int i, j, k, r;
    double maxv, hr;
    double hv [m0];
    int p [m0];
    for (j = 0; j < m0; j++) p [j] = j;
    for (j = 0; j < m0; j++) {
        maxv = fabs (a [j][j]);
        r = j;
        for (i = j + 1; i < m0; i++)
            if (fabs (a [i][j]) > maxv) {
                maxv = fabs (a [i][j]);
                r = i;
	    }
        if (maxv == 0.0) {
	    puts ("Matrix ist singulr\n");
//          abort ();
        }
        if (r > j) {
            for (k = j; k < m0; k++) Swap (a [j][k], a [r][k]);
            Swap (p [j], p [r]);
        }
        hr = 1.0 / a [j][j];
        for (i = 0; i < m0; i++)
            a [i][j] *= hr;
        a [j][j] = hr;
        for (k = 0; k < m0; k++)
            if (k != j) {
		for (i = 0; i < m0; i++)
                    if (i != j)
                        a [i][k] -= a[i][j] * a[j][k];
                a [j][k] *= -hr;
            }
    }
    for (i = 0; i < m0; i++) {
        for (k = 0; k < m0; k++) hv [p [k]] = a [i][k];
        for (k = 0; k < m0; k++) a [i][k] = hv [k];
    }
}


// Hilfsfunktionen fr dreiparametrige Schtzer

const int n0 = 3;

static double Det (double a [3][3]) {
    double sum = 0.0;
    for (int i = 0; i < 3; i++) {
        sum += a[0][i%3]*a[1][(i+1)%3]*a[2][(i+2)%3];
        sum -= a[0][(i+2)%3]*a[1][(i+1)%3]*a[2][i%3];
    }
    return sum;
}

/*
static int IsPositiveDefinit (double a [3][3]) {
    return a [0][0] > 0.0 &&
	   a [0][0] * a [1][1] - a [0][1] * a [1][0] > 0.0 &&
	   Det (a) > 0.0;
}
*/

static int IsNegativeDefinit (double a [3][3]) {
    return a [0][0] < 0.0 &&
	   a [0][0] * a [1][1] - a [0][1] * a [1][0] > 0.0 &&
	   Det (a) < 0.0;
}

static void Invert (double a [n0][n0]) {
    int i, j, k, r;
    double maxv, hr;
    double hv [n0];
    int p [n0];
    for (j = 0; j < n0; j++) p [j] = j;
    for (j = 0; j < n0; j++) {
        maxv = fabs (a [j][j]);
        r = j;
	for (i = j + 1; i < n0; i++)
            if (fabs (a [i][j]) > maxv) {
                maxv = fabs (a [i][j]);
		r = i;
            }
        if (maxv == 0.0) {
	    puts ("Matrix ist singulr\n");
            abort ();
        }
        if (r > j) {
            for (k = j; k < n0; k++) Swap (a [j][k], a [r][k]);
            Swap (p [j], p [r]);
        }
	hr = 1.0 / a [j][j];
        for (i = 0; i < n0; i++)
            a [i][j] *= hr;
        a [j][j] = hr;
	for (k = 0; k < n0; k++)
            if (k != j) {
                for (i = 0; i < n0; i++)
                    if (i != j)
                        a [i][k] -= a[i][j] * a[j][k];
                a [j][k] *= -hr;
	    }
    }
    for (i = 0; i < n0; i++) {
        for (k = 0; k < n0; k++) hv [p [k]] = a [i][k];
        for (k = 0; k < n0; k++) a [i][k] = hv [k];
    }
}


#ifndef TERMINAL

/**********************************************************************/
/*                  Minimum-L2-Distance-Estimator                     */
/**********************************************************************/

TNormalMDEstimator::TNormalMDEstimator (String Name, int MenuId, int DialogId, BOOL AddList)
 : TNormalEstimator (Name, MenuId, DialogId, AddList) {
}

TEstimator *TNormalMDEstimator::Duplicate () {
    return new TNormalMDEstimator;
}

TSampleType TNormalMDEstimator::GetSampleType () {
    return UNIVARIATE_SAMPLE;
}

double TNormalMDEstimator::ix (int i, int j, int k) {
   return bar[i] + (bar[i+1]-bar[i]) * double(j)/double(k);
}

double TNormalMDEstimator::Integral (double (TNormalMDEstimator::*f) (double, double, double), double s, double m) {
   double r = 0;
   for (int i=0; i<n; i++) {
       int k = 20;
       double t = 0;
       for (int j=0; j<k; j++) {
	   double a = ix(i,2*j,2*k);
	   double b = ix(i,2*j+1,2*k);
	   double c = ix(i,2*j+2,2*k);
	   t += (this->*f)(a,s,m) + 4*(this->*f)(b,s,m) + (this->*f)(c,s,m);
       }
       r += t * (bar[i+1]-bar[i]) / (6*k);
   }
   return r;
}

BOOL TNormalMDEstimator::IsRangeValid (double sigma,double mu) {
    return sigma > 0.0;
}

double TNormalMDEstimator:: dNormdm (double x, double s, double m) {
    return NormDen(x,m,s) * (x-m) / (s * s);
}

double TNormalMDEstimator:: dNormds (double x, double s, double m) {
    return NormDen(x,m,s) * ((((x-m)/s)*((x-m)/s))-1) / s;
}

double TNormalMDEstimator:: d2Normdmdm (double x, double s, double m) {
    return (dNormdm(x,s,m)*(x-m) - NormDen(x,m,s)) / (s*s);
}

double TNormalMDEstimator:: d2Normdsds (double x, double s, double m) {
    return (NormDen(x,m,s)*(1-3*(((x-m)/s)*((x-m)/s)))/s + dNormds(x,s,m)*((((x-m)/s)*((x-m)/s))-1) ) / s;
}

double TNormalMDEstimator:: d2Normdmds (double x, double s, double m) {
    return dNormds(x,s,m)*(x-m)/(s*s) - 2*NormDen(x,m,s)*(x-m)/(s*s*s) ;
}

// Der zu einem Punkt x gehrige Histogramm-Wert
double TNormalMDEstimator:: hy (double x) {
   if (x >= bar[n]) return 0.0;
   if (x < bar[0]) return 0.0; //p->GetFrequency(0);
   int i;
   for (i=0;  x >= bar[i+1]; i++) {};
   if (i>n-1) (i = n-1);
   return HScale * frequency[i];
}

double TNormalMDEstimator:: fDist (double x, double s, double m) {
   return (hy(x) - NormDen (x,m,s)) *(hy(x) - NormDen (x,m,s));
}
double TNormalMDEstimator:: Dist (double s, double m) {
  return Integral (&TNormalMDEstimator::fDist,s,m);
}

double TNormalMDEstimator:: fdDistdm (double x, double s, double m) {
  return 2 * dNormdm(x,s,m) * (NormDen(x,m,s) - hy(x));
}
double TNormalMDEstimator:: dDistdm (double s, double m) {
  return Integral(&TNormalMDEstimator::fdDistdm,s,m);
}

double TNormalMDEstimator:: fdDistds (double x, double s, double m) {
  return 2 * dNormds(x,s,m) * (NormDen(x,m,s) - hy(x));
}
double TNormalMDEstimator:: dDistds (double s, double m) {
  return Integral(&TNormalMDEstimator::fdDistds,s,m);
}

double TNormalMDEstimator:: fd2Distdmdm (double x, double s, double m) {
  return 2 * (d2Normdmdm(x,s,m) * (NormDen(x,m,s)-hy(x)) + pow((dNormdm(x,s,m)),2) );
}
double TNormalMDEstimator:: d2Distdmdm (double s, double m) {
  return Integral(&TNormalMDEstimator::fd2Distdmdm,s,m);
}

double TNormalMDEstimator:: fd2Distdsds (double x, double s, double m) {
  return 2 * (d2Normdsds (x,s,m) * (NormDen(x,m,s)-hy(x)) + pow((dNormds(x,s,m)),2) );
}
double TNormalMDEstimator:: d2Distdsds (double s, double m) {
  return Integral(&TNormalMDEstimator::fd2Distdsds,s,m);
}

double TNormalMDEstimator:: fd2Distdmds (double x, double s, double m) {
  return 2 * (d2Normdmds (x,s,m) * (NormDen(x,m,s)-hy(x)) + dNormdm(x,s,m)*dNormds(x,s,m) );
}
double TNormalMDEstimator:: d2Distdmds (double s, double m) {
  return Integral(&TNormalMDEstimator::fd2Distdmds,s,m);
}

void TNormalMDEstimator::CalcDF (double sigma, double mu, double DF [2][2]) {
    DF [0] [0] = d2Distdsds (sigma, mu);
    DF [0] [1] = d2Distdmds (sigma, mu);
    DF [1] [0] = DF [0] [1];
    DF [1] [1] = d2Distdmdm (sigma, mu);
}

BOOL TNormalMDEstimator::Iter (double &s, double &m) {
    double d, rho = 1.0;
    double DF [2][2];
    CalcDF (s, m, DF);
    double stepsigma, stepmu, sigmas, mus;

    if (!IsPositiveDefinit (DF)) {
        stepsigma = -dDistds (s, m);
        stepmu = -dDistdm (s, m);
        d = sqrt (stepsigma * stepsigma + stepmu * stepmu);
	stepsigma /= d;
        stepmu /= d;
    } else {
	Invert (DF);
	stepsigma = -(DF[0][0] * dDistds(s,m) + DF[0][1] * dDistdm(s,m));
	stepmu    = -(DF[1][0] * dDistds(s,m) + DF[1][1] * dDistdm(s,m));
    }
    double distances = Dist (s, m);
//  ProgressField.SetText (form (50, "Distanz: %g", distances));
    do {
        sigmas = s + rho * stepsigma;
        mus = m + rho * stepmu;
	rho /= 2.0;
    } while ((!IsRangeValid (sigmas, mus) || distances < Dist (sigmas, mus)) &&
               rho * sqrt (stepsigma * stepsigma + stepmu * stepmu) >= 1e-7);
    s = sigmas; m = mus;
    ProgressField.SetText (form (100, "Last step: (%g, %g)", rho * stepsigma, rho * stepmu));
    if (rho * sqrt (stepsigma * stepsigma + stepmu * stepmu) < 1e-7) {
	ProgressField.SetText ("Iteration terminated because step width is smaller than 1e-7\n");
	return 1;
    }
    return 0;
}

BOOL TNormalMDEstimator::ExtractData (TSample *p) {
    TUnivariateSample &Sample = *(TUnivariateSample *) p;
    size = Sample.GetDataSize ();
    n = 20;
    bar = new double [n+1];
    frequency = new int [n];

    bar[0] = Sample[0];
    bar[n] = Sample[size-1];
    barwidth = (bar[n]-bar[0]) / n;

    int i;
    for (i = 1; i <= n; i++) {
	bar[i] = bar[i-1] + barwidth;
	frequency[i-1] = 0;
    }

    int j = 0;
    for (i = 1; i <= n; i++) {
	while (Sample[j] < bar[i]) {
	    frequency[i-1] ++;
	    j++;
	    if (j == size) {i= n+1; break;}
	}
    }

    HScale = 1 / (size * barwidth);
    ProgressField.SetText ("Values returned by initial estimator");
    Parameters[0] = Sample[(int) (floor(0.5*size)-1)];
    Parameters[1] = (Sample[(int) floor(0.75*size)-1] - Sample[(int) floor(0.25*size)-1])/(NormQF(0.75,0,1)-NormQF(0.25,0,1));
    ShowCurrentEstimation ();
    return TRUE;
}

BOOL TGroupedNormalMDEstimator::ExtractData (TSample *q) {
    TGroupedSample *p = (TGroupedSample *) q;
    n = p->GetGroupCount ();
    bar = new double [n+1];
    frequency = new int [n];
    size = 0;
    int i;
    for (i = 0; i < n; i++) {
	 size += (int) p->GetFrequency (i);
	 bar [i] = p->GetPartition(i);
	 frequency[i] = (int) p->GetFrequency(i);
    }
    bar[n] = p->GetPartition(n);
    HScale = 0.0;
    for (i = 0; i < n; i++)
        HScale += p->GetFrequency (i) * (p->GetPartition (i + 1) - p->GetPartition (i));
    if (HScale == 0.0) return FALSE;
    HScale = 1 / HScale;

    // Startschtzung
    ProgressField.SetText ("Values returned by initial estimator");
    Parameters[0] = (p->EmpQF(0.5));
    Parameters[1] = ((p->EmpQF(0.75)) - (p->EmpQF(0.25)))/(NormQF(0.75,0,1)-NormQF(0.25,0,1));
    ShowCurrentEstimation ();
    return TRUE;
}

TEstResult TNormalMDEstimator::Estimate (int k, TSample *p) {
    if (!ExtractData (p)) {
        ProgressField.SetText ("The data set contains only empty cells.");
	return EST_SAMPSIZE;
    }
    BOOL Done = FALSE;
    int i;
    for (i = 0; i <= 50 && !Done; i++) {
	Done = Iter (Parameters [1], Parameters [0]);
	ShowCurrentEstimation ();
    }
    if (i > 50) ProgressField.SetText ("Maximum number of iterations performed");
    delete bar;
    delete frequency;
    return EST_OK;
}

TGroupedNormalMDEstimator::TGroupedNormalMDEstimator (BOOL AddList)
 : TNormalMDEstimator ("MDE(Gaussian, Grouped)", IDM_MDEGroupedGauss, IDD_GaussEstimator, AddList) {
}

TSampleType TGroupedNormalMDEstimator::GetSampleType () {
    return GROUPED_SAMPLE;
}

TEstimator *TGroupedNormalMDEstimator::Duplicate () {
    return new TGroupedNormalMDEstimator;
}

/**********************************************************************/
/*               Minimum-Hellinger-Distance-Estimator                 */
/**********************************************************************/


TNormalMHDEstimator::TNormalMHDEstimator (String Name, int MenuId, int DialogId, BOOL AddList)
 : TNormalEstimator (Name, MenuId, DialogId, AddList) {
}

TEstimator *TNormalMHDEstimator::Duplicate () {
    return new TNormalMHDEstimator;
}

TSampleType TNormalMHDEstimator::GetSampleType () {
    return UNIVARIATE_SAMPLE;
}

double TNormalMHDEstimator::ix (int i, int j, int k) {
   return bar[i] + (bar[i+1]-bar[i]) * double(j)/double(k);
}

double TNormalMHDEstimator::Integral (double (TNormalMHDEstimator::*f) (double, double, double), int i, double s, double m) {
   int k = 20;
   double t = 0;
   for (int j=0; j<k; j++) {
	double a = ix(i,2*j,2*k);
        double b = ix(i,2*j+1,2*k);
        double c = ix(i,2*j+2,2*k);
	t += (this->*f)(a,s,m) + 4*(this->*f)(b,s,m) + (this->*f)(c,s,m);
   }
   return t * (bar[i+1]-bar[i])/(6*k);
 }

BOOL TNormalMHDEstimator::IsRangeValid (double sigma,double mu) {
    return sigma > 0.0;
}

double TNormalMHDEstimator:: dNormdm (double x, double s, double m) {
    return NormDen(x,m,s) * (x-m) / (s * s);
}

double TNormalMHDEstimator:: dNormds (double x, double s, double m) {
    return NormDen(x,m,s) * ((((x-m)/s)*((x-m)/s))-1) / s;
}

//double TNormalMHDEstimator:: fg (double x, double s, double m) {
//   return NormDen(x,m,s);
//}

double TNormalMHDEstimator:: fDist (int j, double s, double m) {
   return NormDFPol(bar[j+1],m,s) - NormDFPol(bar[j],m,s);
}
double TNormalMHDEstimator:: Dist (double s, double m) {
    double r = 0.0;
    for (int j = 0; j < n; j++)
	if (fDist(j,s,m)>EPSILON) //(frequency[j]>0)
	r += sqrt(frequency[j] * fDist(j,s,m) );
    return r;
}

double TNormalMHDEstimator:: f1 (double s, double m) {
    double r = 0.0;
    for (int j = 0; j < n; j++)
	if (fDist(j,s,m)>EPSILON)  //(frequency[j]>0)
	r += sqrt(frequency[j] / fDist(j,s,m))*Integral(&TNormalMHDEstimator::dNormds,j,s,m);
    return r;
}

double TNormalMHDEstimator:: f2 (double s, double m) {
    double r = 0.0;
    for (int j = 0; j < n; j++)
	if (fDist(j,s,m)>EPSILON) //(frequency[j]>0)
	r += sqrt(frequency[j] / fDist(j,s,m))*Integral(&TNormalMHDEstimator::dNormdm,j,s,m);
    return r;
}

double TNormalMHDEstimator:: df1ds (double s, double m) {
  return (f1(s+EPSILON,m)- tf1)/EPSILON;
}
double TNormalMHDEstimator:: df1dm (double s, double m) {
  return (f1(s,m+EPSILON)- tf1)/EPSILON;
}
double TNormalMHDEstimator:: df2dm (double s, double m) {
  return (f2(s,m+EPSILON)- tf2)/EPSILON;
}

void TNormalMHDEstimator::CalcDF (double sigma, double mu, double DF [2][2]) {
    DF [0] [0] = df1ds(sigma, mu);
    DF [0] [1] = df1dm(sigma, mu);
    DF [1] [0] = DF [0] [1];
    DF [1] [1] = df2dm(sigma, mu);
}

BOOL TNormalMHDEstimator::Iter (double &s, double &m) {
    double d, rho = 1.0;
    double DF [2][2];
    tf1 = f1 (s, m);
    tf2 = f2 (s, m);
    CalcDF (s, m, DF);
    double stepsigma, stepmu, sigmas, mus;

    if (!IsNegativeDefinit (DF)) {
        stepsigma = tf1;
        stepmu = tf2;
	d = sqrt (stepsigma * stepsigma + stepmu * stepmu);
        stepsigma /= d;
        stepmu /= d;
    } else {
	Invert (DF);
	stepsigma = -(DF[0][0] * tf1 + DF[0][1] * tf2);
	stepmu    = -(DF[1][0] * tf1 + DF[1][1] * tf2);
    }
    double distances = Dist (s, m);
    do {
        sigmas = s + rho * stepsigma;
        mus = m + rho * stepmu;
        rho /= 2.0;
    } while ((!IsRangeValid (sigmas, mus) || distances > Dist (sigmas, mus)) &&
	       rho * sqrt (stepsigma * stepsigma + stepmu * stepmu) >= 1e-7);
    s = sigmas; m = mus;
    ProgressField.SetText (form (100, "Last step: (%g, %g)", rho * stepsigma, rho * stepmu));
    if (rho * sqrt (stepsigma * stepsigma + stepmu * stepmu) < 1e-7) {
	ProgressField.SetText ("Iteration terminated because step width is smaller than 1e-7\n");
        return 1;
    }
    return 0;
}

BOOL TNormalMHDEstimator::ExtractData (TSample *p) {
    TUnivariateSample &Sample = *(TUnivariateSample *) p;
    size = Sample.GetDataSize ();

    n = 40;
    bar = new double [n+1];
    frequency = new int [n];
    bar[0] = Sample[0];
    bar[n] = Sample[size-1];
    barwidth = (bar[n]-bar[0]) / n;

    int i;
    for (i = 1; i <= n; i++) {
	bar[i] = bar[i-1] + barwidth;
	frequency[i-1] = 0;
    }

    int j = 0;
    for (i = 1; i <= n; i++) {
	while (Sample[j] < bar[i]) {
	    frequency[i-1] ++;
	    j++;
	    if (j == size) {i= n+1; break;}
	}
    }
    ProgressField.SetText ("Values returned by initial estimator");
    Parameters[0] = Sample[(int) floor(0.5*size)-1];
    Parameters[1] = (Sample[(int) floor(0.75*size)-1] - Sample[(int) floor(0.25*size)-1])/(NormQF(0.75,0,1)-NormQF(0.25,0,1));
    ShowCurrentEstimation ();
    return TRUE;
}

BOOL TGroupedNormalMHDEstimator::ExtractData (TSample *q) {
    TGroupedSample *p = (TGroupedSample *) q;
    n = p->GetGroupCount ();
    bar = new double [n+1];
    frequency = new int [n];
    size = 0;
    for (int i = 0; i < n; i++) {
	 size += (int) p->GetFrequency (i);
	 bar [i] = p->GetPartition(i);
	 frequency[i] = (int) p->GetFrequency(i);
    }
    if (!size) return FALSE;
    bar[n] = p->GetPartition(n);
    ProgressField.SetText ("Values returned by initial estimator");
    Parameters[0] = (p->EmpQF(0.5));
    Parameters[1] = ((p->EmpQF(0.75)) - (p->EmpQF(0.25)))/(NormQF(0.75,0,1)-NormQF(0.25,0,1));
    ShowCurrentEstimation ();
    return TRUE;
}

TEstResult TNormalMHDEstimator::Estimate (int k, TSample *p) {
    if (!ExtractData (p)) {
        ProgressField.SetText ("The data set contains only empty cells.");
	return EST_SAMPSIZE;
    }
    BOOL Done = FALSE;
    int i;
    for (i = 0; i <= 50 && !Done; i++) {
	Done = Iter (Parameters [1], Parameters [0]);
	ShowCurrentEstimation ();
    }
    if (i > 50) ProgressField.SetText ("Maximum number of iterations performed");
    delete bar;
    delete frequency;
    return EST_OK;
}

TGroupedNormalMHDEstimator::TGroupedNormalMHDEstimator (BOOL AddList)
 : TNormalMHDEstimator ("MHDE(Gaussian, Grouped)", IDM_MHDEGroupedGauss, IDD_GaussEstimator, AddList) {
}

TSampleType TGroupedNormalMHDEstimator::GetSampleType () {
    return GROUPED_SAMPLE;
}

TEstimator *TGroupedNormalMHDEstimator::Duplicate () {
    return new TGroupedNormalMHDEstimator;
}

#endif

/***********************************************************************
 *                  Minimum Hellinger Distance Estimator fr EVDs      *
 ***********************************************************************/

TEstMode TEVDMDEstimator::GetEstimatorMode () {
    return FULL_BETA_MODE;
}

#ifdef TERMINAL
TEVDMDEstimator::TEVDMDEstimator (String Name)
 : TEVDEstimator (Name) {
#else
TEVDMDEstimator::TEVDMDEstimator (String Name, int id1, int id2, BOOL AddList)
 : TEVDEstimator (Name, id1, id2, AddList) {
#endif
    Data.SetMode (BETA_MODE);
}

TEstimator *TEVDMDEstimator::Duplicate () {
    TEVDMDEstimator *p = new TEVDMDEstimator;
    p->Data = Data;
    return p;
}

TSampleType TEVDMDEstimator::GetSampleType () {
    return UNIVARIATE_SAMPLE;
}

double TEVDMDEstimator::dFdb (double x, double b, double s, double m) {
  if (b*(x-m)/s <= -1.0+EPSILON) return 0.0;
  return - vonMisesEXTDF(x,b,s,m) * pow(1+b*(x-m)/s,-1/b) * (log(1+b*(x-m)/s)/b - (x-m)/(s*(1+b*(x-m)/s)) )  /b ;
}                                                                          // -                          // /s

double TEVDMDEstimator::dFds (double x, double b, double s, double m) {
  if (b*(x-m)/s <= -1.0+EPSILON) return 0.0;
  return - vonMisesEXTDF(x,b,s,m) * pow(1+b*(x-m)/s,-(1+1/b)) * (x-m) / (s*s);
}

double TEVDMDEstimator::dFdm (double x, double b, double s, double m) {
  if (b*(x-m)/s <= -1.0+EPSILON) return 0.0;
  return - vonMisesEXTDF(x,b,s,m) * pow(1+b*(x-m)/s,-(1+1/b)) / s;

}

double TEVDMDEstimator:: EVDp (int j, double b, double s, double m) {
  double r = vonMisesEXTDF(bar[j+1],b,s,m)-vonMisesEXTDF(bar[j],b,s,m);
  return r;
}
double TEVDMDEstimator:: Dist (double b, double s, double m) {
    double r = 0.0;
    for (int j = 0; j < n-1; j++)
	if (frequency[j]>0)
	  r += sqrt( frequency[j] * EVDp(j,b,s,m) );
    return r;
}

double TEVDMDEstimator:: dpdb (int j, double b, double s, double m) {
  return dFdb (bar[j+1],b,s,m) - dFdb (bar[j],b,s,m) ;
}
double TEVDMDEstimator:: f1 (double b, double s, double m) {
    double r = 0.0;
    for (int j = 0; j < n-1; j++)
	if (EVDp(j,b,s,m) >0 && frequency[j]>0)
	r += sqrt(frequency[j] / EVDp(j,b,s,m)) * dpdb(j,b,s,m);
    return r;
}

double TEVDMDEstimator:: dpds (int j, double b, double s, double m) {
  return dFds (bar[j+1],b,s,m) - dFds (bar[j],b,s,m) ;
}
double TEVDMDEstimator:: f2 (double b, double s, double m) {
    double r = 0.0;
    for (int j = 0; j < n-1; j++)
	if (EVDp(j,b,s,m) >0 && frequency[j]>0)
	r += sqrt(frequency[j] / EVDp(j,b,s,m)) * dpds(j,b,s,m);
    return r;
}

double TEVDMDEstimator:: dpdm (int j, double b, double s, double m) {
  return dFdm (bar[j+1],b,s,m) - dFdm (bar[j],b,s,m) ;
}
double TEVDMDEstimator:: f3 (double b, double s, double m) {
    double r = 0.0;
    for (int j = 0; j < n-1; j++)
	if (EVDp(j,b,s,m) >0 && frequency[j]>0)
	r += sqrt(frequency[j] / EVDp(j,b,s,m)) * dpdm(j,b,s,m);
    return r;
}

double TEVDMDEstimator::df1db (double b, double s, double m) {
   return (f1 (b + EPSILON, s, m) - tf1) / EPSILON;
}

double TEVDMDEstimator::df1ds (double b, double s, double m) {
   return (f1 (b, s + EPSILON, m) - tf1) / EPSILON;
}

double TEVDMDEstimator::df1dm (double b, double s, double m) {
   return (f1 (b, s, m + EPSILON) - tf1) / EPSILON;
}

//double TEVDMDEstimator::df2db (double b, double s, double m) {
//   return (f2 (b + EPSILON, s, m) - tf2) / EPSILON;
//}

double TEVDMDEstimator::df2ds (double b, double s, double m) {
   return (f2 (b, s + EPSILON, m) - tf2) / EPSILON;
}

double TEVDMDEstimator::df2dm (double b, double s, double m) {
   return (f2 (b, s, m + EPSILON) - tf2) / EPSILON;
}

//double TEVDMDEstimator::df3db (double b, double s, double m) {
//   return (f3 (b + EPSILON, s, m) - tf3) / EPSILON;
//}

//double TEVDMDEstimator::df3ds (double b, double s, double m) {
//   return (f3 (b, s + EPSILON, m) - tf3) / EPSILON;
//}

double TEVDMDEstimator::df3dm (double b, double s, double m) {
   return (f3 (b, s, m + EPSILON) - tf3) / EPSILON;
}

BOOL TEVDMDEstimator::IsRangeValid (double beta, double sigma, double mu) {
    return beta * (bar [0] - mu) / sigma > -1.0 &&   
           beta * (bar [n] - mu) / sigma > -1.0 &&
           sigma > 0.0;
}

void TEVDMDEstimator::CalcDF (double beta, double sigma, double mu, double a [3][3]) {
    a [0][0] = df1db (beta, sigma, mu);
    a [0][1] = df1ds (beta, sigma, mu);
    a [0][2] = df1dm (beta, sigma, mu);
    a [1][0] = a [0][1];
    a [1][1] = df2ds (beta, sigma, mu);
    a [1][2] = df2dm (beta, sigma, mu);
    a [2][0] = a [0][2];
    a [2][1] = a [1][2];
    a [2][2] = df3dm (beta, sigma, mu);
}

BOOL TEVDMDEstimator::Iter (double &b, double &s, double &m) {
    BOOL AlgUsed;
    double d, rho = 1.0;
    double DF [3][3];
    tf1 = f1 (b, s, m);     //
    tf2 = f2 (b, s, m);     //    Zur Verbesserung der Laufzeit
    tf3 = f3 (b, s, m);     //
    CalcDF (b, s, m, DF);
    double stepbeta, stepsigma, stepmu, betas, sigmas, mus;
    if (!IsNegativeDefinit (DF)) {
	AlgUsed = FALSE;
	stepbeta = tf1;
	stepsigma = tf2;
	stepmu = tf3;
	d = sqrt (stepbeta * stepbeta + stepsigma * stepsigma + stepmu * stepmu);
	stepbeta /= d;
	stepsigma /= d;
	stepmu /= d;
    } else {
	AlgUsed = TRUE;
	Invert (DF);
	stepbeta  = -(DF[0][0] * tf1 + DF[0][1] * tf2 + DF[0][2] * tf3),
	stepsigma = -(DF[1][0] * tf1 + DF[1][1] * tf2 + DF[1][2] * tf3),
	stepmu    = -(DF[2][0] * tf1 + DF[2][1] * tf2 + DF[2][2] * tf3);
    }
    double distances = Dist(b,s,m);
    do {
	betas = b + rho * stepbeta;
	sigmas = s + rho * stepsigma;
	mus = m + rho * stepmu;
	rho /= 2.0;
    } while ((!IsRangeValid (betas, sigmas, mus) || (distances > Dist(betas,sigmas,mus))) &&
	       rho * sqrt (stepbeta * stepbeta + stepsigma * stepsigma + stepmu * stepmu) >= 1e-7);
    if (!IsRangeValid (betas, sigmas, mus)) throw EST_FAILURE;
    b = betas; s = sigmas; m = mus;
#ifndef TERMINAL
    ProgressField.SetText (form (100, "Last step: (%g, %g, %g), %s\n", 2*rho * stepbeta, 2*rho * stepsigma, 2*rho * stepmu, AlgUsed ? "Newton" : "Gradient"));
#endif
    if (rho * sqrt (stepbeta * stepbeta + stepsigma * stepsigma + stepmu * stepmu) < 1e-7) {
#ifndef TERMINAL
	ProgressField.SetText ("Iteration terminated because step width is smaller than 1e-7\n");
#endif
	return 1;
    }
    return 0;
}

void TEVDMDEstimator::LeastSquaresEstimate () {
    double zaehler, nenner;
    nenner = vonMisesEXTQF (0.75, RawParameters [0], 1.0, 0.0) - vonMisesEXTQF (0.25, RawParameters [0], 1.0, 0.0);
    zaehler = p->SampleQf (0.75) - p->SampleQf(0.25);
    if (nenner == 0.0) RawParameters [2] = DBL_MAX;
    else RawParameters[2] = zaehler / nenner;
    RawParameters[1] = p->SampleQf(0.25) - RawParameters[2] * vonMisesEXTQF (0.25, RawParameters [0], 1.0, 0.0);
   }

void TEVDMDEstimator::InitEstimator (double &b, double &s, double &m) {
    double q1 = 0.1,
	   q2 = 0.464,
	   q3 = 0.774,
	   q4 = 0.918;

    double zaehler1 = p->SampleQf(q3) - p->SampleQf(q2),
	   nenner1  = p->SampleQf(q2) - p->SampleQf(q1),
	   zaehler2 = p->SampleQf(q4) - p->SampleQf(q3),
	   nenner2  = p->SampleQf(q3) - p->SampleQf(q2);

    if (nenner1 == 0.0 || nenner2 == 0.0) RawParameters [0] = MYMAXDOUBLE;
    else RawParameters [0] = (0.2 * log (zaehler1 / nenner1) + 0.8 * log (zaehler2 / nenner2)) / log (3.0);
    LeastSquaresEstimate ();
}

BOOL TEVDMDEstimator::ExtractData (TSample *p) {
    TUnivariateSample &Sample = *(TUnivariateSample *) p;
    size = Sample.GetDataSize ();
    n = 20;
    bar = new double [n+1];
    frequency = new int [n];

    bar[0] = Sample[0];
    bar[n] = Sample[size-1];
    barwidth = (bar[n]-bar[0]) / n;

    int i;
    for (i = 1; i <= n; i++) {
	bar[i] = bar[i-1] + barwidth;
	frequency[i-1] = 0;
    }
    int j = 0;
    for (i = 1; i <= n; i++) {
	while (Sample[j] < bar[i]) {
	    frequency[i-1] ++;
	    j++;
	    if (j == size) {i= n+1; break;}
	}
    }
    return TRUE;
}

BOOL TGroupedEVDMDEstimator::ExtractData (TSample *q) {
    TGroupedSample *p = (TGroupedSample *) q;
    n = p->GetGroupCount ();
    bar = new double [n+1];
    frequency = new int [n];
    size = 0;
    for (int i = 0; i < n; i++) {
	 size += (int) p->GetFrequency (i);
	 bar [i] = p->GetPartition (i);
	 frequency [i] = (int) p->GetFrequency (i);
    }
    bar [n] = p->GetPartition (n);
    return !!size;
}


TGroupedEVDMDEstimator::TGroupedEVDMDEstimator (BOOL AddList)
#ifdef TERMINAL
 : TEVDMDEstimator ("MDE(EV, Grouped)") {
#else
 : TEVDMDEstimator ("MDE(EV, Grouped)", IDM_GroupedMDEEVD, IDD_EVDEstimator, AddList) {
#endif
}

TSampleType TGroupedEVDMDEstimator::GetSampleType () {
    return GROUPED_SAMPLE;
}

TEstimator *TGroupedEVDMDEstimator::Duplicate () {
    return new TGroupedEVDMDEstimator;
}

TEstResult TEVDMDEstimator::Estimate (int k, TSample *q) {
    p=q;
    if (!ExtractData (p)) {
#ifndef TERMINAL
        ProgressField.SetText ("The data set contains only empty cells.");
#endif
	return EST_SAMPSIZE;
    }

#ifndef TERMINAL
    ProgressField.SetText ("Values returned by initial estimator");
#endif
    InitEstimator (RawParameters [0], RawParameters [2], RawParameters [1]);
    if (!IsRangeValid (RawParameters [0], RawParameters [2], RawParameters [1]))
#ifndef TERMINAL
	ProgressField.SetText ("Fehler!");
#else
	return EST_FAILURE;
#endif

    Parameters [0] = RawParameters [0];
    Parameters [2] = RawParameters [2];
    Parameters [1] = RawParameters [1];
#ifndef TERMINAL
    ShowCurrentEstimation ();
#endif
    BOOL Done = FALSE;
    TEstResult res = EST_OK;
    int i;
    for (i = 0; i <= 50 && !Done; i++) {
	try {
	    Done = Iter (RawParameters [0], RawParameters [2], RawParameters [1]);
        }
	catch (TEstResult t) {
	    res = t;
	    Done = TRUE;
 	}
	Parameters [0] = RawParameters [0];
	Parameters [2] = RawParameters [2];
	Parameters [1] = RawParameters [1];
#ifndef TERMINAL
	ShowCurrentEstimation ();
#endif
    }
#ifndef TERMINAL
    if (i > 50) ProgressField.SetText ("Maximum number of iterations performed");
#endif
    TransformModels (k, NULL);

    delete bar;
    delete frequency;

    return res;
}

TEstimatorData *TEVDMDEstimator::GetParameter () {
    return &Data;
}

