/*  jensest.hpp
*/

#ifndef JENSEST_HPP
#define JENSEST_HPP

#ifndef TERMINAL
  #include "resource.h"
#endif

class TEVDMDEstimator : public TEVDEstimator {
    public:
#ifdef TERMINAL
	TEVDMDEstimator (String = "MDE(EV)");
#else
	TEVDMDEstimator (String = "MDE(EV)", int = IDM_MDEEV, int = IDD_EVDEstimator, BOOL = FALSE);
#endif
	virtual TEstimator *Duplicate ();
	TSampleType GetSampleType ();
	virtual TEstResult Estimate (int k, TSample *);
	virtual TEstimatorData *GetParameter ();

    protected:
	virtual TEstMode GetEstimatorMode ();
	TEstimatorData Data;
	TUnivariateSample *Sample;
	int n, size;
	double barwidth;
	double *bar;
	int *frequency;
	virtual BOOL ExtractData (TSample *);
	double tf1,tf2,tf3;
	void LeastSquaresEstimate ();
	double dFdb (double x, double b, double s, double m);
	double dFds (double x, double b, double s, double m);
        double dFdm (double x, double b, double s, double m);
        double EVDp (int j, double b, double s, double m);
        double Dist (double b, double s, double m);
        double dpdb (int j, double b, double s, double m);
        double f1 (double b, double s, double m);
        double dpds (int j, double b, double s, double m);
        double f2 (double b, double s, double m);
        double dpdm (int j, double b, double s, double m);
        double f3 (double b, double s, double m);
        double df1db (double b, double s, double m);
        double df1ds (double b, double s, double m);
        double df1dm (double b, double s, double m);
	double df2db (double b, double s, double m);
	double df2ds (double b, double s, double m);
	double df2dm (double b, double s, double m);
	double df3db (double b, double s, double m);
	double df3ds (double b, double s, double m);
	double df3dm (double b, double s, double m);
	BOOL IsRangeValid (double beta, double sigma, double mu);
	void CalcDF (double beta, double sigma, double mu, double a [3][3]);     // Hessian matrix
	BOOL Iter (double &b, double &s, double &m);
	void InitEstimator (double &b, double &s, double &m);
	void InitEstimator2 (double &b, double &s, double &m);
};


class TGroupedEVDMDEstimator : public TEVDMDEstimator {
    public:
	TGroupedEVDMDEstimator (BOOL AddList = FALSE);
	TSampleType GetSampleType();
	virtual TEstimator *Duplicate ();
    protected:
	virtual BOOL ExtractData (TSample *);
};

#ifndef TERMINAL

// typedef double NormMDE_ft(double x, double my, double sigma);

class TNormalMDEstimator : public TNormalEstimator {
    public:
	TNormalMDEstimator (String = "MDE(Gaussian)", int = IDM_MDEGauss, int = IDD_GaussEstimator, BOOL = FALSE);
	TEstimator *Duplicate ();
	virtual TEstResult Estimate (int k, TSample *);
	TSampleType GetSampleType();
    protected:
	TUnivariateSample *Sample;
	int n, size;
	double barwidth;
	double *bar;
	int *frequency;
	double HScale;    // Skalierung fr Histogramm
	virtual BOOL ExtractData (TSample *);
	double ix (int i,int j,int k);
	double Integral (double (TNormalMDEstimator::*f) (double, double, double), double s, double m);
	BOOL IsRangeValid (double sigma, double mu);
	double dNormdm (double x,double s, double m);
	double dNormds (double x,double s, double m);
	double d2Normdmdm (double x,double s, double m);
	double d2Normdsds (double x,double s, double m);
	double d2Normdmds (double x,double s, double m);
	double hy (double x);
	double fDist (double x,double s, double m);
	double Dist (double s, double m);
	double fdDistdm (double x, double s, double m);
	double dDistdm (double s, double m);
	double fdDistds (double x, double s, double m);
	double dDistds (double s, double m);
        double fd2Distdmdm (double x, double s, double m);
        double d2Distdmdm (double s, double m);
        double fd2Distdsds (double x, double s, double m);
        double d2Distdsds (double s, double m);
        double fd2Distdmds (double x, double s, double m);
	double d2Distdmds (double s, double m);
        void   CalcDF (double sigma, double mu, double Hess [2][2]);
        BOOL   Iter (double &s, double &m);

};

class TGroupedNormalMDEstimator : public TNormalMDEstimator {
    public:
	TGroupedNormalMDEstimator (BOOL AddList = FALSE);
	TSampleType GetSampleType();
	virtual TEstimator *Duplicate ();
    protected:
	virtual BOOL ExtractData (TSample *);
};


class TNormalMHDEstimator : public TNormalEstimator {
    public:
	TNormalMHDEstimator (String = "MHDE(Gaussian)", int = IDM_MHDEGauss, int = IDD_GaussEstimator, BOOL = FALSE);
	TEstimator *Duplicate ();
	virtual TEstResult Estimate (int k, TSample *);
	TSampleType GetSampleType();
    protected:
	TUnivariateSample *Sample;
	int n, size;
	double barwidth;
	double *bar;
	int *frequency;
	double tf1, tf2;
	virtual BOOL ExtractData (TSample *);
	double ix (int i,int j,int k);
	double Integral (double (TNormalMHDEstimator::*f) (double, double, double), int j, double s, double m);
        BOOL IsRangeValid (double sigma, double mu);
        double dNormdm (double x,double s, double m);
        double dNormds (double x,double s, double m);
	double fg (double x, double s, double m);
	double fDist (int i,double s, double m);
	double Dist (double s, double m);
	double f1 (double s, double m);
	double f2 (double s, double m);
        double df1ds (double s, double m);
        double df1dm (double s, double m);
        double df2dm (double s, double m);
	void   CalcDF (double sigma, double mu, double Hess [2][2]);
        BOOL   Iter (double &s, double &m);

};

class TGroupedNormalMHDEstimator : public TNormalMHDEstimator {
    public:
	TGroupedNormalMHDEstimator (BOOL AddList = FALSE);
	TSampleType GetSampleType();
	virtual TEstimator *Duplicate ();
    protected:
	virtual BOOL ExtractData (TSample *);
};


#endif

#endif



