/*  list.hpp

    Declaration of base classes for list management.
    Copyright (C) 1998 Michael Thomas.
*/

#ifndef	LIST_HPP
#define	LIST_HPP

#include <stdio.h>
#include <string.h>

#include "boolean.hpp"
#include "string.hpp"

/**********************************************************/
/*  TObject is the base class of all objects that may be  */
/*  stored in lists. Objects registered as observers will */
/*  be notified by the destructor of TObject.             */
/**********************************************************/

class TList;		// forward declaration

class TObject {
    public:
        TObject ();
	virtual	~TObject ();
	virtual	String Name ();
#ifdef _Windows
//	virtual	BOOL	Save (HFILE);
//	virtual	BOOL	Load (HFILE);
#endif
	void AddObserver (TObject *);      
        void RemoveObserver (TObject *);
        virtual void NotifyDelete (TObject *);
    private:
    	TList *Observers;
};

class TList : public TObject {
    friend class TIterator;
    public:
	TList ();
	~TList ();

	void Append (TObject *);
        void Remove (TObject *, BOOL DelContents = FALSE);
	void InsertBeforeActive	(TObject *);

	TObject * GetFirst (),
		* GetLast (),
		* GetNext (),
		* GetPrev (),
		* GetEntry (int),
		* GetActive ();
	void Advance (int);
	void SeekEntry (int);
	String GetActiveName ();

	BOOL	ActiveMarked ();
	void ToggleActive ();
	void ResetMarks	();

	int Entries ();
	void DeleteAll (BOOL	DelContents = FALSE);
	void DeleteMarked (BOOL DelContents = FALSE);
	void DeleteActive (BOOL DelContents = FALSE);
#ifdef _Windows
//	virtual	BOOL	Save (HFILE);
//	virtual	BOOL	Load (HFILE);
#endif
    protected:

	struct Node {
	    TObject *El;
	    Node   *Next, *Prev;
	    BOOL marked;
	    Node () { marked = FALSE; }
	} *Begin, *End,	*ActPtr;

	int entries;
	void DeleteNode	(Node *);
	void InsertBefore (TObject *, Node *);

	virtual	TObject * CreateEl ();
};

/*********************************************************/
/*  Iterator over list members. GetNext returns NULL at  */
/*  the end of the list					 */
/*********************************************************/

class TIterator : public TObject {
    public:
        TIterator (TList *List);
        TObject *GetNext ();
        void Restart ();
        BOOL IsActiveMarked ();
        void ToggleActive ();
    private:
        TList *List;
        TList::Node *ActNode;
};

typedef TObject TListEl;

class TStringEl	: public TListEl {
    public:
	TStringEl (String s);
	String Name ()	     { return buf; }
    private:
	String buf;
};

/*********************************************************/
/*  Sorted list (requires compare function)              */
/*********************************************************/

typedef	int (*TCmpFunc)(TObject *, TObject *);

class TSortedList : public TList {
    public:
	TSortedList (TCmpFunc);
	void Insert (TObject *);
    protected:
	TCmpFunc cmp;
};

/*********************************************************/
/*			Stack				 */
/*********************************************************/

class TStack : public TList {
    public:
	void Push (TObject *p)		{ Append (p); }
	void Pop () 			{ GetLast (); DeleteActive (); }
        TObject *Top ()			{ return GetLast (); }
};

#endif

