/*  slope.cpp

    Implemenation of L-Moment (GP) estimator.

    This file is part of Xtremes for Windows,
    Copyright (C) 1991, 2000 Xtremes Group, Siegen.
*/

#include <math.h>
#include "allest.hpp"
#include "mathfunc.hpp"
#include "stddef.hpp"

#ifndef TERMINAL
#include "resource.h"
#endif

TLMomentGPEstimator::TLMomentGPEstimator (BOOL AddMenu)
#ifdef TERMINAL
  : TUnivariateGPDEstimator ("L-Moment(GP)") {
#else
  : TUnivariateGPDEstimator ("L-Moment(GP)", IDM_LMomentGPEstimator, IDD_Estimator, AddMenu) {
#endif
    Data.SetMode (BETA_MODE);
}

TEstMode TLMomentGPEstimator::GetEstimatorMode () {
    return FULL_BETA_MODE;
}

TEstResult TLMomentGPEstimator::Estimate (int k, TSample *q) {
    TUnivariateSample *p = (TUnivariateSample *) q;
    int n = p->GetDataSize ();
    if (k < 3) return EST_EXT;
    if (k > n) return EST_PAR;

    double lambda1 = 0.0,
           lambda2 = 0.0,
           mu = (*p)[n - k];
    int i;
    for (i = n - k + 1; i < n; i++)
        lambda1 += ((*p)[i] - mu);
    lambda1 /= (k - 1);
    for (i = n - k + 2; i < n; i++)
    	lambda2 += ((double) (i - (n - k + 1))) / ((double) (k - 2)) * ((*p)[i] - mu);
    lambda2 = lambda2 * 2.0 / (k - 1) - lambda1;

    double gamma = 2.0 - lambda1 / lambda2,
           sigma = (1.0 - gamma) * lambda1;

    if (gamma == 0.0) return EST_FAILURE;

    RawParameters [0] = gamma;
    RawParameters [2] = sigma * pow ((double) k / (double) n, gamma);
    RawParameters [1] = mu - (sigma - RawParameters [2]) / gamma;

    // transform to tail

    

    TransformModels (k, p);
    return EST_OK;
}

TEstimatorData *TLMomentGPEstimator::GetParameter () {
    return &Data;
}

TEstimator *TLMomentGPEstimator::Duplicate () {
    TLMomentGPEstimator *p = new TLMomentGPEstimator;
    p->Data = Data;
    return p;
}

int TLMomentGPEstimator::GetMinK () {
    return 3;
}

void TLMomentGPEstimator::CalcEstimator (Array<double> &alpha, int &kmax, TSample *q) {
    TUnivariateSample &p = *(TUnivariateSample *) q;
    int k = GetMinK ();
    for (; k <= p.GetDataSize (); k++) {
	Estimate (k, q);
	alpha [k] = RawParameters [0];
    }
    kmax = p.GetDataSize ();
}
