/*  moment.cpp

    Implemenation of moment estimator.

    This file is part of Xtremes for Windows,
    Copyright (C) 1994 Xtremes Group, Siegen.
*/

// b,m,s

#include <math.h>
#include "allest.hpp"
#include "mathfunc.hpp"

#ifndef TERMINAL
#include "resource.h"
#endif

TMomentEstimator::TMomentEstimator (BOOL AddMenu)
#ifdef TERMINAL
  : TUnivariateGPDEstimator ("Moment(GP)") {
#else
  : TUnivariateGPDEstimator ("Moment(GP)", IDM_MomentEstimator, IDD_Estimator, AddMenu) {
#endif
    Data.SetMode (BETA_MODE);
}

double TMomentEstimator::BetaEst (TUnivariateSample &s, int k) {
    int n = s.GetDataSize ();
    double M1 = 0.0, M2 = 0.0, l, nk = (s [n - k]);
    for (int i = 1; i < k; i++) {
	l = log (s [n-i]/nk);
	M1 += l;
	M2 += l * l;
    }
    M1 /= (k-1); M2 /= (k-1);
    double g = M2 == 0.0 ?
		 0 :
		 M1 + 1.0 - 0.5 / (1.0 - M1 * M1 / M2);
    return g;
}

TEstMode TMomentEstimator::GetEstimatorMode () {
    return BETA_MODE;
}

TEstResult TMomentEstimator::Estimate (int k, TSample *q) {
    TUnivariateSample *p = (TUnivariateSample *) q;
    int n = p->GetDataSize ();
    if (k < 3) return EST_EXT;
    if (k > n) return EST_PAR;
    if ((*p) [n-k] * (*p) [n-1] <= 0.0) return EST_SIGN;
    RawParameters [0] = BetaEst (*p, k);
    TransformModels (k, p);
    return EST_OK;
}

TEstimatorData *TMomentEstimator::GetParameter () {
    return &Data;
}

TEstimator *TMomentEstimator::Duplicate () {
    TMomentEstimator *p = new TMomentEstimator;
    p->Data = Data;
    return p;
}

int TMomentEstimator::GetMinK () {
    return 3;
}

void TMomentEstimator::CalcEstimator (Array<double> &alpha, int &kmax, TSample *q) {
    TUnivariateSample &p = *(TUnivariateSample *) q;
    int k = GetMinK ();
    BOOL Done = FALSE;
    for (; k <= p.GetDataSize () && !Done; k++)
	if (Estimate (k, q) == EST_SIGN)
	    Done = TRUE;
	else
	    alpha [k] = RawParameters [0];
    kmax = k - 1;
}
