/*  mulbase.cpp

    Multivariate data sets.

    This file belongs to sample.cpp.
*/

#include <assert.h>
#include <math.h>
#include <stdlib.h>
#include <float.h>
#include <limits.h>

#include "sample.hpp"
#include "samplist.hpp"
#include "stddef.hpp"
#include "allest.hpp"

#ifndef TERMINAL

#include "boxplot.hpp"
#include "display.hpp"
#include "dbldlg.hpp"
#include "edit.hpp"
#include "empiric.hpp"
#include "filedlg.hpp"
#include "hazard.hpp"
#include "stddlg.hpp"
#include "kernel.hpp"
#include "mathfunc.hpp"
#include "global.hpp"
#include "samplwin.hpp"
#include "usrtrafo.hpp"
#include "visual.hpp"
#include "wintypes.hpp"
#include "scat3d.hpp"
#include "scatter.hpp"
#include "fillmiss.hpp"
#include "nyi.hpp"

#pragma hdrstop
#include "resource.h"
#include "privmsg.hpp"

#endif

/********************************************************************/
/*                     Multivariate Data                            */
/********************************************************************/

#ifndef TERMINAL

class TMultivariateListWindow : public TSampleListWindow {
    typedef TSampleListWindow inherited;
    public:
        TMultivariateListWindow (TMDIFrame *, TMultivariateSample *p, int breadth);
    protected:
        virtual int GetSubMenuPos ();
        virtual String GetLine (int i);
};

TMultivariateListWindow::TMultivariateListWindow (TMDIFrame *MDIFrame, TMultivariateSample *p, int breadth)
 : TSampleListWindow (p, MDIFrame, p->GetFileName ()) {
    SetHeight (p->GetDataSize () + 2);
    SetWidth (breadth);
}

int TMultivariateListWindow::GetSubMenuPos () {
    return 4;
}

String TMultivariateListWindow::GetLine (int i) {
    if (i == 0) {
        String Result = "";
        for (int j = 0; j < s->GetDimension (); j++)
	    Result = Result & form (100, "%-15s", (const char *) s->GetColumnName (j));
        return Result;
    }
    if (i == 1) {
        String Result = "";
        for (int j = 0; j < s->GetDimension (); j++)
            Result = Result & "---------------";
        return Result;
    }
    i -= 2;
    if (i >= s->GetDataSize ()) return "";
    String Result = "";
    for (int j = 0; j < s->GetDimension (); j++)
        if ((*(TMultivariateSample *) s)(i, j) == NOTAVAILABLE)
            Result = Result & ".              ";
        else
            Result = Result & form (20, "%-15g", (*(TMultivariateSample *)s) (i, j));
    return Result;
}



class TColumnSelectDialog : public TDialog {
    typedef TDialog inherited;
    public:
        TColumnSelectDialog (TList *InList, TList *OutListList, String fn);
        String ReadBuf ();
    protected:
        virtual void NotifyActionEvent (TDialogMember *m);
    private:
        TList *InList, *OutList;
        TListField fInList, fOutList;
        TWriteFileField fname;
        void TransferMarked (TList *in, TList *out);
};

TColumnSelectDialog::TColumnSelectDialog (TList *InList0, TList *OutList0, String fn)
 : TDialog (IDD_CombineBox),
   fname (IDD_FileName, fn),
   fInList (IDD_List, InList0),
   fOutList (IDD_ListNew, OutList0) {
    *this << fInList << fOutList << fname;
    SetTitel ("Select Columns");
    InList = InList0;
    OutList = OutList0;
}

String TColumnSelectDialog::ReadBuf () {
    return fname.ReadBuf ();
}

void TColumnSelectDialog::NotifyActionEvent (TDialogMember *m) {
    if (m->GetID () == IDD_Right || m->GetID () == IDD_Left) {
        fInList.EvaluateDialog ();
        fOutList.EvaluateDialog ();
        if (m->GetID () == IDD_Right)
            TransferMarked (InList, OutList);
        else
            TransferMarked (OutList, InList);
        fInList.SetList (InList);
        fOutList.SetList (OutList);
    } else inherited::NotifyActionEvent (m);
}

void TColumnSelectDialog::TransferMarked (TList *in, TList *out) {
    TObject *p = in->GetFirst ();
    while (p)
        if (in->ActiveMarked ()) {
            in->DeleteActive ();
            out->Append (p);
            p = in->GetActive ();
        } else
            p = in->GetNext ();
}

#endif

void TMultivariateSample::AllocatePointers () {
    // It is necessary to allocate these pointers in a separate method
    // because otherwise the Linux port of GNU C++ 2.7.2 crashes.
    DataPtr = new (double * [Dimension]);
    ColumnNamePtr = new String [Dimension];
}


TMultivariateSample::TMultivariateSample () {
    DataPtr = NULL;
    ColumnNamePtr = NULL;
    PMLgroupNr = PMLgroup = PMLbeta = PMLmu = PMLsigma = PMLu = PMLd = NULL;
}

TMultivariateSample::~TMultivariateSample () {
    if (DataPtr) {
        for (int i = 0; i < GetDimension (); i++)
            if (DataPtr [i]) delete [] DataPtr [i];
        delete [] DataPtr;
        delete [] ColumnNamePtr;
    }
}


BOOL TMultivariateSample::GetMemory () {
    int i;
    AllocatePointers ();
    for (i = 0; i < Dimension; i++) {
        double *dummy = new double [DataSize];
        DataPtr [i] = dummy;
    }
    return TRUE;
}

int TMultivariateSample::GetDataSize () {
    return DataSize;
}

double TMultivariateSample::operator () (int i, int j) {
    return DataPtr [j][i];
}

int TMultivariateSample::GetDimension () {
    return Dimension;
}

double TMultivariateSample::GetData (int i, int j) {
    return DataPtr [j][i];
}

String TMultivariateSample::GetColumnName (int i) {
    return ColumnNamePtr [i];
}

void TMultivariateSample::SetData (int dim, int size) {
    Dimension = dim;
    DataSize = size;
    GetMemory ();
}

void TMultivariateSample::SetFilename (String fn, String des) {
    filename = fn;
    name = des;
}

void TMultivariateSample::SetData (int dim, String fn, int size) {
    SetData (dim, size);
    filename = fn;
}

void TMultivariateSample::SetColumn (int c, double *Data, String Header) {
    memcpy (DataPtr [c], Data, sizeof (double) * DataSize);
    ColumnNamePtr [c] = Header;
}

void TMultivariateSample::SetColumn (int c, double *Data, int n, String Header) {
    memcpy (DataPtr [c], Data, sizeof (double) * min (n, DataSize));
    for (int i = n; i < DataSize; i++)
        DataPtr [c][i] = NOTAVAILABLE;
    ColumnNamePtr [c] = Header;
}

BOOL TMultivariateSample::Append (double *v) {
    if (DataSize >= AllocatedSize) {
        AllocatedSize += 100;
        if (AllocatedSize > MAXSAMPLESIZE) return FALSE;
        for (int i = 0; i < Dimension; i++) {
            DataPtr [i] = (double *) realloc (DataPtr [i], AllocatedSize * sizeof (double));
            if (!DataPtr [i]) return FALSE;
        }
    }
    for (int i = 0; i < Dimension; i++)
        DataPtr [i][DataSize] = v [i];
    DataSize++;
    return TRUE;
}

BOOL TMultivariateSample::ReadData (TTextFileReader &f) {
    int i;
    int EntrySize; BOOL b = TRUE;
    if (!ReadSampleSize (f, EntrySize)) return FALSE;
    if (f.GetString (TRUE) == String ("dimension")) {
        int c = f.GetNextChar ();
        if (c != ':') f.UngetChar (c);
        Dimension = f.GetInt ();
        if (f.IsError ()) {
            ReadMessage = LOADERRNUM;
            return FALSE;
        }
    } else {
        ReadMessage = "'Dimension:' expected";
        return FALSE;
    }
    AllocatePointers ();
    for (i = 0; i < Dimension; i++)
        DataPtr [i] = NULL;
    for (i = 0; i < Dimension && !f.IsError (); i++)
        ColumnNamePtr [i] = f.GetQuotedString ();
    if (f.IsError ()) {
        ReadMessage = "Column header expected";
        return FALSE;
    }

    for (i = 0; i < Dimension; i++) DataPtr [i] = NULL;
    DataSize = AllocatedSize = 0;
    double *line = new double [Dimension];
    do {
        for (i = 0; i < Dimension && !f.IsError (); i++)
            line [i] = f.GetDoubleNA ();
        if (!f.IsError ()) b = Append (line);
    } while (!f.IsError () && b);
    if (f.GetError () == TTextFileReader::END_OF_FILE) {
        delete [] line;
        return TRUE;
    } else if (f.GetError ())
        ReadMessage = LOADERRNUM;
    else ReadMessage = MAXSAMPLESIZE == ACADEMICSIZE ? ACADEMIC : INSUFFMEM;
    delete [] line;
    return FALSE;
}

BOOL TMultivariateSample::LoadUntypedData (TTextFileReader &f) {
    // find dimension
    int i;
    Dimension = 0;
    while (f.GetLineNumber () == 1 && !f.IsError ()) {
        f.GetDoubleNA ();
        Dimension++;
    }
    f.Reset ();
    // load data
    AllocatePointers ();
    for (i = 0; i < Dimension; i++) {
        ColumnNamePtr [i] = form (20, "Column %d", i + 1);
        DataPtr [i] = NULL;
    }
    DataSize = AllocatedSize = 0;
    double *line = new double [Dimension];
    BOOL b = TRUE;
    do {
        for (i = 0; i < Dimension && !f.IsError (); i++)
            line [i] = f.GetDoubleNA ();
        if (!f.IsError ()) b = Append (line);
    } while (!f.IsError () && b);
    if (f.GetError () == TTextFileReader::END_OF_FILE) {
        delete [] line;
        return TRUE;
    } else if (f.GetError ())
        ReadMessage = LOADERRNUM;
    else ReadMessage = MAXSAMPLESIZE == ACADEMICSIZE ? ACADEMIC : INSUFFMEM;
    delete [] line;
    return FALSE;
}

BOOL TMultivariateSample::WriteData (FILE *f) {
    int i, j;
    fprintf (f, "Samplesize: %d\n", DataSize);
    fprintf (f, "Dimension: %d\n", Dimension);
    for (j = 0; j < Dimension; j++)
	fprintf (f, "%-15s\t", (const char *) ("\"" & ColumnNamePtr [j] & "\""));
    fputc ('\n', f);
    for (i = 0; i < DataSize; i++) {
        for (j = 0; j < Dimension; j++)
            if (DataPtr [j][i] != NOTAVAILABLE)
                fprintf (f, "%-15g\t", DataPtr [j][i]);
            else
                fprintf (f, "%-15c\t", '.');
        fputc ('\n', f);
    }
    return TRUE;
}

String TMultivariateSample::GetHeader () {
    return SampleHeaders [MULTIVARIATE_SAMPLE];
}

TSampleType TMultivariateSample::GetSampleType () {
    return MULTIVARIATE_SAMPLE;
}

#ifndef TERMINAL

class TMultiFunctionalDialog : public TModelessDialog {
    typedef TModelessDialog inherited;
    public:
        TMultiFunctionalDialog (String s);
        void SetCourierFont ();
    protected:
	virtual void NotifyActionEvent (TDialogMember *m);
    private:
        TEditField e;
        String t;
};

TMultiFunctionalDialog::TMultiFunctionalDialog (String s)
 : inherited (IDD_MultivariateFunctionalParameters),
   e (IDD_MultiFunctionalEdit, "", s.Len ()) {
    t = s;
    e.SetBuf ((const char *) t);
    *this << e;
}

void TMultiFunctionalDialog::SetCourierFont () {
    e.SelectCourierFont ();
}

void TMultiFunctionalDialog::NotifyActionEvent (TDialogMember *m) {
    if (m->GetID () == IDOK) CloseDialog ();
    else inherited::NotifyActionEvent (m);
}


void TMultivariateSample::ShowFunctionalParameters (TWindow *parent) {
    int i , j;
    String s;

    s = s & "             ";
    for (i = 0; i < Dimension; i++)
        s = s & form (20, "%10s ", (const char *) ColumnNamePtr [i]);
    s = s & "\r\n-------------";
    for (i = 0; i < Dimension; i++) s = s & "-----------";
    s = s & "\r\n";


    s = s & "Mean       | ";
    for (i = 0; i < Dimension; i++)
        s = s & form (20, "%10g ", mean (DataPtr [i], DataSize));
    s = s & "\r\n";

    s = s & "Variance   | ";
    for (i = 0; i < Dimension; i++)
        s = s & form (20, "%10g ", variance (DataPtr [i], DataSize));
    s = s & "\r\n\r\n";

    s = s & "Covariance   ";
    for (i = 0; i < Dimension; i++)
        s = s & form (20, "%10s ", (const char *) ColumnNamePtr [i]);
    s = s & "\r\n-------------";
    for (i = 0; i < Dimension; i++) s = s & "-----------";
    s = s & "\r\n";
    for (i = 0; i < Dimension; i++) {
        s = s & form (20, "%10s | ", (const char *) ColumnNamePtr [i]);
        for (j = 0; j < Dimension; j++)
            s = s & form (20, "%10g ", covariance (DataPtr [i], DataPtr [j], DataSize));
        s = s & "\r\n";
    }
    TMultiFunctionalDialog *d = new TMultiFunctionalDialog (s);
    d->OpenDialog (parent);
    d->SetCourierFont ();
}

void TMultivariateSample::ScatterPlot2D (TMDIFrame *MDIFrame) {
    TScatterPlotWindow::PerformMultivariateDialog (MDIFrame, this);
}

void TMultivariateSample::ScatterPlot3D (TMDIFrame *MDIFrame) {
    TMultivariateScatterplotWindow::PerformMultivariateDialog (MDIFrame, this);
}

void TMultivariateSample::PlotSampleDepFunction (TWindow *parent) {
    PlotFunction (parent, new TEmpiricalDepFunction (this), DEPFUNC); 
}

void TMultivariateSample::PlotDTail1 (TWindow *parent){
    PlotFunction (parent, new TEmpiricalTailDepDF1 (this), DEPFUNC); 
}

void TMultivariateSample::PlotDTail2 (TWindow *parent){
    PlotFunction (parent, new TEmpiricalTailDepDF2 (this), DEPFUNC); 
}

void TMultivariateSample::PlotDTail3 (TWindow *parent){
    PlotFunction (parent, new TEmpiricalTailDepDF3 (this), DEPFUNC); 
}

void TMultivariateSample::PlotDTail4 (TWindow *parent){
    PlotFunction (parent, new TEmpiricalTailDepDF4 (this), DEPFUNC); 
}

void TMultivariateSample::PlotCTail1 (TWindow *parent){
    PlotFunction (parent, new TEmpiricalTail1DepFunction (this), DEPFUNC); 
}

void TMultivariateSample::PlotCTail2 (TWindow *parent){
    PlotFunction (parent, new TEmpiricalTail2DepFunction (this), DEPFUNC); 
}

void TMultivariateSample::PlotCTail3 (TWindow *parent){
    PlotFunction (parent, new TEmpiricalTail3DepFunction (this), DEPFUNC); 
}

void TMultivariateSample::PlotCTail4 (TWindow *parent){
    PlotFunction (parent, new TEmpiricalTail4DepFunction (this), DEPFUNC); 
}

void TMultivariateSample::PlotCParaC (TWindow *parent){
    PlotFunction (parent, new TEmpiricalTailDepPARA2 (this), DEPFUNC); 
}

void TMultivariateSample::PlotDParaC (TWindow *parent){
    PlotFunction (parent, new TEmpiricalTailDepPARA1(this), DEPFUNC); 
}

void TMultivariateSample::PlotChi (TWindow *parent){
    PlotFunction (parent, new TEmpiricalTailDepChi (this), DEPFUNC); 
}

void TMultivariateSample::PlotChiBarE (TWindow *parent){
    PlotFunction (parent, new TEmpiricalTailDepChibarE (this), DEPFUNC); 
}

void TMultivariateSample::PlotChiBarP (TWindow *parent){
    PlotFunction (parent, new TEmpiricalTailDepChibarP (this), DEPFUNC); 
}

void TMultivariateSample::BoxPlot (TMDIFrame *MDIFrame) {
    int i;
    double xmin = 0, xmax = Dimension + 1.0;

    // check column headers for numbers
    double *pos = new double [Dimension];
    BOOL AllValid = TRUE;
    for (i = 0; AllValid && i < Dimension; i++)
        AllValid = ParseReal (ColumnNamePtr [i], pos [i]) && ColumnNamePtr [i] != String ("");
    if (!AllValid)
        for (i = 0; i < Dimension; i++)
            pos [i] = i + 1;
    else {
    	xmin = MYMAXDOUBLE; xmax = -MYMAXDOUBLE;
        for (i = 0; i < Dimension; i++) {
            xmin = min (xmin, pos [i]);
            xmax = max (xmax, pos [i]);
        }
        xmin -= 1.0; xmax += 1.0;
    }

    // perform plot
    TPlotWindow *w = PlotWindowManager.GetOpenWindow ("Boxplot of " & Name ());
    double ymax = -MYMAXDOUBLE, ymin = MYMAXDOUBLE, ymean,
           *yval = new double [Dimension];
    for (i = 0; i < Dimension; i++) {
        double y0, y1;
        TFunction *f = new TBoxPlot (DataSize, DataPtr [i], pos [i], ColumnNamePtr [i]);
        ((TBoxPlot *) f)->GetBoxRange (y0, y1);
        yval [i] = y1 - y0;
        f->GetYRange (y0, y1);
        ymax = max (y1, ymax); ymin = min (y0, ymin);
        w->InsertFunction (f);
    }

    qsort (yval, Dimension, sizeof (double), realcomp);
    ymean = yval [Dimension / 2];
    delete [] yval;

    double h = (ymax - ymin) / 5.0;
    ymin -= h;
    ymax += h;

    TRectangle r = w->GetCanvas ()->GetOutputRect ();
    double q = (ymax - ymin) / (xmax - xmin),
           qs = ymean * r.x1 / (5.0 * r.x1);
    if (q < qs) {
    	double h = qs * (xmax - xmin),
               ym = (ymax + ymin) / 2.0;
        ymax = ym + h / 2.0;
        ymin = ym - h / 2.0;
    } else if (q > qs) {
        double b = (ymax - ymin) / qs,
               xm = (xmax + xmin) / 2.0;
        xmax = xm + b / 2.0;
        xmin = xm - b / 2.0;
    }
    w->SetFrameMode (TRUE);
    w->SetNewCoordinates (TDblRectangle (xmin, ymin, xmax, ymax), FALSE);
    delete [] pos;
}

void TMultivariateSample::ListData (TMDIFrame *MDIFrame) {
    new TMultivariateListWindow (MDIFrame, this, 15 * Dimension);
}

void TMultivariateSample::UserTransformation (TWindow *parent) {
    TSample *q = new TMultiUserTrafoSample (this);
    if (!q->Create (parent)) delete q;
}

void TMultivariateSample::FillMissing (TWindow *parent) {
    TSample *q = new TMultiFilledSample (this);
    if (!q->Create (parent)) delete q;
}

unsigned TMultivariateSample::EditParameter (TWindow *parent) {
    return IDCANCEL;
}

BOOL TMultivariateSample::CreateData (TWindow *parent) {
    return FALSE;
}

TSample* TMultivariateSample::ConvertToUnivariate (TWindow *parent) {
    TList Fields;
    for (int i = 0; i < GetDimension (); i++)
        Fields.Append (new TStringEl (GetColumnName (i)));
    TListField a (IDD_Index2, &Fields);
    TWriteFileField e (IDD_Name, ModifyName ('u'));
    TDialog d (IDD_ProjectUnivariate);
    d << e << a;
    unsigned ch = d.Handle (parent);
    TUnivariateSample *p = NULL;
    if (ch == IDOK) {
        int col = a.Selection ();
        p = new TUnivariateSample;
        p->SetName ("Univariate " & GetName ());
        TList s;
        s.Append (new TStringEl ("Data set generated by Xtremes applying the Convert to Univariate option to"));
        s.Append (new TStringEl (GetName ()));
        p->SetDescription (&s);
        s.DeleteAll (FALSE);
        double *x = new double [DataSize];
        int Missing = 0;
        for (int i = 0; i < DataSize; i++)
            if (DataPtr [col][i] == NOTAVAILABLE) Missing++;
            else x[i - Missing] = DataPtr [col][i];
        p->SetData (DataSize - Missing, x, e.ReadBuf ());
        delete [] x;
    }
    Fields.DeleteAll (TRUE);
    return p;
}

TSample *TMultivariateSample::ConvertToCopulaScale (TWindow *parent)
{

 		  int n = GetDataSize();
        int k = GetDimension();
        double *x = new double [k*n];
        double *ss = new double [k];
	int i;
        for ( i = 0; i < n; i++)
        {
	int j;
        		for (j = 0; j < k;j++)
            	ss[j]=0;
        		for (j = 0; j < n; j++)
            		{
                  for (int h = 0; h < k;h++)
                  	{
	            		if (DataPtr[h][i]>DataPtr[h][j])
                     	ss[h]++;
                  	}
                  }
            for (int h = 0;h<k;h++)
            	x[h*n+i]=ss[h]/n;
        }
        	     TMultivariateSample *p = new TMultivariateSample();
        p->SetName ("Copula Scale " & GetName ());
        p->SetData(k,n);
        p->GetMemory();
        double * help = new double[n];
        for (i=0;i<k;i++)
        {
			for (int j=0;j<n;j++)
         	help[j]=x[i*n+j];
			p->SetColumn(i, help, n,"copulascale");
        }
        //delete [] help;
        delete [] x; delete [] ss;
return p;
}

BOOL TMultivariateSample::ConvertBivariateBase (TWindow *parent, TBivariateBaseSample *p) {
    TList Fields;
    for (int i = 0; i < GetDimension (); i++)
        Fields.Append (new TStringEl (GetColumnName (i)));
    TListField a (IDD_Index1, &Fields, 0),
               b (IDD_Index2, &Fields, min (1, GetDimension ()));
    TWriteFileField e (IDD_Name, ModifyName ('p'));
    TDialog d (IDD_ProjectMultivariate);
    d << e << a << b;
    unsigned ch = d.Handle (parent);
    if (ch == IDOK) {
        p->SetName ("Time Series" & GetName ());
        TList s;
	s.Append (new TStringEl (form (200, "Data set generated by Xtremes extracting the columns %s and %s from", (const char *) GetColumnName (b.Selection ()), (const char *) GetColumnName (a.Selection ()))));
        s.Append (new TStringEl (GetName ()));
        p->SetDescription (&s);
        s.DeleteAll (FALSE);
        p->SetData (DataSize, DataPtr [a.Selection ()], DataPtr [b.Selection ()], e.ReadBuf ());
    }
    Fields.DeleteAll (TRUE);
    return ch == IDOK;
}

TSample *TMultivariateSample::ConvertToTimeSeries (TWindow *parent) {
    TBivariateBaseSample *p = new TTimeSeriesSample;
    if (ConvertBivariateBase (parent, p)) return p;
    delete p;
    return NULL;
}

TSample *TMultivariateSample::ConvertToGrouped (TWindow *parent) {
    TList Fields;
    for (int i = 0; i < GetDimension (); i++)
	Fields.Append (new TStringEl (GetColumnName (i)));
    TListField a (IDD_Index1, &Fields, 0),
	       b (IDD_Index2, &Fields, min (1, GetDimension ()));
    TWriteFileField e (IDD_Name, ModifyName ('p'));
    TDialog d (IDD_ProjectMultivariate);
    d.SetTitel ("Convert to Grouped Data");
    d << e << a << b;
    unsigned ch = d.Handle (parent);
    TGroupedSample *p = NULL;
    if (ch == IDOK) {
        p = new TGroupedSample;
        p->SetName ("Grouped " & GetName ());
        TList s;
        s.Append (new TStringEl ("Data set generated by Xtremes applying the Convert to Grouped option to"));
        s.Append (new TStringEl (GetName ()));
        p->SetDescription (&s);
        s.DeleteAll (FALSE);
	p->SetData (DataSize, DataPtr [a.Selection ()], DataPtr [b.Selection ()], e.ReadBuf ());
        filename = e.ReadBuf ();
    }
    Fields.DeleteAll (TRUE);
    return p;
}

void TMultivariateSample::SelectComponent (TWindow *parent) {
    TList InList, OutList;
    for (int i = 0; i < GetDimension (); i++) InList.Append (new TStringEl (form (10, "%2d: ", i + 1) & GetColumnName (i)));
    TColumnSelectDialog d (&InList, &OutList, "univ.dat");
    if (d.Handle (parent) == IDOK)
        if (OutList.Entries ()) {
            TMultivariateSample *s = new TMultivariateSample;
            s->SetData (OutList.Entries (), GetDataSize ());
            int col = 0;
            for (TListEl *p = OutList.GetFirst (); p; p = OutList.GetNext ()) {
		const char *t = p->Name ();
                int n = atoi (t);
                s->SetColumn (col++, DataPtr [n - 1], String (t + 4));
            }
            s->SetFileName (d.ReadBuf ());
            s->SaveData (parent);
            SampleList.InsertSample (s);
        } else ErrorBox (parent, "No columns have been selected.");
    InList.DeleteAll (TRUE);
    OutList.DeleteAll (TRUE);
}

void TMultivariateSample::ExtractMaxima (TWindow *parent) {
    TIntField f (IDD_GroupSize, GlobalData.BlockMaximaSize, 1, INT_MAX);
    TWriteFileField e (IDD_Name, ModifyName ('m'));
    TDialog d (IDD_MaximaDialog);
    d << e << f;
    unsigned ch = d.Handle (parent);
    if (ch == IDOK) {
        int GroupLength = f.GetValue (),
            NewDataSize = GetDataSize () / GroupLength;
        if (!NewDataSize) {
            ErrorBox (parent, "No maxima found");
            return;
        }
        GlobalData.BlockMaximaSize = GroupLength;
        double *NewData = new double [NewDataSize];
        TMultivariateSample *s = new TMultivariateSample;
        s->SetData (GetDimension (), NewDataSize);
        for (int dim = 0; dim < GetDimension (); dim++) {
            int i, j = 0, k;
            for (i = 0; i < NewDataSize; i++) {
                double maxv = DataPtr [dim][j]; j++;
                for (k = 1; k < GroupLength; k++) {
                    if (DataPtr [dim][j] > maxv) maxv = DataPtr [dim][j];
                    j++;
                }
                NewData [i] = maxv;
            }
            s->SetColumn (dim, NewData, GetColumnName (dim));
        }
        TList l;
        l.Append (new TStringEl ("Blocks maxima "));
        l.Append (new TStringEl ("Group length:  " & form (20, "%d", GroupLength)));
        l.Append (new TStringEl ("Applied to:    " & GetName ()));
        s->SetName ("Blocks Maxima of " & GetName ());
        s->SetDescription (&l);
        s->SetFileName (e.ReadBuf ());
        s->SaveData (parent);
        SampleList.InsertSample (s);
        l.DeleteAll (FALSE);
        delete [] NewData;
    }
}

void TMultivariateSample::TransformSample (TWindow *parent) {
    TWriteFileField e (IDD_Name, ModifyName ('a'));
    TCalcDoubleField e1 (IDD_Mu, GlobalData.AffineTrafoMu),
                 e2 (IDD_Sigma, GlobalData.AffineTrafoSigma);
    TDialog d (IDD_AffineTrafoSample);
    d << e << e1 << e2;
    unsigned ch = d.Handle (parent);
    if (ch == IDOK) {
        double *NewData = new double [GetDataSize ()],
               a = e1.GetValue (), b = e2.GetValue ();
        GlobalData.AffineTrafoMu = a;
        GlobalData.AffineTrafoSigma = b;
        TMultivariateSample *s = new TMultivariateSample;
        s->SetData (GetDimension (), GetDataSize ());
        for (int dim = 0; dim < GetDimension (); dim++) {
            for (int i = 0; i < GetDataSize (); i++)
                NewData [i] = a + b * DataPtr [dim][i];
            s->SetColumn (dim, NewData, GetColumnName (dim));
        }
        TList l;
        l.Append (new TStringEl (form (150, "Affine transformation x -> %g x + %g applied to", b, a)));
        l.Append (new TStringEl (GetName ()));
        s->SetDescription (&l);
        s->SetFileName (e.ReadBuf ());
        s->SaveData (parent);
        SampleList.InsertSample (s);
        l.DeleteAll (FALSE);
        delete [] NewData;
    }
}


class TTSSumTestDialog : public TDialog{
    typedef TDialog inherited;
    public:
	TTSSumTestDialog (double *x, int n, double *y, int m);
    protected:
	virtual void NotifyActionEvent (TDialogMember *m);
    private:
    	double *x, *y;
        int n, m;
	void PerformTest ();
	TAlterField a;
	TCalcDoubleField fsigma;
	TStaticField fres, fsigmat;
	double sigma;
	int testnum;
};

TTSSumTestDialog::TTSSumTestDialog (double *x0, int n0, double *y0, int m0)
 : a (0, 6, IDD_TSMuSS, IDD_TSMu, IDD_TSTTestSS, IDD_TSTTest, IDD_TSWelchSS, IDD_TSWelch),
   fsigma (IDD_Sigma0, 1.0),
   fres (IDD_pValue),
   fsigmat (IDD_SigmaT),
   TDialog (IDD_BiSumTest) {
    *this << a << fsigma << fres << fsigmat;
    x = x0; y = y0;
    n = n0; m = m0;
}

void TTSSumTestDialog::NotifyActionEvent (TDialogMember *m) {
    int id = m->GetID ();
    if (id == IDOK) {
	EvaluateDialog ();
	if (IsCorrect ()) {
            sigma = fsigma.GetValue ();
	    testnum = a.Selector ();
	    PerformTest ();
	}
    } else if (id == IDD_TSMu || id == IDD_TSMuSS) {
        fsigma.Enable ();
        fsigmat.Enable ();
    } else if (id == IDD_TSTTest || id == IDD_TSWelch || id == IDD_TSTTestSS || id == IDD_TSWelchSS) {
        fsigma.Disable ();
        fsigmat.Disable ();
    } else
	inherited::NotifyActionEvent (m);
}

void TTSSumTestDialog::PerformTest () {
    double pValue = 0.0;
    switch (testnum) {
        case 0: pValue = umpss (x, n, y, m, sigma); break;
        case 1: pValue = ump (x, n, y, m, sigma); break;
        case 2: pValue = ttestss (x, n, y, m); break;
        case 3: pValue = ttest (x, n, y, m); break;
        case 4: pValue = welchtestss (x, n, y, m); break;
        case 5: pValue = welchtest (x, n, y, m); break;
    }
    fres.SetText (form (50, "p-value = %g", pValue));
}

void TMultivariateSample::GaussianTwoSampleTest (TWindow *parent) {
    if (Dimension != 2)
        ErrorBox (parent, "This test can only be applied to bivariate data sets.");
    else {
        int n = 0, m = 0, col1 = 0, col2 = 1, i;
        for (i = 0; i < DataSize; i++) {
            if (DataPtr [col1][i] != NOTAVAILABLE) n++;
            if (DataPtr [col2][i] != NOTAVAILABLE) m++;
        }
        if (n < 2 || m < 2)
            ErrorBox (parent, "Both columns must contain at least 2 values");
        else {
            double *x = new double [n],
                   *y = new double [m];
            n = m = 0;
            for (i = 0; i < DataSize; i++) {
                if (DataPtr [col1][i] != NOTAVAILABLE) x [n++] = DataPtr [col1][i];
                if (DataPtr [col2][i] != NOTAVAILABLE) y [m++] = DataPtr [col2][i];
            }
            TTSSumTestDialog d (x, n, y, m);
            d.Handle (parent);
            delete [] x;
            delete [] y;
        }
    }
}

void TMultivariateSample::WilcoxonTest (TWindow *parent) {
    if (Dimension != 2)
        ErrorBox (parent, "This test can only be applied to bivariate data sets.");
    else {
        TDialog d (IDD_WilcoxonTest);
        TStaticField res (IDD_pValue);
        res.SetText (form (100, "p-value = %g",
	    wilcoxontest (DataPtr [0], DataSize, DataPtr [1], DataSize)));
        d << res;
        d.Handle (parent);
    }
}

#endif

void TMultivariateSample::TruncateSample (String filename, double threshold, int Column) {
    int i, j = 0;
    double **NewData = new double * [GetDimension ()];
    for (i = 0; i < GetDimension (); i++)
        NewData [i] = new double [GetDataSize ()];
    for (i = 0; i < GetDataSize (); i++)
        if (DataPtr [Column][i] >= threshold) {
            for (int k = 0; k < GetDimension (); k++)
                NewData [k][j] = DataPtr [k][i];
            j++;
        }
    TMultivariateSample *s = new TMultivariateSample;
    s->SetData (GetDimension (), j);
    for (i = 0; i < GetDimension (); i++)
        s->SetColumn (i, NewData [i], GetColumnName (i));
    TList l;
    l.Append (new TStringEl (form (150, "Exceedances above %g (%s) from", threshold, (const char *) GetColumnName (Column))));
    l.Append (new TStringEl (GetName ()));
    s->SetDescription (&l);
    s->SetFileName (filename);
    s->SaveData (0);
    SampleList.InsertSample (s);
    l.DeleteAll (FALSE);
    for (i = 0; i < GetDimension (); i++)
        delete [] NewData [i];
    delete [] NewData;
}

void TMultivariateSample::Sort (double *NewData, int *Position, int l, int r) {
    int i = l, j = r;
    double x0 = NewData [(l + r) / 2];
    do {
        while (NewData [i] < x0) i++;
        while (x0 < NewData [j]) j--;
        if (i <= j) {
            Swap (NewData [i], NewData [j]);
            Swap (Position [i], Position [j]);
            i++; j--;
        }
    } while (i <= j);
    if (l < j) Sort (NewData, Position, l, j);
    if (i < r) Sort (NewData, Position, i, r);
}

void TMultivariateSample::OrderSample (String filename, int Column) {
    double *NewData = new double [GetDataSize ()];
    int *Position = new int [GetDataSize ()];
    for (int i = 0; i < GetDataSize (); i++) {
        NewData [i] = DataPtr [Column][i];
        Position [i] = i;
    }
    Sort (NewData, Position, 0, GetDataSize () - 1);
    TMultivariateSample *s = new TMultivariateSample;
    s->SetData (GetDimension (), GetDataSize ());
    for (int dim = 0; dim < GetDimension (); dim++) {
        for (int i = 0; i < GetDataSize (); i++)
            NewData [i] =  DataPtr [dim][Position [i]];
        s->SetColumn (dim, NewData, GetColumnName (dim));
    }
//    TList l;
//    l.Append (new TStringEl ("Concomitant Sample"));
//    l.Append (new TStringEl   (GetName ()));
//    s->SetDescription (&l);
    s->SetFileName (filename);
    s->SaveData (0);
    SampleList.InsertSample (s);
//    l.DeleteAll (FALSE);
    delete [] NewData;
    delete [] Position;
}


// Price Return Transformation

BOOL IsLeapYear (int year) {
    if (year % 400 == 0) return TRUE;
    if (year % 100 == 0) return FALSE;
    if (year % 4 == 0) return TRUE;
    return FALSE;
}

int CalculateDifference (int day, int month, int year) {
    int MonthLength [13] = {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
    int i;
    int sum = 0;
    for (i = 0; i < year; i++)
        sum += (365 + (int) IsLeapYear (i));
    for (i = 1; i < month; i++) {
        sum += MonthLength [i];
        if (i == 2 && IsLeapYear (year)) sum++;
    }
    return sum + day - 1;
}

void NextDay (int d, int m, int y, int &nd, int &nm, int &ny) {
    int MonthLength [13] = {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
    nd = d; nm = m; ny = y;
    nd++;
    if (nd <= MonthLength [nm] || (nd == 29 && nm == 2 && IsLeapYear (ny))) return;
    nd = 1; nm++;
    if (nm <= 12) return;
    nm = 1; ny++;
}

void TMultivariateSample::Entry (int day, int i) {
    assert (0 <= day && day < maxday);
    assert (0 <= i && i < DataSize);
    int j;
    for (j = 0; j < 3; j++)
        x [j][day] = DataPtr [j][i];
    for (j = 3; j < Dimension; j++)
        if (DataPtr [j][i] > 0.0) {
            x [j][day] = log (DataPtr [j][i]) - plog [j];
            plog [j] = log (DataPtr [j][i]);
        }
}

void TMultivariateSample::TradingDays () {
    int i = 0, j = 0, k;
    BOOL zero;
    while (j < maxday) {
        zero = FALSE;
        for (k = 3; k < Dimension; k++)
            if (x [k][j] == -1.0) zero = TRUE;
        if (!zero) {
            if (i != j)
                for (k = 0; k <= Dimension; k++)
                    x [k][i] = x [k][j];
            i++;
        }
        j++;
    }
    maxday = i;
}

void TMultivariateSample::FillGap () {
    int i, j, k, count;
    for (k = 3; k < Dimension; k++)
        for (i = 0; i < maxday; i++)
            if (x [k][i] == -1.0) {
                j = i + 1;
                while (j < maxday -1 && x [k][j] == -1.0) j++;
                for (count = i; count <= j; count++)
                    x [k][count] = x [k][j] / (j - i + 1);
            }
}

void TMultivariateSample::RemoveGap () {
    int i = 0, j = 0, k;
    BOOL zero = FALSE, prevzero;
    while (j < maxday) {
        prevzero = zero;
        zero = FALSE;
        for (k = 3; k < Dimension; k++)
            if (x [k][j] == -1.0) zero = TRUE;
        if (!(zero || prevzero)) {
            if (i != j)
                for (k = 0; k <= Dimension; k++)
                    x [k][i] = x [k][j];
            i++;
        }
        j++;
    }
    maxday = i;
    for (i = 0; i < maxday; i++)
        x [Dimension][i] = i + 1;
}

void TMultivariateSample::FillDate () {
    int m, d, y;
    for (int i = 1; i < maxday; i++)
        if (x [0][i] == -1.0 && x [0][i - 1] != -1.0) {
            NextDay ((int) x [0][i - 1], (int) x [1][i - 1], (int) x [2][i - 1], d, m, y);
            x [0][i] = d;
            x [1][i] = m;
            x [2][i] = y;
        }
}

void TMultivariateSample::PriceReturnTrafo (String filename, int TreatMondays) {
    if (!CheckReturns ()) return; 
    x = new (double *[Dimension + 1]);
    plog = new double [Dimension];
    startday = CalculateDifference ((int) DataPtr [0][0], (int) DataPtr [1][0], (int) DataPtr [2][0]);
    maxday = CalculateDifference ((int) DataPtr [0][DataSize - 1], (int) DataPtr [1][DataSize - 1], (int) DataPtr [2][DataSize - 1]) - startday;
    int i;
    for (i = 0; i <= Dimension; i++) {
        x [i] = new double [(int) maxday];
        if (i < Dimension)
            for (int j = 0; j < maxday; j++)
                x [i][j] = -1.0;
        else
            for (int j = 0; j < maxday; j++)
                x [i][j] = j + 1;
        if (i > 2 && i < Dimension) plog [i] = log (DataPtr [i][0]);
    }
    for (i = 1; i < DataSize; i++)
        Entry ((int) (CalculateDifference ((int) DataPtr [0][i], (int) DataPtr [1][i], (int) DataPtr [2][i]) - startday - 1), i);
    switch (TreatMondays) {
        case 0: break;
        case 1: RemoveGap (); break;
        case 2: FillGap (); break;
        case 3: TradingDays (); break;
    }
    FillDate ();


    TMultivariateSample *s = new TMultivariateSample;
    s->SetData (GetDimension () + 1, (int) maxday);
    s->SetColumn (0, x [GetDimension ()], "Number");
    for (int dim = 0; dim < GetDimension (); dim++) {
        s->SetColumn (dim + 1, x [dim], GetColumnName (dim));
    }
    TList l;
    l.Append (new TStringEl ("Price Return Transformation of " & GetName ()));
    switch (TreatMondays) {
        case 0: l.Append (new TStringEl ("Monday returns are unchanged")); break;
        case 1: l.Append (new TStringEl ("Monday returns are omitted")); break;
        case 2: l.Append (new TStringEl ("Monday returns are distributed")); break;
        case 3: l.Append (new TStringEl ("Only trading days are listed")); break;
    }
    s->SetDescription (&l);
    s->SetFileName (filename);
    s->SaveData (0);
    SampleList.InsertSample (s);
    l.DeleteAll (FALSE);
    for (i = 0; i <= Dimension; i++)
        delete [] x [i];
    delete [] x;
    delete [] plog;
}

// Date transformation

BOOL TMultivariateSample::CheckReturns () {
    return CheckDates ();
/*
    if (!CheckDates ()) return FALSE;
    for (int i = 0; i < DataSize; i++)
        for (int j = 3; j < GetDimension (); j++)
            if (DataPtr [j][i] <= 0.0)
#ifndef TERMINAL
	        ErrorBox (PlotWindowManager.GetFramePtr (), form (100, "Non positive price found in line %d.", i));
#endif
        	return FALSE;
    return TRUE;
*/
}

BOOL TMultivariateSample::CheckDates () {
    if (GetDimension () < 4) {
#ifndef TERMINAL
        ErrorBox (PlotWindowManager.GetFramePtr (), "Data set should have at least four columns.");
#endif
        return FALSE;
    }
    int length [] = {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
    for (int i = 0; i < DataSize; i++)
        if (fabs (DataPtr [0][i] - (int) DataPtr [0][i]) > 1e-10 ||
            fabs (DataPtr [1][i] - (int) DataPtr [1][i]) > 1e-10 ||
	    fabs (DataPtr [2][i] - (int) DataPtr [2][i]) > 1e-10 ||
            DataPtr [0][i] < 0.99 ||
            DataPtr [1][i] < 0.99 ||
            DataPtr [1][i] > 12.01 ||
            DataPtr [0][i] > length [(int) DataPtr [1][i]] + IsLeapYear ((int) DataPtr [2][i]) + 0.01 ||
            DataPtr [2][i] < 0 ||
            DataPtr [2][i] > 2100) {
#ifndef TERMINAL
            ErrorBox (PlotWindowManager.GetFramePtr (), form (100, "Error in date in row %d.", i + 1));
#endif
	    return FALSE;
        }
    return TRUE;
}

void TMultivariateSample::DateTransformation (String filename) {
    if (!CheckDates ()) return;
    startday = CalculateDifference ((int) DataPtr [0][0], (int) DataPtr [1][0], (int) DataPtr [2][0]);
    double *x = new double [DataSize];
    for (int i = 0; i < DataSize; i++)
        x[i] = CalculateDifference ((int) DataPtr [0][i], (int) DataPtr [1][i], (int) DataPtr [2][i]) - startday + 1;
    TMultivariateSample *s = new TMultivariateSample;
    s->SetData (GetDimension () - 2, DataSize);
    s->SetColumn (0, x, "Number");
    for (int dim = 3; dim < GetDimension (); dim++)
        s->SetColumn (dim - 2, DataPtr [dim], GetColumnName (dim));
    TList l;
    l.Append (new TStringEl ("Date Transformation of " & GetName ()));
    s->SetDescription (&l);
    s->SetFileName (filename);
    s->SaveData (0);
    SampleList.InsertSample (s);
    l.DeleteAll (FALSE);
    delete [] x;
}


#ifndef TERMINAL

TSample *TMultivariateSample::ConvertTo (TSampleType st, TWindow *parent) {
    if (st == TIMESERIES_SAMPLE)
	return ConvertToTimeSeries (parent);
    if (st == UNIVARIATE_SAMPLE)
        return ConvertToUnivariate (parent);
    if (st == GROUPED_SAMPLE)
        return ConvertToGrouped (parent);
    if (st == COPULA_SAMPLE)
    	  return ConvertToCopulaScale (parent);	
    return TSample::ConvertTo (st, parent);
}

void TMultivariateSample::TruncateSample (TWindow *parent) {
    TList list;
    for (int i = 0; i < GetDimension (); i++)
        list.Append (new TStringEl (GetColumnName (i)));
    TCalcDoubleField f (IDD_Threshold, GlobalData.TruncSampleThreshold);
    TWriteFileField e (IDD_FileName, ModifyName ('e'));
    TDialog d (IDD_ExcMultivariate);
    TListField l (IDD_ComponentList, &list);
    d.SetTitel ("Save exceedances");
    d << e << f << l;
    unsigned ch = d.Handle (parent);
    if (ch == IDOK) {
        GlobalData.TruncSampleThreshold = f.GetValue ();
        TruncateSample (e.ReadBuf (), GlobalData.TruncSampleThreshold, l.Selection ());
    }
    list.DeleteAll (TRUE);
}

void TMultivariateSample::LogScaleSample (TWindow *parent) {
    TWriteFileField e (IDD_Name, ModifyName ('a'));
    TDialog d (IDD_ConverseSample);
    d.SetTitel ("Price, Log-Scaled");
    d << e;
    unsigned ch = d.Handle (parent);
    if (ch == IDOK) {
	int n = GetDataSize ();
        double *t = DataPtr [0],
               *r = new double [n];
        TMultivariateSample *s = new TMultivariateSample;
        s->SetData (GetDimension (), n);
        s->SetColumn (0, t, GetColumnName (0));
	for (int i = 1; i < GetDimension (); i++) {
	    int j;
	    double *u = DataPtr [i];
	    for (j = 0; j < n; j++)
	        r [j] = u [j] > 0.0 ? log (u [j]) : -1000.0;
	    double *c = PolynomialRegression (1, n, t, r);
	    for (j = 0; j < n; j++)
	        r [j] -= (c [0] + c [1] * t [j]);
	    s->SetColumn (i, r, GetColumnName (i));
	    delete [] c;
	}
	delete [] r;
        TList l;
        l.Append (new TStringEl ("Data set generated by Xtremes applying a log scale transformation to"));
        l.Append (new TStringEl (GetName ()));
        s->SetDescription (&l);
        s->SetFileName (e.ReadBuf ());
        s->SaveData (parent);
        SampleList.InsertSample (s);
        l.DeleteAll (FALSE);
    }
}

void TMultivariateSample::ConverseSample (TWindow *parent) {
    TWriteFileField e (IDD_Name, ModifyName ('a'));
    TDialog d (IDD_ConverseSample);
    d << e;
    unsigned ch = d.Handle (parent);
    if (ch == IDOK) {
        double *NewData = new double [GetDataSize ()];
        TMultivariateSample *s = new TMultivariateSample;
        s->SetData (GetDimension (), GetDataSize ());
        for (int dim = 0; dim < GetDimension (); dim++) {
            for (int i = 0; i < GetDataSize (); i++)
                NewData [i] = -DataPtr [dim][i];
            s->SetColumn (dim, NewData, GetColumnName (dim));
        }
        TList l;
        l.Append (new TStringEl ("Conversed Sample"));
        l.Append (new TStringEl (GetName ()));
        s->SetDescription (&l);
        s->SetFileName (e.ReadBuf ());
        s->SaveData (parent);
        SampleList.InsertSample (s);
        l.DeleteAll (FALSE);
        delete [] NewData;
    }
}

void TMultivariateSample::SaveBlocksSum (TWindow *parent) {
    TIntField f (IDD_GroupSize, GlobalData.BlockSumSize, 1, INT_MAX);
    TWriteFileField e (IDD_Name, ModifyName ('m'));
    TDialog d (IDD_MaximaDialog);
    d.SetTitel ("Save blocks sum");
    d << e << f;
    unsigned ch = d.Handle (parent);
    if (ch == IDOK) {
        int GroupLength = f.GetValue (),
            NewDataSize = GetDataSize () / GroupLength;
        if (!NewDataSize) {
            ErrorBox (parent, "Block size is larger than sample size");
            return;
        }
	GlobalData.BlockSumSize = GroupLength;
        double *NewData = new double [NewDataSize];
        TMultivariateSample *s = new TMultivariateSample;
        s->SetData (GetDimension (), NewDataSize);
        for (int dim = 0; dim < GetDimension (); dim++) {
            int i, j = 0, k;
            for (i = 0; i < NewDataSize; i++) {
                NewData [i] = 0;
                for (k = 0; k < GroupLength; k++) {
                    NewData [i] += DataPtr [dim][j];
                    j++;
                }
            }
            s->SetColumn (dim, NewData, GetColumnName (dim));
        }
        TList l;
        l.Append (new TStringEl ("Blocks sums "));
        l.Append (new TStringEl ("Group length:  " & form (20, "%d", GroupLength)));
        l.Append (new TStringEl ("Applied to:    " & GetName ()));
        s->SetName ("Blocks Maxima of " & GetName ());
        s->SetDescription (&l);
        s->SetFileName (e.ReadBuf ());
        s->SaveData (parent);
        SampleList.InsertSample (s);
        l.DeleteAll (FALSE);
        delete [] NewData;
    }
}

void TMultivariateSample::SaveMovingMaxima (TWindow *parent) {
    TIntField f (IDD_GroupSize, GlobalData.MovingMaximaSize, 1, INT_MAX);
    TWriteFileField e (IDD_Name, ModifyName ('m'));
    TDialog d (IDD_MaximaDialog);
    d.SetTitel ("Save moving maxima");
    d << e << f;
    unsigned ch = d.Handle (parent);
    if (ch == IDOK) {
        int GroupLength = f.GetValue (),
            NewDataSize = GetDataSize () - GroupLength + 1;
        if (NewDataSize <= 0) {
            ErrorBox (parent, "Block size is larger than sample size");
            return;
        }
	GlobalData.MovingMaximaSize = GroupLength;
        double *NewData = new double [NewDataSize];
        TMultivariateSample *s = new TMultivariateSample;
        s->SetData (GetDimension (), NewDataSize);
        for (int dim = 0; dim < GetDimension (); dim++) {
            int i, k;
            for (i = 0; i < NewDataSize; i++) {
                NewData [i] = DataPtr [dim][i];
                for (k = i; k < i + GroupLength; k++)
                    if (DataPtr [dim][k] > NewData [i]) NewData [i] = DataPtr [dim][k];
            }
            s->SetColumn (dim, NewData, GetColumnName (dim));
        }
        TList l;
        l.Append (new TStringEl ("Blocks sums "));
        l.Append (new TStringEl ("Group length:  " & form (20, "%d", GroupLength)));
        l.Append (new TStringEl ("Applied to:    " & GetName ()));
        s->SetName ("Blocks Maxima of " & GetName ());
        s->SetDescription (&l);
        s->SetFileName (e.ReadBuf ());
        s->SaveData (parent);
        SampleList.InsertSample (s);
        l.DeleteAll (FALSE);
        delete [] NewData;
    }
}

void TMultivariateSample::SaveMovingSum (TWindow *parent) {
    TIntField f (IDD_GroupSize, GlobalData.MovingSumSize, 1, INT_MAX);
    TWriteFileField e (IDD_Name, ModifyName ('m'));
    TDialog d (IDD_MaximaDialog);
    d.SetTitel ("Save moving sums");
    d << e << f;
    unsigned ch = d.Handle (parent);
    if (ch == IDOK) {
        int GroupLength = f.GetValue (),
            NewDataSize = GetDataSize () - GroupLength + 1;
        if (NewDataSize <= 0) {
            ErrorBox (parent, "Block size is larger than sample size");
            return;
        }
	GlobalData.MovingSumSize = GroupLength;
        double *NewData = new double [NewDataSize];
        TMultivariateSample *s = new TMultivariateSample;
        s->SetData (GetDimension (), NewDataSize);
        for (int dim = 0; dim < GetDimension (); dim++) {
            double val = 0.0;
            int i;
            for (i = 0; i < GroupLength; i++) val += DataPtr [dim][i];
            NewData [0] = val;
            for (i = 1; i < NewDataSize; i++)
                NewData [i] = NewData [i - 1] - DataPtr [dim][i - 1] + DataPtr [dim][i + GroupLength - 1];
            s->SetColumn (dim, NewData, GetColumnName (dim));
        }
        TList l;
        l.Append (new TStringEl ("Blocks sums "));
        l.Append (new TStringEl ("Group length:  " & form (20, "%d", GroupLength)));
        l.Append (new TStringEl ("Applied to:    " & GetName ()));
        s->SetName ("Blocks Maxima of " & GetName ());
        s->SetDescription (&l);
        s->SetFileName (e.ReadBuf ());
        s->SaveData (parent);
        SampleList.InsertSample (s);
        l.DeleteAll (FALSE);
        delete [] NewData;
    }
}

void TMultivariateSample::OrderSample (TWindow *parent) {
    TList list;
    for (int i = 0; i < GetDimension (); i++)
        list.Append (new TStringEl (GetColumnName (i)));
    TWriteFileField e (IDD_FileName, ModifyName ('e'));
    TDialog d (IDD_OrderMultivariate);
    TListField l (IDD_ComponentList, &list);
    d.SetTitel ("Order Data");
    d << e << l;
    unsigned ch = d.Handle (parent);
    if (ch == IDOK)
        OrderSample (e.ReadBuf (), l.Selection ());
    list.DeleteAll (TRUE);
}

void TMultivariateSample::CumulateSample (TWindow *parent) {
    TWriteFileField e (IDD_Name, ModifyName ('a'));
    TDialog d (IDD_ConverseSample);
    d.SetTitel ("Cumulate sample");
    d << e;
    unsigned ch = d.Handle (parent);
    if (ch == IDOK) {
        double *NewData = new double [GetDataSize ()];
        TMultivariateSample *s = new TMultivariateSample;
        s->SetData (GetDimension (), GetDataSize ());
        for (int dim = 0; dim < GetDimension (); dim++) {
            NewData [0] = DataPtr [dim][0];
            for (int i = 1; i < GetDataSize (); i++)
                NewData [i] =  NewData [i - 1] + DataPtr [dim][i];
            s->SetColumn (dim, NewData, GetColumnName (dim));
        }
        TList l;
        l.Append (new TStringEl ("Cumulated Sample"));
        l.Append (new TStringEl (GetName ()));
        s->SetDescription (&l);
        s->SetFileName (e.ReadBuf ());
        s->SaveData (parent);
        SampleList.InsertSample (s);
        l.DeleteAll (FALSE);
        delete [] NewData;
    }
}

void TMultivariateSample::PriceReturnTrafo (TWindow *parent) {
    TAlterField a (GlobalData.PRMode, 4, IDD_Opt1, IDD_Opt2, IDD_Opt3, IDD_TradingDays);
    TWriteFileField e (IDD_FileName, GlobalData.PRName);
    TDialog d (IDD_PriceReturnTrafo);
    d << a << e;
    if (d.Handle (parent) == IDOK) {
        GlobalData.PRName = e.ReadBuf ();
        GlobalData.PRMode = a.Selector ();
        PriceReturnTrafo (GlobalData.PRName, GlobalData.PRMode);
    }
}


void TMultivariateSample::DateTransformation (TWindow *parent) {
    TWriteFileField e (IDD_FileName, GlobalData.DateName);
    TDialog d (IDD_DateTransformation);
    d << e;
    if (d.Handle (parent) == IDOK) {
        GlobalData.DateName = e.ReadBuf ();
        DateTransformation (GlobalData.DateName);
    }
}
#endif

