/*  normest.hpp

    Estimators for Gaussian, stable and t distributions.

    This file is part of Xtremes for Windows,
    Copyright (C) 1991, 2000 Xtremes Group, Siegen.
*/

#ifndef NORMEST_HPP
#define NORMEST_HPP

#include "estimate.hpp"

#ifndef TERMINAL
#include "ufodlg.hpp"
#endif

class TNormalMLEstimator : public TNormalEstimator {
    public:
	TNormalMLEstimator (BOOL AddList = FALSE);
	TEstimator *Duplicate ();
	virtual TEstResult Estimate (int k, TSample *);
};

class TNormalLSEstimator : public TNormalEstimator {
    public:
	TNormalLSEstimator (BOOL AddList = FALSE);
	TEstimator *Duplicate ();
	virtual TEstResult Estimate (int k, TSample *);
};

class TNormalQuickEstimatorData : public TEstimatorData {
    public:
        double q;
        BOOL Symmetric;
};

class TNormalQuickEstimator : public TNormalEstimator {
    typedef TNormalEstimator inherited;
    public:
    	TNormalQuickEstimator (BOOL AddList = FALSE);
        TEstimator *Duplicate();
        virtual TEstResult Estimate (int, TSample *);
        virtual TEstimatorData *GetParameter ();
    protected:
#ifndef TERMINAL
	void NotifyActionEvent (TDialogMember *m);
	virtual void EvaluateAdditionalParameters ();
#endif
    private:
#ifndef TERMINAL
	TCalcDoubleField qField;
        TSelectField SymmetricField;
#endif
    	TNormalQuickEstimatorData Data;
};

#ifdef USE_STABLE

class TStableMLEstimator : public TStableEstimator {
    public:
	TStableMLEstimator (BOOL AddList = FALSE);
	TEstimator *Duplicate ();
	virtual TEstResult Estimate (int k, TSample *);
};

class TStableKKWEstimator : public TStableEstimator {
    public:
	TStableKKWEstimator (BOOL AddList = FALSE);
	TEstimator *Duplicate ();
	virtual TEstResult Estimate (int k, TSample *);
};

#endif

class TTMLEstimator : public TTEstimator {
    public:
	TTMLEstimator (BOOL AddList = FALSE);
	TEstimator *Duplicate ();
	virtual TEstResult Estimate (int k, TSample *);
    private:
	virtual double SearchL (double *p);
	virtual BOOL SearchBounds (double *p);
};

class TNCTMLEstimator : public TNCTEstimator {
    public:
	TNCTMLEstimator (BOOL AddList = FALSE);
	TEstimator *Duplicate ();
	virtual TEstResult Estimate (int k, TSample *);
    private:
	virtual double SearchL (double *p);
	virtual BOOL SearchBounds (double *p);
};

void OpenGARCHDialog (TTimeSeriesSample *p, TWindow *parent);

#endif

