/* pathman
  ~~~~~~~~~
  $Id: pathman.cpp,v 1.20 2001/12/06 16:32:00 reinhard Exp $

  Description:

  Todo:

  Bugs:

  created: 2000-03-09 Reinhard Pfau <Reinhard.Pfau@unix-ag.org>
  modified:

*/


#include "pathman.hpp"

#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <pwd.h>
#include <sys/types.h>
#include <stdlib.h>
#include <ctype.h>

#include <list>
#include <algorithm>

#include "debug.hpp"


using std::list;


namespace {

  String GetHomeDir()
  {
    struct passwd *pEntry = 0;
    String homedir("/");
    setpwent();
    for (pEntry= getpwent(); pEntry; pEntry=getpwent()) {
      if (pEntry->pw_uid == getuid()) {
        homedir = pEntry->pw_dir;
        break;
      }
    }
    endpwent();
    return homedir;
  }


  bool DirExists(const String& path)
  {
    bool result = false;
    struct stat statInfo;
    if (! stat(path,&statInfo)) {
      result = S_ISDIR(statInfo.st_mode);
    }
    return result;
  }


  bool FileExists(const String& path)
  {
    bool result = false;
    struct stat statInfo;
    if (! stat(path,&statInfo)) {
      result = S_ISREG(statInfo.st_mode);
    }
    return result;
  }


  bool Mkdir(const String& path)
  {
    return !mkdir(path,0777);
  }


  String GetCwd()
  {
    String result;
    char *buf;
    buf = getcwd(0,0);
    if (buf) {
      result = buf;
      free(buf);
    }
    return result;
  }


  bool SetWorkingDirectory(String path)
  {
    return !chdir(path);
  }


  struct PathWithFileTester {
      PathWithFileTester(const String& _filename) :
        filename(_filename)
      {}

      bool operator () (const String& path)
      {
        //MCERR("");
        String fullpath;
        fullpath= path & "/" & filename;
        return FileExists(fullpath);
      }

      String filename;
  };



  /* internal variables
   *
   */
  list<String> pathList;
  String workPath;
  String userDataPath;
  String sharedDataPath;

}


namespace PathManager {

#define SHOWLIST1() \
  FCERR("pL: "<< &*pathList.begin() << " - " << &*pathList.end())

#define SHOWLIST() \
  do { \
    FCERR("** pathList: begin"); \
    for (list<String>::iterator it= pathList.begin();it != pathList.end();++it)\
    { \
      std::cerr << it._M_node << ": prev=" << it._M_node->_M_prev \
                << " ; next=" << it._M_node->_M_next << std::endl; \
      std::cerr << it._M_node << ": " << &*it << ": " << std::flush; \
      std::cerr << *it << std::endl; \
    } \
    FCERR("** pathList: end"); \
  } while (0) 

  
  bool Init(String program)
  {
    bool result = true;
    String home = GetHomeDir();
    char *p;
    if ((p=getenv("XTREMES_SHARED_DATA_PATH"))) {
      sharedDataPath = p;
    }
    userDataPath = home & "/." & program;
    workPath = GetCwd();
    if (! DirExists(userDataPath)) {
      if (! Mkdir(userDataPath)) {
        result = false;
        userDataPath = workPath;
      }
    }
    AddSearchPath(workPath,false);
    if (result) {
      // add the userDataPath to our search list.
      AddSearchPath(userDataPath,false);
    }
    if (sharedDataPath.Len()>0) {
      AddSearchPath(sharedDataPath,false);
    }
    return result;
  }


  String GetWorkPath()
  {
    return workPath;
  }
  

  String GetSharedDataPath()
  {
    return sharedDataPath;
  }


  String GetUserDataPath()
  {
    return userDataPath;
  }

  
  bool AddSearchPath(const String& newPath, bool prepend)
  {
    //FCERR(&newPath << ":" << newPath);
    if (find(pathList.begin(), pathList.end(), newPath) == pathList.end()) {
      //FCERR("prepend is " << prepend);
      if (prepend) {
        pathList.push_front(newPath);
      } else {
        pathList.push_back(newPath);
      }
      //SHOWLIST1();
      //SHOWLIST();
      return true;
    }
    //SHOWLIST1();
    //SHOWLIST();
    //FCERR("");
    return false;
  }
  

  void DelSearchPath(const String& oldPath)
  {
    FCERR(oldPath);
    //SHOWLIST1();
    //SHOWLIST();
    pathList.remove(oldPath);
    //SHOWLIST1();
    //SHOWLIST();
  }
  

  String Resolv(String filename, bool cwdFirst)
  {
    String result;
    String cwd;
    if (filename.Len()>0 && *(const char*)filename=='/') {
      // if the filename is absolute, no path search will be done!
      return filename;
    }
    if (cwdFirst) {
      cwd = GetCwd();
      AddSearchPath(cwd,true);
    }
    list<String>::iterator it =
      find_if(pathList.begin(), pathList.end(), PathWithFileTester(filename) );
    //FCERR("it=" << it._M_node );
    if (it != pathList.end()) {
      result = *it & "/" & filename;
    }
    if (cwdFirst) {
      DelSearchPath(cwd);
    }
    return result;
  }

}
