/*  pcode.hpp
 *
 *  Declaration of classes of the virtual machine for XPL.
 *
 *  This file is part of XPL.
 *  Copyright (C) 1991, 1999 Xtremes Group, Siegen.
 */

#ifndef PCODE_HPP
#define PCODE_HPP

#include <complex>

#include "boolean.hpp"
#include "string.hpp"

#include "maxstat.hpp"

#define N_SET_CELLS	4		
#define N_MAX_SETORD    (N_SET_CELLS * sizeof (double) * 8)

#define LOOPSTACKSIZE	100
#define INPUTBUFSIZE    10000

typedef int TCode;

enum { LOAD, LOADT, FLOAD, LOADMEM, LOADVAL, LOADPTR,
       STOREMEM, STOREVAL, INC, DEC, INCPTR, COPYMEM,
       LOADSTR,
       POPIND, SETIND, LOADIND, VLOADIND, STOREIND, VSTOREIND,
       LOADLS, LOOP, MVSPIND, ADDIND,
       CHECK, ISMA, RSP,
       ALLOC, DISPOSE, AVAIL,
       JUMP, JZ, JNZ, CALL, ITERINIT, ITERDO, ITERLOOP, PUSH,
       ENTER, LEAVE, ADDCSP,
       DUP,
       FADD, FSUB, FMUL, FDIV, FDIVC, FPOW, FFAC, FNEG, INEG,
       FEQ, FNE, FLT, FGT, FLE, FGE,
       IEQ, INE, ILT, IGT, ILE, IGE,
       SEQ, SNE, SLT, SGT, SLE, SGE,
       CIF, CFI,
       IADD, ISUB, ISUBD, IMUL, IDIV, IDIVC, IMOD, IPOW,
       IMODC, IAND, IOR, IXOR, INOT,
       CMPJEQ, CMPRANGE,

       CMAKE, CADD, CSUB, CMUL, CDIV, CDIVC,

       SETCREATE, SETAND, SETOR, SETNEG, SETINC, SETEMPTY,

       VINIT, VFREE, VLOAD, VSTORE, VPUSH, VREAL, VINT, VCIF,
       VCVAL, VCVEC, VUNIT,
       VPRINTDBL, VPRINTINT, VPRINTBOOL,
       MPRINTDBL, MPRINTINT, MPRINTBOOL, MPRINTCHAR,
       VREV, VCUMSUM, VSUM, VFSRT, VFRANK, VFMAX, VFMIN, VIMAX, VIMIN,
       VFADD, VFSUB, VFMUL, VFDIV, VFDIVC, VFPOW,
       VIADD, VISUB, VIMUL, VIDIV, VIDIVC, VIMOD, VIMODC, VIDIVF, VIDIVFC, VIPOW, 
       VIAND, VIOR, VIXOR, VINOT, VINEG, VFNEG,

       MFMUL, MIMUL,
       MTRANS, MINV, MUNIT, MSQRT, MREAL,
       MCBIND, MRBIND,

       VFEQ, VFNE, VFLT, VFGT, VFLE, VFGE,
       VIEQ, VINE, VILT, VIGT, VILE, VIGE,
       VMAX2, VMIN2, VIMAX2, VIMIN2,
       VSIZE, VSSIZE, VCHECK, VINDINT, VINDINTC, VINDBOOL, 
       VSINDINT, VSINDINTC, VSINDBOOL, VLVINDINT,

       VSIN, VARCSIN, VCOS, VARCCOS, VTAN, VARCTAN,
       VCOSH, VSINH, VTANH, VLOG, VLOGC, VEXP, VSQRT, VSQRTC, VSIGN, VFRAC,
       VROUND, VTRUNC, VMAXV, VMINV, VISQR, VFSQR, VEXIST, VFORALL,
       VIABS, VFABS, VIODD,

       ISQR, FSQR, IABS, IMAX, IMIN, IODD,
       FABS, FSIN, FASIN, FCOS, FACOS, FTAN,
       FATAN, FCOSH, FSINH, FTANH, FLOG, FEXP,
       FSQRT, FSIGN, FFRAC, FINT, FTRUNC,
       FMAX, FMIN, FCHR,

       FRANDOM,

       CFS,
       PRINTBOOL, PRINTCHAR, PRINTINT, PRINTDBL, PRINTSTR, PRINTC, PRINTLF,
//       PRINTBOOL1, PRINTCHAR1, PRINTINT1, PRINTDBL1, PRINTSTR1,
//       PRINTDBL2,	
       GETCHAR, GETINT, GETDBL, GETSTR, GETLF, SCAN,
       PAGE, TXTCOL, TXTBCK, GOTOXY, KEYP,

       FASSIGN, FTASSIGN, FRESET, FREWRITE, FWRITE, FREAD, FCLOSE, FEOF,
       FFLUSH,

       SYSTEM, SYSDATE, SYSTIME, ARGC, ARGV,

       SPHALT
};

typedef int int32;
typedef std::complex<double> dcomplex;

struct TPCode {
    TPCode () { code = SPHALT; }
    ~TPCode ();
    TCode code;
    union tagpara {
        struct tagintp {
	    int32 intpara, intpara2;
	} intp;
	double doublepara;
	String *stringpara;
    } para;
};

union TStorage {
    double doubleval;
    int  intval;
    struct {int32 l1, l2; } s;
};

enum TRunTimeErrorType { RT_OK, RT_OPCODE, RT_DIVZERO, RT_RANGECHECK, RT_STACKOVERFLOW,
                         RT_HEAP_FULL, RT_ARGUMENT, RT_NOSAMPLE, RT_MATRIXOP, RT_VECTOREMPTY,
			 RT_OVERLOW, RT_USERBREAK };

class TFileDescriptor {
    public:
	TFileDescriptor ();
	virtual ~TFileDescriptor ();
	virtual void Reset ();
	virtual void Rewrite ();
	virtual void OutputBin (void *ptr, int size);
	virtual void InputBin (void *ptr, int size);
	virtual void OutputText (String s);
	virtual int Eof ();
	virtual void FFlush ();

	virtual void Page ();
        virtual void SetTextColor (int Nr);
        virtual void SetTextBackground (int Nr);
        virtual void GotoXY (int x, int y);	// only for terminal windows
	virtual int GetKey ();			// dto.

	// Input operations

        double GetDouble ();
        int GetInt ();
        int GetChar ();
        String GetString ();
	void GetLF ();
        virtual void FillInputBuf ();
	char InputBuf [INPUTBUFSIZE], *InputPtr;
};

class TBinFileDescriptor : public TFileDescriptor {
    public:
 	TBinFileDescriptor (String s);
	virtual ~TBinFileDescriptor ();
	virtual void Reset ();
	virtual void Rewrite ();
	virtual void OutputBin (void *ptr, int size);
	virtual void InputBin (void *ptr, int size);
	virtual int Eof ();
	virtual void FFlush ();
    protected:
	FILE *f;
	String s;
};

class TTextFileDescriptor : public TFileDescriptor {
    public:
	TTextFileDescriptor (String s);
	virtual ~TTextFileDescriptor ();
	virtual void Reset ();
	virtual void Rewrite ();
	virtual void OutputText (String s);
	virtual void FillInputBuf ();
	virtual int Eof ();
	virtual void FFlush ();
    protected:
	FILE *f;
	String s;
};

class TConsoleFileDescriptor : public TFileDescriptor {
    public:
	virtual void OutputText (String s);
        virtual void FillInputBuf ();
	virtual int Eof ();
};  	

class TFileList {
    public:
	TFileList (int N = 20);
	virtual ~TFileList ();
	virtual int OpenBin (String fn);
	virtual int OpenText (String fn);
	TFileDescriptor *GetFileDescriptor (int);
	virtual void Close (int);
	virtual void CloseAll ();
	void SetStandardOutput (TFileDescriptor *);
	void SetStandardInput (TFileDescriptor *);
    protected:
   	int FindFree ();
	TFileDescriptor **p;
	BOOL *isFree;
	int N;
};

class TPCodeInterpreter {
    public:
        TPCodeInterpreter ();
        virtual ~TPCodeInterpreter ();

        BOOL CreateMemory (int MemSize, int PointerTableSize, int CalcStackSize, int CodeSize);
	BOOL CreateDataMemory (int MemSize, int PointerTableSize, int CalcStackSize);
	void SetFileList (TFileList *);
	void DeleteMemory ();
	void DeleteDataMemory ();
	void SetProgramArguments (int argc, char **argv);

	TRunTimeErrorType Run ();		// init and execute

	void Init ();                           // used for nodes to pass parameters
        void PushValue (double);
	void PushValue (int);
        void PushVector (int n, double *v);
	void PushMatrix (int lines, int cols, double *v);
        double GetDataValue (int Offset);
	int GetIntValue (int Offset);
        void GetVector (int Offset, int &n, double *&v);
	void GetMatrix (int Offset, int &lines, int &cols, double *&v);
	TRunTimeErrorType Execute ();

	int GetPC () { return pc; }

	String ListingLine (int i);
	virtual void AddListingLine (String s);
	void Listing ();

	void Reset ()           { pc = 0; }

	void OutCode (TCode);
	void OutCode (TCode, int, int = 0);
	void OutCode (TCode, double);
	void OutCode (TCode, String);

	int GetPos ();
	void Fixup (int FixupPos, TCode, int);
	void SetProgramSize ()          { ProgramSize = pc; }

	void Save (FILE *);
	void Load (FILE *);

    protected:
	virtual TRunTimeErrorType ExtendedCode (TPCode &c);  // called for unknown codes

	// I-O Routines
	virtual void WriteString (int fd, String s);    // virtual output routines
	virtual void WriteChar (int fd, int c);
	virtual void WriteLinefeed (int fd);

	// Heap
	int Allocate (int n);
	void Deallocate (int p);
	void Compactify ();

	void PushStringAsVector (String s);
	String PopStringFromVector ();
	void PopDoubleVector (double *&a, int &n);
	void PopDoubleMatrix (double *&a, int &n, int &m);
	void LoadVectorOnCalculatorStack (TStorage VectorData);

	int VectorLength () 		{ return store [csp - 1].s.l1; }
	int VectorRows ()		{ return store [csp - 2].s.l1; }
        int VectorCols ()		{ return store [csp - 2].s.l2; }
        int VectorMemberSize ()         { return store [csp - 1].s.l2; }
	TStorage *VectorStart ()	{ return store + csp - VectorLength () - 1; }

	void ReleaseCalcStack (int n)   { csp -= n; }
	void ReserveCalcStack (int n)   { csp += n; }
	TStorage *CalcStackTop ()	{ return store + csp - 1; }
	TStorage *CalcStackFirstFree () { return store + csp; }

	void DPush (double a)   { store [csp++].doubleval = a; }
	double DPop ()          { return store [--csp].doubleval; }

        void CPush (dcomplex);
        dcomplex CPop ();

	void IPush (int a)      { store [csp++].intval = a; }
	int IPop ()             { return store [--csp].intval; }

	void IPush (int32 l1, int32 l2)		{ store [csp].s.l1 = l1; store [csp++].s.l2 = l2; }
	void IPop (int32 &l1, int32 &l2)	{ l1 = store [--csp].s.l1; l2 = store [csp].s.l2; }

	void Push (TStorage a)  { store [csp++] = a; }
	TStorage Pop ()         { return store [--csp]; }

	TStorage *store;                        // data storage and routine stack
	TPCode *code;                           // program storage
	int pc, sp, csp;                        // program pointer, double stack pointer,
						// calculator stack pointer

	int heapptr;				// heap pointer
	int freeptr;				// free list (heap)

	int loopsp;				// loop stack pointer
	int loopstack [LOOPSTACKSIZE];
	int indexptr;				// index register

	int MemSize, CodeSize,
            PointerTableSize, CalcStackSize;
	int DisplayBP [MAXSTATIC];
	int ProgramSize;

	TFileList *FileList;

	int argc; 
	char **argv;
};

#endif
