/*  pml.cpp

    Implementation of PML related methods of TMultivariateSample

    This file is part of Xtremes for Windows,
    Copyright (C) 1991, 1997 Xtremes Group, Siegen.
*/

#include <stdlib.h>
#include <limits.h>
#include <math.h>

#include "allest.hpp"
#include "mathfunc.hpp"
#include "samplist.hpp"

#ifndef TERMINAL
#include "empiric.hpp"
#include "wintypes.hpp"
#endif

/*******************************************************************/
/*		 	PML transformations			   */
/*******************************************************************/

void TMultivariateSample::PMLSortPML (String filename) {
    OrderSample (filename, 0);
}

void TMultivariateSample::PMLAddDegrees (String filename) {
    int i, j;
    x = new double *[6];
    for (j = 0; j < 6; j++) {
	x [j] = new double [DataSize];
	if (j < 5) memcpy (x [j], DataPtr [j], DataSize * sizeof (double));
    }
    for (i = 0; i < DataSize; i++)
	x [5][i] = DataPtr [4][i] / DataPtr [3][i];
    char *NameField [6] = { "Group", "From", "To", "PML", "Claim size", "Degree" };
    TMultivariateSample *s = new TMultivariateSample;
    s->SetData (6, DataSize);
    for (i = 0; i < 6; i++) {
	s->SetColumn (i, x [i], NameField [i]);
    }
    TList l;
    l.Append (new TStringEl ("PML degrees added to " & GetName ()));
    s->SetDescription (&l);
    s->SetFileName (filename);
    s->SaveData (0);
    SampleList.InsertSample (s);
    l.DeleteAll (FALSE);
    for (i = 0; i < 6; i++)
        delete [] x [i];
    delete [] x;
}

BOOL TMultivariateSample::PMLAddPriorities (String filename, int m, double *p) {
    BOOL Error = FALSE;
    int i, j;
    x = new double *[5];
    for (j = 0; j < 5; j++)
        x [j] = new double [DataSize];
    for (i = 0; i < DataSize; i++) {
        x [3][i] = DataPtr [0][i];
	x [4][i] = DataPtr [1][i];
    }
    sort (0, DataSize - 1, x [3], x [4]);
    for (i = 0; i < DataSize; i++) {
	j = 0;
	while (j < m && x [3][i] > p [j]) j++;
	if (j == m) {
	    Error = TRUE;
	    x [1][i] = p [m - 1];
	    x [2][i] = x [3][i];
	} else if (x [3][i] <= p [0]) {
	    Error = TRUE;
            x [1][i] = 0;
	    x [2][i] = p [0];
        } else {
            x [1][i] = p [j - 1];
            x [2][i] = p [j];
        }
        x [0][i] = j;
    }

    char *NameField [5] = { "Group", "From", "To", "PML", "Claim size" };
    TMultivariateSample *s = new TMultivariateSample;
    s->SetData (5, DataSize);
    for (i = 0; i < 5; i++) {
        s->SetColumn (i, x [i], NameField [i]);
    }
    TList l;
    l.Append (new TStringEl ("PML priorities added to " & GetName ()));
    s->SetDescription (&l);
    s->SetFileName (filename);
    s->SaveData (0);
    SampleList.InsertSample (s);
    l.DeleteAll (FALSE);
    for (i = 0; i < 5; i++)
        delete [] x [i];
    delete [] x;

    return Error;
}

void TMultivariateSample::PMLEntry (double center, double claimmean, double claimvar, double degmean, double degvar) {
    if (Count == Allocated) {
        Allocated += 20;
        for (int i = 0; i < 5; i++)
	    x [i] = (double *) realloc (x [i], Allocated * sizeof (double));
    }
    x [0][Count] = center;
    x [1][Count] = claimmean;
    x [2][Count] = claimvar;
    x [3][Count] = degmean;
    x [4][Count] = degvar;
    Count++;
}

void TMultivariateSample::PMLDistributeEntry (double val) {
    if (Count == Allocated) {
        Allocated += 50;
        y = (double *) realloc (y, Allocated * sizeof (double));
    }
    y [Count++] = val;
}

void TMultivariateSample::PMLEstimatorEntry (double groupnr, double t, double b, double m, double s, double empmean, double d) {
    if (PMLEstCount == PMLEstAllocated) {
	PMLEstAllocated += 50;
	PMLgroupNr = (double *) realloc (PMLgroupNr, PMLEstAllocated * sizeof (double));
	PMLgroup   = (double *) realloc (PMLgroup,   PMLEstAllocated * sizeof (double));
	PMLbeta    = (double *) realloc (PMLbeta,    PMLEstAllocated * sizeof (double));
	PMLmu      = (double *) realloc (PMLmu,      PMLEstAllocated * sizeof (double));
	PMLsigma   = (double *) realloc (PMLsigma,   PMLEstAllocated * sizeof (double));
	PMLu       = (double *) realloc (PMLu,       PMLEstAllocated * sizeof (double));
	PMLd       = (double *) realloc (PMLd,       PMLEstAllocated * sizeof (double));
    }
    PMLgroupNr [PMLEstCount] = groupnr;
    PMLgroup [PMLEstCount] = t;
    PMLbeta [PMLEstCount] = b;
    PMLmu [PMLEstCount] = m;
    PMLsigma [PMLEstCount] = s;
    PMLu [PMLEstCount] = empmean;
    PMLd [PMLEstCount] = d;
    PMLEstCount++;
}

void TMultivariateSample::PMLDistributeMakeDataSet (String filename, int group, double s0, double s1,
    int PlotMode, double t, int EstimatorNr, BOOL UseClaimSize) {
    String fn = filename & form (10, "%d", group);
    TUnivariateSample *s = new TUnivariateSample;
    s->SetName ("Group " & form (10, "%d", group));
    double meanval = 0.0;
    for (int i = 0; i < Count; i++)
	meanval += y [i];
    if (Count) meanval /= Count;
    else meanval = NOTAVAILABLE;
    if (!UseClaimSize) {
	for (int i = 0; i < Count; i++)
	    y [i] -= 1.0;
	PMLDistributeEntry (t / s1 - 1.0);
    } else PMLDistributeEntry (t);

    TList l;
    if (UseClaimSize) 
        l.Append (new TStringEl (form (100, "Claims in PML group %d in ", group) & GetFileName ()));
    else
	l.Append (new TStringEl (form (100, "Degrees in PML group %d in ", group) & GetFileName ()));
    l.Append (new TStringEl ("******************************************************"));
    l.Append (new TStringEl ("Treshold:  " & form (30, "%g", t)));
    l.Append (new TStringEl (form (100, "PML group: from %g to %g", s0, s1)));
    s->SetDescription (&l);
    s->SetName ("PML group " & form (20, "%d", group));
    s->SetData (Count, y, fn);

    int k = Count; //  n = s->GetDataSize ();
    TEstimator *e = NULL;
    switch (EstimatorNr) {
	case 0: e = HillGP1; break;
	case 1: e = MLEGP; break;
	case 2: e = MomentGP; break;
	case 3: e = MLEExponential; break;
    }
    String wn = fn & form (50, " [%g, %g)", s0, s1);
    if (e->Estimate (k, s) == EST_OK) {
	double shape = e->GetEstimatedParameterValue (0),
	       loc = e->GetEstimatedParameterValue (1),
	       scale = e->GetEstimatedParameterValue (2), u, d;
	if (!UseClaimSize) loc += 1.0;
#ifndef TERMINAL
	if (PlotMode == 2)
	    if (EstimatorNr == 0)
		if (shape < 0.0) {
		    loc = 1.0;
		    PlotWindowManager.GetOpenWindow (wn)->InsertFunction (new TBetaDistribution (shape, scale, loc, 1));
		} else
		    PlotWindowManager.GetOpenWindow (wn)->InsertFunction (new TParetoDistribution (shape, scale, loc, 1));
	    else
		PlotWindowManager.GetOpenWindow (wn)->InsertFunction (new TGPDDistribution (shape, scale, loc, 1));
	else if (PlotMode == 1)
	    if (EstimatorNr == 0)
		if (shape < 0.0) {
		    loc = 1.0;
		    PlotWindowManager.GetOpenWindow (wn)->InsertFunction (new TBetaQuantile (shape, scale, loc, 1));
		} else
		    PlotWindowManager.GetOpenWindow (wn)->InsertFunction (new TParetoQuantile (shape, scale, loc, 1));
	    else
		PlotWindowManager.GetOpenWindow (wn)->InsertFunction (new TGPDQuantile (shape, scale, loc, 1));
#endif
	u = t / s1;
	if (EstimatorNr == 0)
	    if (shape < 0.0) {
		d = (1.0 + u * fabs (shape)) / (1.0 + fabs (shape));
                scale = 1.0 - u;
	    } else if (shape > 1.0) d = scale * shape / (shape - 1.0);
	    else d = NOTAVAILABLE;
	else
	    if (shape < 1.0) d = loc + scale / (1.0 - shape);
	    else d = NOTAVAILABLE;
	PMLEstimatorEntry (group, (s0 + s1) / 2.0, shape, loc, scale, meanval, d);
    } else
	PMLEstimatorEntry (group, (s0 + s1) / 2.0, NOTAVAILABLE, NOTAVAILABLE, NOTAVAILABLE, meanval, NOTAVAILABLE);

#ifndef TERMINAL
    if (!UseClaimSize) {
	s = new TUnivariateSample;
        s->SetName ("Group " & form (10, "%d", group));
        for (int i = 0; i < Count; i++)
            y [i] += 1.0;
    	s->SetDescription (&l);
	s->SetData (Count, y, fn);
    }

    if (PlotMode == 2)
	PlotWindowManager.GetOpenWindow (wn)->InsertFunction (new TEmpiricalDF (s));
    else if (PlotMode == 1)
	PlotWindowManager.GetOpenWindow (wn)->InsertFunction (new TEmpiricalQF (s));
#endif

    delete [] y;
    y = NULL;
    Count = 0;
    Allocated = 0;
}

void TMultivariateSample::PMLDistributeDataSet (String filename, String ParaFilename,
    int PlotMode, BOOL UseClaimSize, double t, int EstimatorNr) {
    PMLEstCount = PMLEstAllocated = Allocated = Count = 0;
    int group = (int) DataPtr [0][0];
    y = NULL;
    double from = DataPtr [1][0], to = DataPtr [2][0];
    for (int i = 0; i < DataSize; i++)
	if (from == DataPtr [1][i] && to == DataPtr [2][i]) {
	    if (DataPtr [4][i] >= t)
		PMLDistributeEntry (DataPtr [UseClaimSize ? 4 : 5][i]);
	} else {
	    PMLDistributeMakeDataSet (filename, group++, from, to, PlotMode, t, EstimatorNr, UseClaimSize);
	    from = DataPtr [1][i];
	    to = DataPtr [2][i];
	    if (DataPtr [4][i] >= t)
		PMLDistributeEntry (DataPtr [UseClaimSize ? 4 : 5][i]);
	}
    PMLDistributeMakeDataSet (filename, group++, from, to, PlotMode, t, EstimatorNr, UseClaimSize);
    // store estimators for groups
    if (PMLEstCount) {
	TMultivariateSample *s = new TMultivariateSample;
	s->SetData (7, PMLEstCount);
	s->SetColumn (0, PMLgroupNr, "Group Nr.");
	s->SetColumn (1, PMLgroup, "Center");
	s->SetColumn (2, PMLu, UseClaimSize ? "Emp.Meansize" : "Emp.Meandeg");
	s->SetColumn (3, PMLd, UseClaimSize ? "Par.Meansize" : "Par.Meandeg");
	s->SetColumn (4, PMLbeta, EstimatorNr == 0 ? "alpha" : "gamma");
	s->SetColumn (5, PMLmu, "mu");
	s->SetColumn (6, PMLsigma, "sigma");

	TEstimator *e = NULL;
	switch (EstimatorNr) {
	    case 0: e = HillGP1; break;
	    case 1: e = MLEGP; break;
	    case 2: e = MomentGP; break;
	    case 3: e = MLEExponential; break;
	}


	TList l;
	l.Append (new TStringEl ("Analysis of PML groups in " & GetFileName ()));
	l.Append (new TStringEl ("***********************************************"));
	l.Append (new TStringEl ("Estimator:  " & e->Name ()));
	l.Append (new TStringEl ("Threshold:  " & form (30, "%g", t)));
	if (UseClaimSize)
	    l.Append (new TStringEl ("Applied to: Claim Sizes"));
	else
	    l.Append (new TStringEl ("Applied to: Degrees"));
	s->SetDescription (&l);

	s->SetFileName (ParaFilename);
	s->SaveData (0);
	SampleList.InsertSample (s);
        delete [] PMLgroupNr; PMLgroupNr = NULL;
	delete [] PMLgroup; PMLgroup = NULL;
	delete [] PMLbeta; PMLbeta = NULL;
        delete [] PMLmu; PMLmu = NULL;
	delete [] PMLsigma; PMLsigma = NULL;
	delete [] PMLu; PMLu = NULL;
	delete [] PMLd; PMLd = NULL;
    }
}

void TMultivariateSample::PMLFunctionalParameterTrafo (String filename) {
    int i;
    x = new double *[5];
    Allocated = 20;
    Count = 0;

    for (i = 0; i < 5; i++)
        x [i] = (double *) malloc (Allocated * sizeof (double));
    double from = DataPtr [1][0], to = DataPtr [2][0],
	   sum = 0.0, sumsq = 0.0,
           sumdeg = 0.0, sumdegsq = 0.0;
    int n = 0;
    for (i = 0; i < DataSize; i++) {
        if (from == DataPtr [1][i] && to == DataPtr [2][i]) {
            sum += DataPtr [4][i];
	    sumsq += square (DataPtr [4][i]);
	    sumdeg += DataPtr [5][i];
            sumdegsq += square (DataPtr [5][i]);
            n++;
        } else {
	    PMLEntry ((from + to) / 2, sum / n, sumsq / n - square (sum / n), sumdeg / n, sumdegsq / n - square (sumdeg / n));
	    sum = DataPtr [4][i];
	    sumsq = square (DataPtr [4][i]);
	    sumdeg = DataPtr [5][i];
            sumdegsq += square (DataPtr [5][i]);
	    n = 1;
	    from = DataPtr [1][i];
	    to = DataPtr [2][i];
	}
    }
    PMLEntry ((from + to) / 2.0, sum / n, sumsq / n - square (sum / n), sumdeg / n, sumdegsq / n - square (sumdeg / n));

    char *NameField [5] = { "Center (PML)", "Mean (Claim)", "Var (Claim)", "Mean (Degree)", "Var (Degree)" };
    TMultivariateSample *s = new TMultivariateSample;
    s->SetData (5, Count);
    for (i = 0; i < 5; i++) {
	s->SetColumn (i, x [i], NameField [i]);
    }
    TList l;
    l.Append (new TStringEl ("Functional Parameter Transformation of " & GetName ()));
    s->SetDescription (&l);
    s->SetFileName (filename);
    s->SaveData (0);
    SampleList.InsertSample (s);
    l.DeleteAll (FALSE);

    for (i = 0; i < 5; i++) free (x [i]);
    delete [] x;
}

/*******************************************************************/
/*		       PML: Nearest Neighbors			   */
/*******************************************************************/

// GP-MLE for sigma

static double f (double *x, int n, double gamma, double mu, double sigma) {
    double sum = 0.0;
    for (int i = 0; i < n; i++)
	sum += (gamma + 1.0) * (x [i] - mu) / (sigma + gamma * (x [i] - mu));
    return sum - n;
}

static double fprime (double *x, int n, double gamma, double mu, double sigma) {
    double sum = 0.0;
    for (int i = 0; i < n; i++)
	sum -= (gamma + 1.0) * (x [i] - mu) / sqr (sigma + gamma * (x [i] - mu));
    return sum;
}

double GPMLESigma (double *x, int n, double gamma, double mu, double sigma) {
    int steps = 0;
    double sigma1 = sigma, sigma0;
    do {
	sigma0 = sigma1;
	sigma1 = sigma0 - f (x, n, gamma, mu, sigma0) / fprime (x, n, gamma, mu, sigma0);
    } while (fabs (sigma0 - sigma1) < 1e-3 && ++steps != 50);
    return sigma1;
}

void TMultivariateSample::PMLNearestNeighbors (String filename, int k, int est, double gamma, double mu) {
    int n = DataSize, i, j;
    double *x = new double [n + 1], *z = new double [n + 1],
	   *px = new double [n], *py1 = new double [n],
	   *py2 = new double [n], *py3 = new double [n],
	   *py4 = new double [n], *py5 = new double [n],
	   *rx = new double [k], *ry = new double [k];
    TUnivariateSample *p = new TUnivariateSample;
    for (i = 1; i <= n; i++) {
	z [i] = DataPtr [0][i - 1];
	x [i] = DataPtr [1][i - 1];
    }
    double para [3];
    for (i = 1; i <= n; i++) {
	int low = i, hi = i;
	double med = z [i];
	while (hi - low < k - 1) {
	    BOOL Lower;
	    if (hi < n && low > 1) Lower = (med - z [low - 1] < z [hi + 1] - med);
	    else Lower = hi == n;
	    if (Lower) low--; else hi++;
	}
	px [i - 1] = z [i];

	py1 [i - 1] = 0.0;
	for (j = low; j <= hi; j++)
	    py1 [i - 1] += x [j];
	py1 [i - 1] /= k;

	p->SetData (k, x + low);
	TEstResult r = EST_OK;
	if (est == 0) {
	    r = MLEGP->Estimate (k, p);
	    for (int i = 0; i < 3; i++)
                para [i] = MLEGP->GetEstimatedParameterValue (i);
	} else {
	    // MLEsigma

	    for (j = 1; j <= k * 2 / 3; j++) {
		rx [j - 1] = XGPQF (gamma, (double) j / (k + 1));
		ry [j - 1] = (*p) [j - 1];
	    }
	    double *a = PolynomialRegression (1, k * 2 / 3, rx, ry);
    //	para [0] = gamma; para [1] = a [0]; para [2] = a [1];
	    para [2] = GPMLESigma (x + low, k, gamma, mu, a [1]);
	    para [1] = mu;
	    para [0] = gamma;
	}

	py3 [i - 1] = para [0];
	py5 [i - 1] = para [1];
	py4 [i - 1] = para [2];
	if (r == EST_OK && para [0] < 1.0) py2 [i - 1] = para [1] + para [2] / (1.0 - para [0]);
	else py2 [i - 1] = 0.0;

    }
    delete p;

    TMultivariateSample *s = new TMultivariateSample;
    s->SetData (6, n);
    s->SetColumn (0, px, "PML");
    s->SetColumn (1, py1, "emp.meansize");
    s->SetColumn (2, py2, "par.meansize");
    s->SetColumn (3, py3, "gamma");
    s->SetColumn (4, py5, "mu");
    s->SetColumn (5, py4, "sigma");

    TList l;
    l.Append (new TStringEl ("Analysis of PML (nearest neighbors) in " & GetFileName ()));
    l.Append (new TStringEl ("******************************************************"));
    if (est == 0)
	l.Append (new TStringEl ("Estimator:  " & MLEGP->Name ()));
    else {
	l.Append (new TStringEl ("Estimator:  MLEScale(GP) with fixed"));
	l.Append (new TStringEl ("            gamma = " & form (30, "%g", gamma) & ", mu = " & form (30, "%g", mu)));
    }
    l.Append (new TStringEl ("Neighbors:  " & form (30, "%d", k)));
    s->SetDescription (&l);


    s->SetFileName (filename);
    s->SaveData (0);
    SampleList.InsertSample (s);

    delete [] x; delete [] z;
    delete [] py1; delete [] py2; delete [] py3;
    delete [] py4; delete [] py5;
    delete [] rx; delete [] ry;
}


/*******************************************************************/
/*		 	PML: dialog routines			   */
/*******************************************************************/

#ifndef TERMINAL

void TMultivariateSample::PMLSortPML (TWindow *parent) {
    if (Dimension != 2) {
	ErrorBox (parent, "The format of the active data set is not correct.\nRequired:  PML  Claim");
	return;
    }
    TWriteFileField e (IDD_Name, GlobalData.PMLOrderName);
    TDialog d (IDD_ConverseSample);
    d.SetTitel ("Functional parameter transformation");
    d << e;
    if (d.Handle (parent) == IDOK) {
	GlobalData.PMLOrderName = e.ReadBuf ();
	PMLSortPML (GlobalData.PMLOrderName);
    }
}


void TMultivariateSample::PMLAddDegrees (TWindow *parent) {
    if (Dimension != 5) {
	ErrorBox (parent, "The format of the active data set is not correct.\nRequired:  Group  From  To  PML  Claim");
	return;
    }
    TWriteFileField e (IDD_Name, GlobalData.PMLDegreeName);
    TDialog d (IDD_ConverseSample);
    d.SetTitel ("Add degrees");
    d.SetHelpNumber (IDM_AddDegrees);
    d << e;
    unsigned ch = d.Handle (parent);
    if (ch == IDOK) {
        GlobalData.PMLDegreeName = e.ReadBuf ();
        PMLAddDegrees (GlobalData.PMLDegreeName);
    }
}

class TPMLPrioritiesDialog : public TDialog {
    typedef TDialog inherited;
    public:
	TPMLPrioritiesDialog   (double *Partition, double *pmls, int size0, double GroupCount, String filename,
                              double fromvalue, double tovalue, double stepvalue);
        TDoubleEditor PartitionEdit;
        TWriteFileField NameEdit;
    protected:
        virtual void NotifyActionEvent (TDialogMember *);	// mouse click on button
        TCalcDoubleField from, to, step, blocksize;
	TAlterField a;
        double *pml;
        int size;
};


TPMLPrioritiesDialog::TPMLPrioritiesDialog (double *Partition, double *pmls, int size0, double GroupCount, String filename,
					    double fromvalue, double tovalue, double stepvalue)
 : TDialog (IDD_PMLPriorities),
   PartitionEdit (IDD_Partition, Partition, GroupCount),
   NameEdit (IDD_Name, filename),
   from (IDD_From, fromvalue),
   to (IDD_To, tovalue),
   step (IDD_Step, stepvalue),
   a (GlobalData.PMLSelector, 2, IDD_UseStep, IDD_UseBlock),
   blocksize (IDD_BlockSize, GlobalData.PMLBlockSize) {
    *this << PartitionEdit << NameEdit << from << to << step << a << blocksize;
    pml = pmls;
    size = size0;
}

void TPMLPrioritiesDialog::NotifyActionEvent (TDialogMember *m) {
    if (m->GetID () == IDD_Generate) {
	GlobalData.PMLSelector = a.Selector ();
        switch (GlobalData.PMLSelector) {
	    case 0:
		if (from.IsCorrect () && to.IsCorrect () && step.IsCorrect ()) {
                    double fromvalue = from.GetValue (),
                           tovalue = to.GetValue (),
                           stepvalue = step.GetValue ();
                    if (stepvalue) {
                        double d = (tovalue - fromvalue) / stepvalue + 2;
                        if (d > 0.0 && d < MAXSAMPLESIZE) {
                            int Count = (int) d;
                            double *v = new double [Count], *q = v;
                            for (int i = 0; i < Count; i++)
                                *q++ = fromvalue + i * stepvalue;
                            PartitionEdit.SetValues (v, Count);
                            delete [] v;
			}
                    }
		    WarningBox (this, "Cannot generate partition");
		}
                break;
	    case 1:
                if (blocksize.IsCorrect ()) {
                    GlobalData.PMLBlockSize = (int) blocksize.GetValue ();
                    int Count = size / GlobalData.PMLBlockSize + 2, index = GlobalData.PMLBlockSize;
		    double *v = new double [Count + 1], *q = v + 1;
		    v [0] = 0.0;
		    Count = 1;
                    while (index < size) {
                        *q++ = pml [index];
                        index += GlobalData.PMLBlockSize;
                        Count++;
                    }
                    if (index != size - 1) {
                        *q = pml [size - 1];
                        Count++;
                    }
		    PartitionEdit.SetValues (v, Count);
		    delete [] v;
                }
		break;
        }
    } else inherited::NotifyActionEvent (m);
}

void TMultivariateSample::PMLAddPriorities (TWindow *parent) {
    static double Tab [] = { 100, 200, 300, 400, 500, 1000, 2000, 3000, 4000, 5000, 10000 };
    if (!GlobalData.PMLPartition) {
	GlobalData.PMLGroupCount = 11;
	GlobalData.PMLPartition = new double [11];
	for (int i = 0; i < 11; i++)
	    GlobalData.PMLPartition [i] = Tab [i];
    }

    TPMLPrioritiesDialog d (GlobalData.PMLPartition, DataPtr [0], DataSize, GlobalData.PMLGroupCount, GlobalData.PMLName,1, 100, 10);
    d.SetTitel ("Add Priorities");
    d.SetHelpNumber (IDM_AddPriorities);
    if (d.Handle (parent) == IDOK) {
	GlobalData.PMLGroupCount = d.PartitionEdit.GetCount ();
        GlobalData.PMLPartition = d.PartitionEdit.GetValues ();
        qsort (GlobalData.PMLPartition, GlobalData.PMLGroupCount, sizeof (double), realcomp);
        GlobalData.PMLName = d.NameEdit.ReadBuf ();
        PMLAddPriorities (GlobalData.PMLName, GlobalData.PMLGroupCount, GlobalData.PMLPartition);
    }
}

void TMultivariateSample::PMLDistributeDataSet (TWindow *parent) {
    if (Dimension < 6) {
	ErrorBox (parent, "The format of the active data set is not correct.\n"
				"Required:  Group  From  To  PML  Claim  Degree");
	return;
    }
    TAlterField a1 (GlobalData.PMLPlotOption, 3, IDD_Nothing, IDD_Qf, IDD_Df),
                a2 (GlobalData.PMLEstimator, 4, IDD_HillEst, IDD_MLEGP, IDD_Moment, IDD_MLEExponential),
                a3 (GlobalData.PMLExtractOption, 2, IDD_ClaimSize, IDD_Degree);
    TCalcDoubleField e1 (IDD_Threshold, GlobalData.PMLThreshold);
    TWriteFileField e2 (IDD_FileName, GlobalData.PMLSegmentFilename),
		    e3 (IDD_NamePara, GlobalData.PMLParaFilename);
    TDialog d (IDD_PMLAnalyse);
    d << a1 << a2 << a3 << e1 << e2 << e3;
    if (d.Handle (parent) == IDOK) {
        GlobalData.PMLPlotOption = a1.Selector ();
        GlobalData.PMLEstimator = a2.Selector ();
        GlobalData.PMLExtractOption = a3.Selector ();
        GlobalData.PMLThreshold = e1.GetValue ();
        GlobalData.PMLSegmentFilename = e2.ReadBuf ();
        GlobalData.PMLParaFilename = e3.ReadBuf ();
	PMLDistributeDataSet (GlobalData.PMLSegmentFilename, GlobalData.PMLParaFilename,
			      GlobalData.PMLPlotOption, !GlobalData.PMLExtractOption,
			      GlobalData.PMLThreshold, GlobalData.PMLEstimator);
    }
}

void TMultivariateSample::PMLFunctionalParameterTrafo (TWindow *parent) {
    if (Dimension < 6) {
	ErrorBox (parent, "The format of the active data set is not correct.\n"
				"Required:  Group  From  To  PML  Claim  Degree");
	return;
    }
    TWriteFileField e (IDD_Name, GlobalData.PMLParaFilename);
    TDialog d (IDD_ConverseSample);
    d.SetTitel ("Functional parameter transformation");
    d << e;
    if (d.Handle (parent) == IDOK) {
        GlobalData.PMLParaFilename = e.ReadBuf ();
	PMLFunctionalParameterTrafo (GlobalData.PMLName);
    }
}

void TMultivariateSample::PMLNearestNeighbors (TWindow *parent) {
    if (Dimension != 2) {
	ErrorBox (parent, "The format of the active data set is not correct.\nRequired:  PML  Claim");
	return;
    }
    TIntField e1 (IDD_Neighbors, GlobalData.PMLNeighbors, 1, INT_MAX);
    TAlterField a (GlobalData.PMLNeighborsEst, 2, IDD_MLEGP, IDD_MLESigma);
    TCalcDoubleField e2 (IDD_Gamma, GlobalData.PMLNeighborsGamma),
		     e3 (IDD_Mu, GlobalData.PMLNeighborsMu, DBL_MIN, DBL_MAX);
    TWriteFileField e4 (IDD_Name, GlobalData.PMLNeighborsName);
    TDialog d (IDD_PMLNeighbor);
    d << e1 << e2 << e3 << e4 << a;
    if (d.Handle (parent) == IDOK) {
	GlobalData.PMLNeighbors = e1.GetValue ();
	GlobalData.PMLNeighborsMu = e3.GetValue ();
	GlobalData.PMLNeighborsGamma = e2.GetValue ();
	GlobalData.PMLNeighborsEst = a.Selector ();
	GlobalData.PMLNeighborsName = e4.ReadBuf ();
	PMLNearestNeighbors (GlobalData.PMLNeighborsName, GlobalData.PMLNeighbors,
		GlobalData.PMLNeighborsEst, GlobalData.PMLNeighborsGamma, GlobalData.PMLNeighborsMu);
    }
}

#endif
