/*  point.hpp

    Class definition for points and rectangles with real coordinates

    This file is part of Xtremes for Windows,
    Copyright (C) 1995 Xtremes Group, Siegen.
*/

#ifndef POINT_HPP
#define POINT_HPP

#include "boolean.hpp"

struct TDblPoint {
    double x, y;
    TDblPoint () {}
    TDblPoint (double x0, double y0) { x = x0; y = y0; }
    friend TDblPoint operator + (TDblPoint, TDblPoint);
    friend TDblPoint operator - (TDblPoint, TDblPoint);
    friend BOOL operator == (TDblPoint, TDblPoint);
    friend BOOL operator != (TDblPoint, TDblPoint);
};

struct TDblRectangle {
    double x0,y0, x1,y1;
    TDblRectangle () {}
    TDblRectangle (double x0s, double y0s, double x1s, double y1s);
    TDblRectangle (TDblPoint p, TDblPoint q);
    TDblRectangle operator * (double);
    BOOL Within (TDblPoint);
};


class TPoint3 {
    public:
	TPoint3 (double, double, double);
	TPoint3 () { x = y = z = 0.0; }

	friend TPoint3 operator * (double, TPoint3);
	friend TPoint3 operator + (TPoint3, TPoint3);
	friend TPoint3 operator - (TPoint3, TPoint3);
	friend double operator * (TPoint3, TPoint3);        // scalar product
	friend TPoint3 cross (TPoint3, TPoint3);            // cross product
        friend double dist (TPoint3, TPoint3);
	friend double norm (TPoint3);
        friend BOOL operator < (TPoint3, TPoint3);	    // checks all components
	double x, y, z;
        double & operator [] (int);
};

class TMatrix3 {
    public:
        TMatrix3 ();
        TPoint3 operator * (TPoint3);
	TMatrix3 operator * (TMatrix3);
        friend TMatrix3 RotateX (double);
        friend TMatrix3 RotateY (double);
        friend TMatrix3 RotateZ (double);
        double p [3][3];
};



#endif

