/*  refcount.cpp

    Generic class supporting reference counting and automatic deletion.

    This file is part of Xtremes for Windows,
    Copyright (C) 1991, 2000 Xtremes Group, Siegen.
*/

#ifndef NDEBUG
#include <stdio.h>
#include <stdlib.h>
#endif

#include "refc.hpp"

TDataObject::TDataObject () {
    RefCount = 1;
}

TDataObject::~TDataObject () {
}

TDataObject * TDataObject::Duplicate () {
    #ifndef NDEBUG
	fputs ("TDataObject::Duplicate called", stderr);
	abort ();
    #endif
    return NULL;
}


TDataRefHandler::TDataRefHandler () {
    RefObject = NULL;
}

TDataRefHandler::TDataRefHandler (TDataObject *p) {
    RefObject = p;
}

TDataRefHandler::TDataRefHandler (const TDataRefHandler &r) {
    puts ("TDataRefHandler::TDataRefHandler (const TDataRefHandler &)");
    RefObject = r.RefObject;
    RefObject->RefCount++;
    printf ("RefCount = %d\n", RefObject->RefCount);
}

TDataRefHandler::~TDataRefHandler () {
    puts ("TDataRefHandler::~TDataRefHandler ()");
    printf ("RefCount = %d\n", RefObject->RefCount);
    if (RefObject && !--RefObject->RefCount) delete RefObject;
}

void TDataRefHandler::operator = (TDataRefHandler r) {
    if (RefObject && !--RefObject->RefCount) delete RefObject;
    RefObject = r.RefObject;
    if (RefObject) RefObject->RefCount++;
}

TDataObject *TDataRefHandler::GetDataObject () {
    return RefObject ? RefObject : NULL;
}


TDataFrame::TDataFrame (TDataObject *p)
 : RefHandler (p) {
}

TDataObject *TDataFrame::GetDataObject () {
    return RefHandler.GetDataObject ();
}

void TDataFrame::MakeUnique () {
    TDataObject *p = RefHandler.GetDataObject ();
    if (p && p->RefCount != 1) {
	p->RefCount--;
	RefHandler = TDataRefHandler (p->Duplicate ());
    }
}

void TDataFrame::operator = (TDataFrame f) {
    RefHandler = f.RefHandler;
}

