/*  risk.hpp

    Simulation of risk process.

    This file is part of Xtremes for Windows,
    Copyright 1991, 1996 Xtremes Group,	Siegen.
*/

#ifndef	RISK_HPP
#define	RISK_HPP

#include "sample.hpp"

#ifndef	TERMINAL
#include "wintypes.hpp"
#include "xplfunc.hpp"

/*******************************************************************/
/*			Path of	Risk Process			   */
/*******************************************************************/

class TRiskProcess : public TXPLFunction {
    typedef TXPLFunction inherited;
    public:
	TRiskProcess ();
	virtual	String Name ();
    protected:
	virtual	void Plot (TCoordinateWindow *, BOOL BackColor = FALSE, TColor BackColorRef = TColor (255, 255, 255));
	virtual	BOOL IncParameter ();
	virtual	BOOL IncParameterMany ();

	virtual	BOOL EditParameter (TWindow *parent) = 0;
	void DrawCurveTo (TCoordinateWindow *w, double	t0, double t1, double delta);
	double GetU (double);
	void DrawNext (double &t, double &u);
        virtual String ClaimSizeName () = 0;
	void Entry (double, double);
	virtual	double DrawEvent () = 0;
	virtual	double GetExpectedEvent	() = 0;
	double lambda, rho, beta, u, T,	CurrT, polalpha, polsigma, meanvalue;
	BOOL PolyaLundbergSim;
	String fn;

        void GeneratePoint ();
        void TransferData ();	// to TXPLFunction
//	void GenerateOne ();
//	void GenerateMany (int n);
	int DataSize, ReqSize;
	double *t, *us,	S;
	int n, Allocated;
	BOOL First, Done;

};

class TExpRiskProcess :	public TRiskProcess {
    public:
	TExpRiskProcess	();
    protected:
	virtual	BOOL EditParameter (TWindow *parent);
	virtual	double DrawEvent ();
	virtual	double GetExpectedEvent	();
	virtual String ClaimSizeName ();
    private:
	double mu, sigma;
};

class TParetoRiskProcess : public TRiskProcess {
    public:
	TParetoRiskProcess ();
    protected:
	virtual	BOOL EditParameter (TWindow *parent);
	virtual	double DrawEvent ();
	virtual	double GetExpectedEvent	();
	virtual String ClaimSizeName ();
    private:
	double alpha, mu, sigma;
};

class TGPRiskProcess : public TRiskProcess {
    public:
	TGPRiskProcess ();
    protected:
	virtual	BOOL EditParameter (TWindow *parent);
	virtual	double DrawEvent ();
	virtual	double GetExpectedEvent	();
	virtual String ClaimSizeName ();
    private:
	double gamma, sigma, mu;
};

#endif

/*******************************************************************/
/*		Simulation of ruin propability			   */
/*******************************************************************/

class TRuinSimulation {
    public:
	TRuinSimulation	(int NrSim, double rho,	double beta, double lambda, double polalpha, double polsigma, BOOL PolyLundbergSim, double maxT);
	virtual	~TRuinSimulation ();
	void SetU (double umin,	double umax, int n);
	TDblPoint GetValue (int	i);
#ifndef	TERMINAL
	void Simulation	(TWindow *parent);
#endif
	void SingleSimulation ();
	virtual	double DrawEvent () = 0;
	virtual	double ExpectedEvent ()	= 0;
	double GetTime (int i)		{ return t [i];	}
    protected:
	void CheckRuin ();
	double *u, *t, *p;
	int RuinPtr, un;
	int NrSim;
	BOOL PolyaLundbergSim;
	double rho, beta, lambda, polalpha, polsigma, meanvalue, maxT, s, tc;
};

class TExponentialRuinSimulation : public TRuinSimulation {
    public:
	TExponentialRuinSimulation (double mu, double sigma, int NrSim, double rho, double beta, double lambda, double polalpha, double polsigma, BOOL PolyLundbergSim, double maxT);
	virtual	double DrawEvent ();
	virtual	double ExpectedEvent ();
    private:
	double mu, sigma;
};

class TParetoRuinSimulation : public TRuinSimulation {
    public:
	TParetoRuinSimulation (double alpha, double mu,	double sigma, int NrSim, double	rho, double beta, double lambda, double polalpha, double polsigma, BOOL PolyLundbergSim, double	maxT);
	virtual	double DrawEvent ();
	virtual	double ExpectedEvent ();
    private:
	double alpha, sigma, mu;
};

class TGPRuinSimulation	: public TRuinSimulation {
    public:
	TGPRuinSimulation (double gamma, double	mu, double sigma, int NrSim, double rho, double	beta, double lambda, double polalpha, double polsigma, BOOL PolyLundbergSim, double maxT);
	virtual	double DrawEvent ();
	virtual	double ExpectedEvent ();
    private:
	double gamma, sigma, mu;
};

#ifndef	TERMINAL

void PerformExpRuinSimulation (TWindow *parent);
void PerformParetoRuinSimulation (TWindow *parent);
void PerformGPRuinSimulation (TWindow *parent);

void PerformExpRuinTimeSimulation(TWindow *parent);
void PerformParetoRuinTimeSimulation(TWindow *parent);
void PerformGPRuinTimeSimulation(TWindow *parent);

#endif

double SimulateSingleGPRuinTime	(double	*p);

#endif
