/*  sampbase.cpp

    Base class for all sample types.

    This file is part of Xtremes for Windows,
    Copyright (C) 1991, 2000 Xtremes Group, Siegen.
*/

#include "sampbase.hpp"
#include "samplist.hpp"

#ifndef TERMINAL
#include "compdlg.hpp"
#include "privmsg.hpp"
#include "samplwin.hpp"
#include "visual.hpp"
#include "wintypes.hpp"

#include "resource.h"
#endif

char * SampleHeaders [] = {
		"Xtremes Univariate Data",
		"Xtremes Time Series",
		"Xtremes Grouped Data",
		"Xtremes Censored Data",
		"Xtremes Discrete Data",
		"Xtremes Multivariate Data" };

const int HeaderCount = sizeof (SampleHeaders) / sizeof (char *);

int MAXSAMPLESIZE = ACADEMICSIZE;

TSample::TSample () {
    Deleted = FALSE;
}

TSample::~TSample () {}

#ifndef TERMINAL
BOOL TSample::Create (TWindow *parent) {
    unsigned ch = EditParameter (parent);
    if (ch == IDOK) {
	GetMemory ();
	if (CreateData (parent)) {
	    CreateDescription ();
	    SaveData (parent);
	    SortData ();
	    SampleList.InsertSample (this);
	    return TRUE;
	} else return FALSE;
    }
    if (ch == IDD_Visualize) {
        CreateDescription ();
	CreateInteractive (parent);
	return TRUE;
    }
    return FALSE;
}

BOOL TSample::CreateMemory (TWindow *parent) {
    if (!GetMemory ()) return FALSE;
    CreateData (parent);
    SortData ();
    return TRUE;
}

void TSample::CreateInteractive (TWindow *parent) {
    ErrorBox (parent, "This data type does not support interactive creation");
}

#endif

void TSample::CreateDescription () {
}

String TSample::Name () {
    String t;
    switch (GetSampleType ()) {
	case UNIVARIATE_SAMPLE: t = "Univariate "; break;
	case TIMESERIES_SAMPLE: t = "Time Series "; break;
	case GROUPED_SAMPLE: t = "Grouped "; break;
	case CENSORED_SAMPLE: t = "Censored "; break;
	case FREQUENCY_SAMPLE: t = "Discrete "; break;
	case MULTIVARIATE_SAMPLE: t = "Multivariate "; break;
	default: t = "Unknown "; break;
    }
    return CutPath (filename) & form (30, " %-17s", (const char *) t) & name;
}

void TSample::SetContamination (double d) {
}

TList *TSample::GetDescription () {
    return &Description;
}

void TSample::SetDescription (TList *s) {
    Description.DeleteAll (TRUE);
    for (TListEl *p = s->GetFirst (); p; p = s->GetNext ())
        Description.Append (p);
}

void TSample::AddDescriptionLine (String s) {
    Description.Append (new TStringEl (s));
}

double TSample::SampleDf (double x) {
    return 0.0;
}

double TSample::SampleQf (double x) {
    return 0.0;
}

double TSample::SampleDen (double x, double beta, int KNo) {
    return 0.0;
}

double TSample::SampleHazard (double x, double beta) {
    return 0.0;
}

double TSample::SampleMean () {
    return 0.0;
}

double TSample::SampleVariance () {
    return 0.0;
}

double TSample::SampleSkewness () {
    return 0.0;
}

double TSample::MeanClusterSize (int mingap, double threshold) {
    return 0.0;
}

void TSample::NotApplicableMessage (TWindow *parent) {
#ifndef TERMINAL
    ErrorBox (parent, "The selected operation is not supported by the active data set.");
#else
    puts ("The selected operation is not supported by the active data set.");
#endif
}

void TSample::Symmetrize (String filename, BOOL Mean, double val) {
    NotApplicableMessage (0);
}

void TSample::PMLDistributeDataSet (String filename, String ParaName, int, BOOL UseClaimSize, double t, int EstimatorNr) {
    NotApplicableMessage (0);
}

void TSample::PMLFunctionalParameterTrafo (String filename) {
    NotApplicableMessage (0);
}

void TSample::PMLAddDegrees (String filename) {
    NotApplicableMessage (0);
}

void TSample::PMLSortPML (String filename) {
    NotApplicableMessage (0);
}

void TSample::PMLNearestNeighbors (String filename, int k, int est, double gamma, double mu) {
    NotApplicableMessage (0);
}

void TSample::DateTransformation (String filename) {
    NotApplicableMessage (0);
}

BOOL TSample::PMLAddPriorities (String filename, int m, double *p) {
    NotApplicableMessage (0);
    return FALSE;
}

void TSample::PriceReturnTrafo (String filename, int TreatMondays) {
    NotApplicableMessage (0);
}

void TSample::OrderSample (String filename, int Column) {
    NotApplicableMessage (0);
}

void TSample::TruncateSample (String filename, double threshold, int Column) {
    NotApplicableMessage (0);
}

void TSample::CalculateHydrograph (int r0, double c, int r1, double * &x, double * &y, int &count) {
    NotApplicableMessage (0);
}

void TSample::AnalysePartialDurationSeries (double u, int k, String fn, double tmin, double tmax, int tpoints) {
    NotApplicableMessage (0);
}

#ifndef TERMINAL
void TSample::Edit (TMDIFrame *MDIFrame) {
    TDataEditor *w = new TDataEditor (MDIFrame, "", "Data Editor:");
    w->LoadContents (GetFileName ());
}

void TSample::PlotHistogram (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::PlotSampleDF (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::PlotSampleQF (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::PlotSampleACF (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::PlotSampleAutoCorrelation (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::PlotSampleDepFunction (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::PlotDTail1 (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::PlotDTail2 (TWindow *parent){
    NotApplicableMessage (parent);
}

void TSample::PlotDTail3 (TWindow *parent){
    NotApplicableMessage (parent);
}

void TSample::PlotDTail4 (TWindow *parent){
    NotApplicableMessage (parent);
}

void TSample::PlotCTail1 (TWindow *parent){
    NotApplicableMessage (parent);
}

void TSample::PlotCTail2 (TWindow *parent){
    NotApplicableMessage (parent);
}

void TSample::PlotCTail3 (TWindow *parent){
    NotApplicableMessage (parent);
}

void TSample::PlotCTail4 (TWindow *parent){
    NotApplicableMessage (parent);
}

void TSample::PlotCParaC (TWindow *parent){
    NotApplicableMessage (parent);
}

void TSample::PlotDParaC (TWindow *parent){
    NotApplicableMessage (parent);
}

void TSample::PlotChi (TWindow *parent){
    NotApplicableMessage (parent);
}

void TSample::PlotChiBarE (TWindow *parent){
    NotApplicableMessage (parent);
}

void TSample::PlotChiBarP (TWindow *parent){
    NotApplicableMessage (parent);
}

void TSample::PlotTimeTailDep(TWindow *parent){
	NotApplicableMessage (parent);
}

void TSample::PlotSampleVar (TWindow *parent, int, int) {
    NotApplicableMessage (parent);
}

void TSample::PlotPath (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::PlotKernelDensity (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::PlotBoundedKernelDensity (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::PlotMeanExcessFunction (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::PlotMedianExcessFunction (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::PlotHazardFunction (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::BoxPlot (TMDIFrame *MDIFrame) {
    NotApplicableMessage (MDIFrame);
}

void TSample::ScatterPlot2D (TMDIFrame *MDIFrame) {
    NotApplicableMessage (MDIFrame);
}

void TSample::ScatterPlot3D (TMDIFrame *MDIFrame) {
    ScatterPlot2D (MDIFrame);
}

void TSample::PlotFunction (TWindow *parent, TFunction *f, String WindowName) {
    if (f->EditParameter (parent))
	PlotWindowManager.GetOpenWindow (WindowName)->InsertFunction (f);
    else delete f;
}

void TSample::TransformSample (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::ConverseSample (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::LogScaleSample (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::TruncateSample (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::ExtractMaxima (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::SaveBlocksSum (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::SaveMovingMaxima (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::SaveMovingSum (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::PriceReturnTrafo (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::PMLDistributeDataSet (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::PMLNearestNeighbors (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::PMLAddDegrees (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::PMLSortPML (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::DeclusterSample (TWindow *parent, int, double) {
    NotApplicableMessage (parent);
}

void TSample::UserTransformation (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::FillMissing (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::CumulateSample (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::OrderSample (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::SelectComponent (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::ShowFunctionalParameters (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::GaussianTwoSampleTest (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::WilcoxonTest (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::PMLAddPriorities (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::PMLFunctionalParameterTrafo (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::Symmetrize (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::DateTransformation (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::CalculateHydrograph (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::AnalysePartialDurationSeries (TWindow *parent) {
    NotApplicableMessage (parent);
}

void TSample::FrequenciesToRelative (TWindow *parent) {
    NotApplicableMessage (parent);
}

#endif

BOOL TSample::SaveData (TWindow *parent) {
    FILE *f = fopen (filename, "wt");
    if (!f) {
#ifndef TERMINAL
	ErrorBox (parent, "Connot open file: " & filename);
#else
	puts ("Cannot open file");
#endif
	return FALSE;
    }
    BOOL Success = WriteHeader (f) && WriteData (f);
    fclose (f);
    return Success;
}

BOOL TSample::LoadData (TTextFileReader &f) {
    BOOL r = ReadHeader (f);
    f.SetCommentMode (TRUE);
    if (r) r = ReadData (f);
    return r;
}

BOOL TSample::WriteHeader (FILE *f) {
    fputs (GetHeader (), f);
    fputc ('\n', f);
    fprintf (f, "Type: %s\n", (const char *) name);
    fputs ("\\begin(description)\n", f);
    for (TListEl *p = Description.GetFirst (); p; p = Description.GetNext ()) {
	fputs (p->Name (), f);
	fputc ('\n', f);
    }
    fputs ("\\end(description)\n", f);
    return TRUE;
}

String TSample::GetReadMessage () {
    return ReadMessage;
}

BOOL TSample::ReadHeader (TTextFileReader &f) {
    String t = f.GetString (TRUE);
    if (t == String ("type")) {
        if (f.GetNextChar () == ':') name = f.GetLine ();
    } else {
	ReadMessage = "'Type:' expected";
	return FALSE;
    }
    int c = f.GetNextChar ();
    if (c == '\\') return ReadDescription (f);
    else f.UngetChar (c);
    return TRUE;
}

BOOL TSample::ReadDescription (TTextFileReader &f) {
    if (f.GetLine () != String ("begin(description)")) return FALSE;
    String t = f.GetLine ();
    while (t != String ("\\end(description)") && !f.IsError ()) {
        Description.Append (new TStringEl (t));
        t = f.GetLine ();
    }
    if (f.IsError ()) {
	ReadMessage = "Missing '\\end description' in data set";
	return FALSE;
    }
    return TRUE;
}

BOOL TSample::ReadSampleSize (TTextFileReader &f, int &EntrySize) {
    int c = f.GetNextChar ();
    f.UngetChar (c);
    if (c == 'D' ||c == 'S') {
	String t = f.GetString (TRUE);
	if ( (t == String ("samplesize")) || (t == String ("datasize")) ||
	     ((t == String ("sample") || t == String ("data")) && f.GetString (TRUE) == String ("size"))) {
	    c = f.GetNextChar ();
	    if (c != ':') f.UngetChar (c);
	    EntrySize = f.GetInt ();
	    if (f.IsError ()) {
		ReadMessage = "Invalid sample size specified";
		return FALSE;
	    }
	} else {
	    ReadMessage = "'Samplesize:' expected";
	    return FALSE;
        }
    } else EntrySize = -1;
    return TRUE;
}

void TSample::SetFileName (String s) {
    filename = s;
}

void TSample::SetName (String t) {
    name = t;
}

String TSample::CutPath (String d) {
    const char *t, *u, *s = d;
#ifdef __unix__
    t= strrchr (s, '/');
    return t ? t+1 : s;
#else
    t = strrchr (s, '\\');
    u = t ? t + 1 : s;
    t = strrchr (u, ':');
    return t ? t + 1 : u;
#endif
}

String TSample::ModifyName (char ch) {
    String d = CutPath (filename);
#if defined(__WIN32__) || defined(__unix__)
    // use long filenames
    return (String) ch & d;
#else
    static char s [15];
    const char *t = d;
    int i, j;
    for (i = 0; i < 7 && t [i] != '.'; i++)
	s [i + 1] = t [i];
    for (j = i; t [j] != '.'; j++);
    while (j < 12 && t [j] != ' ') s [++i] = t [j++];
    s [0] = ch;
    s [++i] = '\0';
    return s;
#endif
}

#ifndef TERMINAL
TSample *TSample::ConvertTo (TSampleType st, TWindow *parent) {
    ErrorBox (parent, "The selected conversion of data types is not supported by Xtremes.");
    return NULL;
}
#endif

void TSample::SortData () {
}

void TSample::SetSampleSize (int s, int k) {
}

#ifndef TERMINAL

TSampleListWindow::TSampleListWindow (TSample *s0, TMDIFrame *MDIFrame, String Titel)
 : inherited (MDIFrame, Titel) {
    s = s0;
    SetIcon (IDI_TextLister);
}

void TSampleListWindow::NotifyMenuEvent (int id) {
    switch (id) {
	case IDM_KernelDensity:
	    s->PlotBoundedKernelDensity (this);
	    break;
	case IDM_SampleHazardFunction:
	    s->PlotHazardFunction (this);
	    break;
	case IDM_Histogram:
	    s->PlotHistogram (this);
	    break;
	case IDM_EmpiricalDF:
	    s->PlotSampleDF (this);
	    break;
	case IDM_EmpiricalQF:
	    s->PlotSampleQF (this);
	    break;
	case IDM_EmpiricalMeanEF:
	    s->PlotMeanExcessFunction (this);
	    break;
	case IDM_EmpiricalMedianEF:
	    s->PlotMedianExcessFunction (this);
	    break;
	case IDM_SampleACF:
	    s->PlotSampleACF (this);
	    break;
	case IDM_FunctionalParameters:
	    s->ShowFunctionalParameters (this);
	    break;
	case IDM_ChangeSign:
	    s->ConverseSample (this);
	    break;
	case IDM_AffineTrafo:
	    s->TransformSample (this);
	    break;
	case IDM_Extremes:
	    s->TruncateSample (this);
	    break;
	case IDM_Maxima:
	    s->ExtractMaxima (this);
	    break;
	case IDM_SaveBlocksSum:
	    s->SaveBlocksSum (this);
	    break;
	case IDM_SaveMovingSums:
	    s->SaveMovingSum (this);
	    break;
	case IDM_SaveMovingMaxima:
	    s->SaveMovingMaxima (this);
	    break;
	case IDM_UserTrafo:
	    s->UserTransformation (this);
	    break;
	case IDM_Order:
	    s->OrderSample (this);
	    break;
	case IDM_CumulateData:
	    s->CumulateSample (this);
	    break;
	case IDM_TimeSeriesData:
	    s->ConvertTo (TIMESERIES_SAMPLE, this);
	    break;
	case IDM_GroupData:
	    s->ConvertTo (GROUPED_SAMPLE, this);
	    break;
	case IDM_FillMissing:
	    s->FillMissing (this);
	    break;
	case IDM_PriceReturnTrafo:
	    s->PriceReturnTrafo (this);
	    break;
	case IDM_ShowScatterPlot2D:
	    s->ScatterPlot2D (PlotWindowManager.GetFramePtr ());
	    break;
	case IDM_ScatPlot3D:
	    s->ScatterPlot3D (PlotWindowManager.GetFramePtr ());
            break;
	case IDM_SelectComponent:
	    s->SelectComponent (this);
            break;
	default:
	    inherited::NotifyMenuEvent (id);
	    break;
    }
}

void TSampleListWindow::NotifyEvent (int ev) {
    switch (ev) {
	case M_VALIDATE_FUNCTIONS:
	    if (s->IsDeleted ()) s = NULL;
	    break;
	case M_VALIDATE_WINDOW :
	    if (!s) DestroyWindow ();
	    break;
    }
    inherited::NotifyEvent (ev);
}

#endif

