/*  sample.cpp

    Class definition for sample	types.

    This file is part of Xtremes for Windows,
    Copyright (C) 1991, 1999 Xtremes Group Siegen.
*/

#include <assert.h>
#include <math.h>
#include <stdlib.h>
#include <float.h>
#include <limits.h>

#include "sample.hpp"
#include "samplist.hpp"
#include "stddef.hpp"
#include "mathfunc.hpp"

#ifndef	TERMINAL

#include "boxplot.hpp"
#include "censor.hpp"
#include "display.hpp"
#include "ufodlg.hpp"
#include "edit.hpp"
#include "empiric.hpp"
#include "filedlg.hpp"
#include "hazard.hpp"
#include "stddlg.hpp"
#include "kernel.hpp"
#include "global.hpp"
#include "samplwin.hpp"
#include "usrtrafo.hpp"
#include "visual.hpp"
#include "wintypes.hpp"
#include "scat3d.hpp"
#include "scatter.hpp"
#include "fillmiss.hpp"

#include "resource.h"
#include "privmsg.hpp"

#endif

/*********************************************************************
			Univariate data	sets
*********************************************************************/

#ifndef	TERMINAL

class TUnivariateListWindow : public TSampleListWindow {
    typedef TSampleListWindow inherited;
    public:
	TUnivariateListWindow (TMDIFrame *, TUnivariateBaseSample *p);
    protected:
	virtual void NotifyMenuEvent (int id);
	virtual	int GetSubMenuPos ();
	virtual	String GetLine (int i);
};

TUnivariateListWindow::TUnivariateListWindow (TMDIFrame *MDIFrame, TUnivariateBaseSample *p)
 : inherited (p, MDIFrame, p->GetFileName ()) {
    SetHeight (p->GetDataSize ());
    SetWidth (25);
//    s =	p;
    SetIcon (IDI_TextLister);
}

int TUnivariateListWindow::GetSubMenuPos () {
    return 3;
}

String TUnivariateListWindow::GetLine (int i) {
    if (i >= s->GetDataSize ())	return "";
    return form	(30, "%4d.   %g", i + 1, ((TUnivariateSample *) s)->GetRawData (i));
}

void TUnivariateListWindow::NotifyMenuEvent (int id) {
    switch (id) {
	case IDM_CensorData : {
	    TSample *q = new TCensoredSample;
	    ((TCensoredSample *) q)->SetSample ((TUnivariateBaseSample *) s);
	    if (!q->Create (this)) delete q; }
	    break;
	default: inherited::NotifyMenuEvent (id);
    }
}

#endif

TUnivariateBaseSample::TUnivariateBaseSample ()	{
    DataPtr = RawDataPtr = NULL;
    DataSize = 0;
    BlockSize =	1;
}

TUnivariateBaseSample::~TUnivariateBaseSample () {
    if (DataPtr) delete	[] DataPtr;
    if (RawDataPtr) delete [] RawDataPtr;
    DataSize = 0;
    DataPtr = NULL;
}

BOOL TUnivariateBaseSample::GetMemory () {
    if (DataPtr) delete	[] DataPtr;
    if (RawDataPtr) delete [] RawDataPtr;
    DataPtr = new double [DataSize];
    if (!DataPtr) return FALSE;
    RawDataPtr = new double [DataSize];
    if (!RawDataPtr) {
	delete [] DataPtr;
        DataPtr	= NULL;
	return FALSE;
    }
    return TRUE;
}

int TUnivariateBaseSample::GetDimension () {
    return 1;
}

int TUnivariateBaseSample::GetDataSize () {
    return DataSize;
}

double TUnivariateBaseSample::GetData (int i, int j) {
    assert (0 <= i && i < DataSize);
    assert (RawDataPtr != NULL);
    return RawDataPtr [i];
}

String TUnivariateBaseSample::GetColumnName (int j) {
    return "N.N.";
}

double TUnivariateBaseSample::operator [] (int index) {
    assert (0 <= index && index	< DataSize);
    assert (DataPtr != NULL);
    return DataPtr [index];
}



BOOL TUnivariateBaseSample::WriteData (FILE *f)	{
    fprintf (f,	"Samplesize: %d\n", DataSize);
    for	(int i = 0; i <	DataSize; i++)
	fprintf	(f, "%g\n", RawDataPtr [i]);
    return TRUE;
}

BOOL TUnivariateBaseSample::Append (double v) {
    if (DataSize >= AllocatedSize) {
	AllocatedSize += 1000;
	if (AllocatedSize > MAXSAMPLESIZE) return FALSE;
	RawDataPtr = (double *)	realloc	(RawDataPtr, AllocatedSize * sizeof (double));
	if (!RawDataPtr) return	FALSE;
    }
    RawDataPtr [DataSize++] = v;
    return TRUE;
}

BOOL TUnivariateBaseSample::ReadData (TTextFileReader &f) {
    int	EntrySize; BOOL	b = TRUE;
    if (!ReadSampleSize	(f, EntrySize))	return FALSE;
    DataSize = AllocatedSize = 0;
    do {
	double v = f.GetDouble ();
	if (!f.IsError ()) b = Append (v);
    } while (!f.IsError	() && b);
    if (f.GetError () == TTextFileReader::END_OF_FILE) {
	DataPtr	= new double [DataSize];
	SortData ();
	return TRUE;
    } else if (f.GetError ())
#ifndef	TERMINAL
	ReadMessage = LOADERRNUM;
    else
	ReadMessage = MAXSAMPLESIZE == ACADEMICSIZE ? ACADEMIC : INSUFFMEM;
#else 
    (void) 0 ;
#endif
    return FALSE;
}

void TUnivariateBaseSample::Sort (int l, int r)	{
    int	i = l, j = r;
    double x0 =	DataPtr	[(l + r) / 2];
    do {
	while (DataPtr [i] < x0) i++;
	while (x0 < DataPtr [j]) j--;
	if (i <= j) {
	    Swap (DataPtr [i], DataPtr [j]);
	    i++; j--;
	}
    } while (i <= j);
    if (l < j) Sort (l,	j);
    if (i < r) Sort (i,	r);
}

void TUnivariateBaseSample::SortData ()	{
    memcpy (DataPtr, RawDataPtr, sizeof	(double) * DataSize);
    qsort (DataPtr, DataSize, sizeof (double), realcomp);
//    Sort (0, DataSize	- 1);
}

//
//

TUnivariateSample::TUnivariateSample ()	{
#ifndef	TERMINAL
    for	(int i = 0; i <	MAXKERN; i++) Estimated	[i] = FALSE;
#endif
}

TSampleType TUnivariateSample::GetSampleType ()	{
    return UNIVARIATE_SAMPLE;
}

BOOL TUnivariateSample::IsEstimated (int Kernel) {
    assert (0 <= Kernel	&& Kernel < MAXKERN);
    return Estimated [Kernel];
}

double TUnivariateSample::GetBandWidth (int Kernel) {
    assert (0 <= Kernel	&& Kernel < MAXKERN);
    assert (IsEstimated	(Kernel));
    return BandWidth [Kernel];
}

void TUnivariateSample::SetBandWidth (int Kernel, double w) {
    assert (0 <= Kernel	&& Kernel < MAXKERN);
    BandWidth [Kernel] = w;
    Estimated [Kernel] = TRUE;
}

double TUnivariateSample::DKern (double z, int KNo) {
    switch (KNo) {
	case 0 : return 0.75 * (1.0 - z * z);
	case 1 : return 0.5;
	case 2 : return 0.125 * (9.0 - 15.0 * z * z);
	case 3 : // return 0.46875*(3.0 - z*z*(10.0 + 7.0*z*z));
	         return 45.0/32.0*(1-10.0/3.0*z*z+7.0/3.0*z*z*z*z);
    }
    return 0.0;
}

// Estimation of bandwidth

#define SCHRANKE 10E-10

double TUnivariateSample::bw1 (double *y, int n, int KNo, double t, double a) {
    double z, summe = 0.0;
    a *= (y[n-1] - y[0]);
    if (a < SCHRANKE) a = SCHRANKE;
    for (int k = 0; k < n; k++) {
	z = (t - y[k]);
	if (fabs (z) <= a) summe += DKern (z / a, KNo);
        if (t < -a) break;
    }
    return (summe - DKern (0.0, KNo)) / (n * a);
}

double TUnivariateSample::bw2 (double *y, int n, int KNo, double a) {
    double sum = 0.0, h;
    for (int l = 0; l < n; l++) {
	h = bw1 (y, n, KNo, y[l], a);
        if (h <= 0.0) return -MYMAXDOUBLE;
        sum += log (h);
    }
    return sum;
}

double TUnivariateSample::bw3 (double *y, int n, int KNo, double sl, double int1, double int2) {
    int b1, b2, invsl;
    double a, b, c = -MYMAXDOUBLE, betav = 0.0;
    assert (sl != 0.0);
    invsl = (int) (1.0 / sl);
    if (int1 == 0.0) b1 = 1;
    else b1 = (int) (invsl * int1);
    b2 = (int) (invsl * int2);
    for (int j = b1; j <= b2; j++) {
	a = sl * j;
	b = bw2 (y, n, KNo, a);
	if (b > c) {
	    c = b;
	    betav = a * (y[n-1] - y[0]);
	}
    }
    return betav;
}

double TUnivariateSample::OptBandW (int KNo, double sl, double int1, double int2) {
    int n = GetDataSize ();
    int n0 = n;
    if (n > 25 && n <= 100) n0 = 25;
    else if (n > 100 && n < 1000) n0 = 25 + (n - 100) / 12;
    else if (n > 1000) n0 = 100;
    assert (n0 != 0);
    int h1 = n / n0;
    int h2 = n - n0 * h1 + h1;
    int h3 = (int) (floor (Random () * h2) + 1);
    double *y = new double [n0];
    for (int i = 0; i < n0; i++)
        y[i] = DataPtr [i * h1 + h3 - 1];
    double betav = bw3 (y, n0, KNo, sl, int1, int2) * pow ((double) n0 / n, 0.2);
    delete [] y;
    return betav;
}

double TUnivariateSample::BoundDenNone (int Kernel, double t, double Bandwidth) {
    double beta = Bandwidth;
    if (Kernel == 3) beta *= 7.0 / 4.0;
    int size = GetDataSize ();
    double xmb = t - beta, xpb = t + beta;
    if (DataPtr [0] > xpb) return 0.0;
    if (DataPtr [size - 1] < xmb) return 0.0;
    int f = 0, l = size - 1, m;
    while (f < l) {
	m = (f + l) / 2;
	if (DataPtr [m] >= xmb) l = m;
	else f = m + 1;
    }
    double z, sum = 0.0;
    double einsdbeta = 1.0 / beta;
    BOOL Goon = TRUE;
    while (Goon) {
	z = (t - DataPtr [f]) * einsdbeta;
	if (z >= -1.0) {
	    sum += DKern (z, Kernel);
	    Goon = ++f < size;
	} else Goon = FALSE;
    }
    return sum / size * einsdbeta;
}

double TUnivariateSample::BoundDenLeft (int Kernel, double t, double Bandwidth, double beta0) {
    int size = GetDataSize ();
    double beta = Bandwidth;
    if (t - DataPtr [0] <= beta)
        beta = beta0 + (t - DataPtr [0]) / beta  * (beta - beta0);
    double z, sum = 0.0,
	   tzweis0 = t - DataPtr [0] - DataPtr [0],
	   einsdbeta = 1.0 / beta;
    if (Kernel == 3) einsdbeta *= 4.0 / 7.0;
    if (t < DataPtr [0]) return 0.0;
    for (int i = 0; i < size; i++) {
	z = (t - DataPtr [i]) * einsdbeta;
	if (z >= -1.0 && z <= 1.0) sum += DKern (z, Kernel);
	z = (tzweis0 + DataPtr [i]) * einsdbeta;
	if (z >= -1.0 && z <= 1.0) sum += DKern (z, Kernel);
    }
    return sum / (size * beta);
}

double TUnivariateSample::BoundDenRight (int Kernel, double t, double Bandwidth, double beta1) {
    t = -t;
    double beta = Bandwidth;
    int size = GetDataSize ();
    if (t < - DataPtr [size-1]) return 0.0;
    if (DataPtr [size-1] + t<= beta)
        beta = beta1 + (DataPtr [size-1] + t) / beta * (beta - beta1);
    double z, sum = 0.0,
	   tzweis0 = t + DataPtr [size - 1] + DataPtr [size - 1],
	   einsdbeta = 1.0 / beta;
    if (Kernel == 3) einsdbeta *= 4.0 / 7.0;
    for (int i = 0; i < size; i++) {
	z = (t + DataPtr [i]) * einsdbeta;
	if (z >= -1.0 && z <= 1.0) sum += DKern (z, Kernel);
	z = (tzweis0 - DataPtr [i]) * einsdbeta;
	if (z >= -1.0 && z <= 1.0) sum += DKern (z, Kernel);
    }
    return sum / (size * beta);
}

double TUnivariateSample::BoundDenBoth (int Kernel, double t, double Bandwidth, double beta0, double beta1) {
    double beta = Bandwidth;
    int size = GetDataSize ();
    if (t < DataPtr [0] || t > DataPtr [size-1]) return 0.0;
    if (t - DataPtr [0] <= beta)
        beta = beta0 + (t - DataPtr [0]) / beta  * (beta - beta0);
    if (DataPtr [size-1] - t<= beta)
        beta = beta1 + (DataPtr [size-1] - t) / beta * (beta - beta1);
    double z, sum = 0.0,
	   tzweis0 = t - DataPtr [0] - DataPtr [0],
	   tzweisn = t - DataPtr [size-1] - DataPtr [size-1],
	   einsdbeta = 1.0 / beta;
    if (Kernel == 3) einsdbeta *= 4.0 / 7.0;
    for (int i = 0; i < size; i++) {
	z = (t - DataPtr [i]) * einsdbeta;
	if (z >= -1.0 && z <= 1.0) sum += DKern (z, Kernel);
	z = (tzweis0 + DataPtr [i]) * einsdbeta;
	if (z >= -1.0 && z <= 1.0) sum += DKern (z, Kernel);
	z = (tzweisn + DataPtr [i]) * einsdbeta;
	if (z >= -1.0 && z <= 1.0) sum += DKern (z, Kernel);
    }
    return sum / (size * beta);
}

String TUnivariateSample::GetHeader () {
    return SampleHeaders [UNIVARIATE_SAMPLE];
}

double TUnivariateSample::SampleDf (double x) {
    int	count =	0;
    while (count < DataSize && DataPtr [count] <= x) count++;
    return (double) count / (double) DataSize;
}

double TUnivariateSample::SampleQf (double q) {
    if (q <= 0.0) return DataPtr [0];
    return DataPtr [min	(DataSize - 1, (int) ceil (DataSize * q))];
}

double TUnivariateSample::SampleDen (double x, double beta, int KNo) {
    return BoundDenNone (KNo, x, beta);
}

double TUnivariateSample::SampleHazard (double t, double beta) {
    int	count =	0;
    double dis = 0.0, den = 0.0;
    for	(int i = 0; i <	DataSize; i++)
	if (DataPtr [i]	< t - beta) count++;
	else if	(DataPtr [i] <=	t + beta) {
	    double y = (t - DataPtr [i]) / beta;
	    dis	+= ((y*y + 3.0)	* y + 2.0) / (4.0 * beta);
	    den	+= 3.0 * (1.0 -	y*y) / (4.0 * beta);
	} else break;
    dis	= (dis + (double) count) / DataSize;
    den	/= DataSize;
    return den / (1.0 -	dis);
}

double TUnivariateSample::SampleMean ()	{
    double v = 0.0;
    for	(int i = 0; i <	DataSize; i++)
	v += DataPtr [i] / DataSize;
    return v;
}

double TUnivariateSample::SampleVariance () {
    double v = 0.0,
	   m = SampleMean ();
    for	(int i = 0; i <	DataSize; i++)
	v += square (DataPtr [i] - m) /	DataSize;
    return v;
}

double TUnivariateSample::SampleSkewness () {
    double m1 = 0.0, m2 = 0.0, m3 = 0.0;
    for (int i = 0; i < DataSize; i++) {
        m1 += DataPtr [i];
        m2 += DataPtr [i] * DataPtr [i];
        m3 += DataPtr [i] * DataPtr [i] * DataPtr [i];
    }
    m1 /= DataSize; m2 /= DataSize; m3 /= DataSize;
    double h = m2 - m1 * m1;
    if (h == 0.0) return DBL_MAX;
    return (m3 - 3 * m1 * m2 + 2 * m1 * m1 * m1) / pow (h, 3.0 / 2.0);
}

void TUnivariateSample::Symmetrize (String filename, BOOL Mean, double val) {
    int n = DataSize,
	NewDataSize = (n & 1) ? 2 * n  - 1 : 2 * n - 2;
    if (!Mean) NewDataSize = 2 * n;
    double *NewData = new double [NewDataSize];
    int j = 0, o1, o2;
    double med;
    if (n & 1) {
        o1 = o2 = (n - 1) / 2;
        med = 2.0 * DataPtr [o1];
    } else {
        o1 = n / 2 - 1;
        o2 = o1 + 1;
        med = DataPtr [o1] + DataPtr [o2];
    }
    if (!Mean) {
        med = 2.0 * val;
        o1 = o2 = -1;
    }
    for (int i = 0; i < DataSize; i++) {
        NewData [j++] = DataPtr [i];
        if (i != o1 && i != o2) NewData [j++] = med - DataPtr [i];
    }
    TList s;
    s.Append (new TStringEl	("Data set containing symmetrized values of"));
    s.Append (new TStringEl	(GetName ()));
    TUnivariateSample *p = new TUnivariateSample;
    p->SetName ("Symmetrized " & GetName ());
    p->SetDescription (&s);
    p->SetData (NewDataSize, NewData, filename);
    s.DeleteAll (FALSE);
    delete [] NewData;
}

void TUnivariateSample::TruncateSample (String filename, double threshold, int Column) {
    int i, NewDataSize = 0, j = 0;
    for (i = 0; i < GetDataSize (); i++)
	if (RawDataPtr [i] >= threshold) NewDataSize++;
    double *NewData = new double [NewDataSize];
    for (i = 0; i < GetDataSize (); i++)
	if (RawDataPtr [i] >= threshold)
	    NewData [j++] = RawDataPtr [i];
    TList s;
    s.Append (new TStringEl ("Data set generated by Xtremes extracting the points"));
    s.Append (new TStringEl (form (100, "above the threshold %g from", threshold)));
    s.Append (new TStringEl (GetName ()));
    TUnivariateSample *p = new TUnivariateSample;
    p->SetName ("Exceedances of " & GetName ());
    p->SetDescription (&s);
    p->SetData (NewDataSize, NewData, filename);
    s.DeleteAll (FALSE);
    delete [] NewData;
}


#ifndef	TERMINAL

void TUnivariateBaseSample::ListData (TMDIFrame *MDIFrame) {
    new	TUnivariateListWindow (MDIFrame, this);
}

void TUnivariateSample::PlotSampleDF (TWindow *) {
    PlotWindowManager.GetOpenWindow (DISTRIBUTION)->InsertFunction (new	TEmpiricalDF (this));
}

void TUnivariateSample::PlotSampleQF (TWindow *) {
    PlotWindowManager.GetOpenWindow (QUANTILE)->InsertFunction (new TEmpiricalQF (this));
}

void TUnivariateSample::PlotKernelDensity (TWindow *) {
//    PlotFunction (hwndParent,	new TKernelDensity (this), IDS_Density);
}

void TUnivariateSample::PlotBoundedKernelDensity (TWindow *parent) {
    PlotFunction (parent, new TBoundedKernelDensity (this), DENSITY);
}

void TUnivariateSample::PlotMeanExcessFunction (TWindow *parent) {
    PlotFunction (parent, new TEmpiricalMeanRLF (this), MEANEXCESS);
}

void TUnivariateSample::PlotMedianExcessFunction (TWindow *parent) {
    PlotWindowManager.GetOpenWindow (MEDIANEXCESS)->InsertFunction (new	TEmpiricalMedianRLF (this));
}

void TUnivariateSample::PlotHazardFunction (TWindow *parent) {
    PlotFunction (parent, new TSampleHazardFunction (this), HAZARD);
}

void TUnivariateSample::PlotPath (TWindow *parent) {
    PlotFunction (parent, new TEmpiricalPath (this), PATH);
}

void TUnivariateSample::PlotTimeTailDep(TWindow *parent)
{
	PlotFunction(parent, new TTimeTailDep(this),PATH);
}


void TUnivariateSample::BoxPlot (TMDIFrame *MDIFramePtr) {
    double ymin, ymax, y0, y1, xmin, xmax;
    TPlotWindow *w = PlotWindowManager.GetOpenWindow ("Boxplot of " & Name ());
    TRectangle r = w->GetClientRectangle ();
    TBoxPlot *f = new TBoxPlot (GetDataSize (), DataPtr, 0.0, Name ());
    f->GetBoxRange (y0, y1);
    f->GetYRange (ymin, ymax);
    double h = (ymax - ymin) / 5.0;
    ymin -= h; ymax += h;
    w->InsertFunction (f);
    double qs = (y1 - y0) * r.x1 / (5.0 * r.x1),
           b = (ymax - ymin) /qs;
    xmin = -b / 2.0;
    xmax = b / 2.0;
    w->SetFrameMode (TRUE);
    w->SetNewCoordinates (TDblRectangle (xmin, ymin, xmax, ymax));
}

class TFunctionalParaDialog : public TDialog {
    typedef TDialog inherited;
    public:
	TFunctionalParaDialog (TUnivariateSample *);
    protected:
	virtual	void NotifyActionEvent (TDialogMember *m);
    private:
	TUnivariateSample *p;
	TStaticField SSize, SMinimum, SMaximum,	SSkewness, SMedian, SMean,
		     SMeanDev, SVariance;
	TCalcDoubleField qQuantile, fDF;
        TIntField fK;
        BOOL Active;
};

TFunctionalParaDialog::TFunctionalParaDialog (TUnivariateSample	*p0)
 : p (p0),
   SSize (IDD_SampleSize),
   SMinimum (IDD_SampleMin),
   SMaximum (IDD_SampleMax),
   SSkewness (IDD_SampleSkewness),
   SMedian (IDD_SampleMedian),
   SMean (IDD_SampleMean),
   SMeanDev (IDD_SampleVariance),	// mean deviation!
   SVariance (IDD_Variance),
   qQuantile (IDD_N, 0.5, 0.0, 1.0),
   fDF (IDD_SampleDF, 0.0, -DBL_MAX, DBL_MAX),
   fK (IDD_Exceedances, 10, 0, INT_MAX),
   TDialog (IDD_FunctionalParameters) {
    Active = FALSE;
    *this << SSize << SMinimum << SMaximum << SSkewness
	  << SMedian <<	SMean << SMeanDev << SVariance << qQuantile << fDF << fK;
    SSize.SetText (form	(20, "%d", p->GetDataSize ()));
    SMinimum.SetText (form (20,	"%g", (*p) [0]));
    SMaximum.SetText (form (20,	"%g", (*p) [p->GetDataSize () -	1]));
    SMedian.SetText (form (20, "%g", (*p) [p->GetDataSize () / 2]));
    SMean.SetText (form	(20, "%g", p->SampleMean ()));
    double g;
    SMeanDev.SetText (form (20, "%g", sqrt (g = p->SampleVariance ())));
    SVariance.SetText (form (20, "%g", g));
    double h = p->SampleSkewness ();
    if (h != DBL_MAX) SSkewness.SetText (form (20, "%g", h));
    else SSkewness.SetText ("Undefined");
    fDF.SetValue (p->SampleQf (0.5));
    fK.SetValue ((int) (p->GetDataSize () * (1.0 - 05)));
    Active = TRUE;
}

void TFunctionalParaDialog::NotifyActionEvent (TDialogMember *m) {
    if (Active) {
        Active = FALSE;
        if (m->GetID () == IDD_N) {
            if (qQuantile.IsCorrect ()) {
                double q = qQuantile.GetValue ();
                fDF.SetValue (p->SampleQf (q));
                fK.SetValue ((int) (p->GetDataSize () * (1.0 - q)));
            }
        } else if (m->GetID () == IDD_SampleDF) {
            fDF.IsCorrect ();
            double q = p->SampleDf (fDF.GetValue ());
            qQuantile.SetValue (q);
            fK.SetValue ((int) (p->GetDataSize () * (1.0 - q)));
        } else if (m->GetID () == IDD_Exceedances) {
            int k = fK.GetValue ();
            double q = 1.0 - ((double) k) / ((double) p->GetDataSize ());
            qQuantile.SetValue (q);
            fDF.SetValue (p->SampleQf (q));
        }
        Active = TRUE;
    }
    inherited::NotifyActionEvent (m);
}

void TUnivariateSample::ShowFunctionalParameters (TWindow *parent) {
    TFunctionalParaDialog d (this);
    d.Handle (parent);
}

void TUnivariateSample::TransformSample	(TWindow *parent) {
    TWriteFileField e (IDD_Name, ModifyName ('a'));
    TCalcDoubleField e1 (IDD_Mu, GlobalData.AffineTrafoMu),
		 e2 (IDD_Sigma,	GlobalData.AffineTrafoSigma);
    TDialog d (IDD_AffineTrafoSample);
    d << e << e1 << e2;
    unsigned ch	= d.Handle (parent);
    if (ch == IDOK) {
	double *NewData	= new double [GetDataSize ()],
	       a = e1.GetValue (), b = e2.GetValue ();
        GlobalData.AffineTrafoMu = a;
        GlobalData.AffineTrafoSigma = b;
	for (int i = 0;	i < GetDataSize	(); i++)
	    NewData [i]	= a + b	* RawDataPtr [i];
	TList s;
        s.Append (new TStringEl (form (150, "Affine transformation x -> %g x + %g applied to", b, a)));
	s.Append (new TStringEl	(GetName ()));
	TUnivariateSample *p = new TUnivariateSample;
	p->SetName ("Transformed " & GetName ());
	p->SetDescription (&s);
	p->SetData (GetDataSize	(), NewData, e.ReadBuf ());
	s.DeleteAll (FALSE);
	delete [] NewData;
    }
}

void TUnivariateSample::ConverseSample (TWindow *parent) {
    TEditField e (IDD_Name, ModifyName ('c') , MAXFILENAME);
    TDialog d (IDD_ConverseSample);
    d.SetHelpNumber (IDM_ChangeSign);
    d << e;
    unsigned ch	= d.Handle (parent);
    if (ch == IDOK) {
	double *NewData	= new double [GetDataSize ()];
	for (int i = 0;	i < GetDataSize	(); i++) NewData [i] = -RawDataPtr [i];
	TList s;
	s.Append (new TStringEl	("Data set generated by	Xtremes	applying the Converse Sign option to"));
	s.Append (new TStringEl	(GetName ()));
	TUnivariateSample *p = new TUnivariateSample;
	p->SetName ("Conversed " & GetName ());
	p->SetDescription (&s);
	p->SetData (GetDataSize	(), NewData, e.ReadBuf ());
	s.DeleteAll (FALSE);
	delete [] NewData;
    }
}

void TUnivariateSample::Symmetrize (TWindow *parent) {
    TAlterField a (GlobalData.SymmetrizeOption, 2, IDD_Mean, IDD_Symm);
    TWriteFileField e1 (IDD_FileName, ModifyName ('s'));
    TCalcDoubleField e2 (IDD_SymmVal, GlobalData.SymmetrizeValue, -DBL_MAX, DBL_MAX);
    TDialog d (IDD_Symmetrize);
    d << a << e1 << e2;
    unsigned ch	= d.Handle (parent);
    if (ch == IDOK) {
	GlobalData.SymmetrizeOption = a.Selector ();
	GlobalData.SymmetrizeValue = e2.GetValue ();
	Symmetrize (e1.ReadBuf (), GlobalData.SymmetrizeOption == 0, GlobalData.SymmetrizeValue);
    }
}

void TUnivariateSample::CumulateSample (TWindow *parent) {
    TEditField e (IDD_Name, ModifyName ('c') , MAXFILENAME);
    TDialog d (IDD_ConverseSample);
    d.SetHelpNumber (IDM_CumulateData);
    d << e;
    d.SetTitel ("Cumulate sample");
    unsigned ch	= d.Handle (parent);
    if (ch == IDOK) {
	double *NewData	= new double [GetDataSize ()];
	NewData [0] = RawDataPtr [0];
	for (int i = 1;	i < GetDataSize	(); i++) NewData [i] = NewData [i - 1] + RawDataPtr [i];
	TList s;
	s.Append (new TStringEl	("Data set generated by	Xtremes	applying the Cumulate data option to"));
	s.Append (new TStringEl	(GetName ()));
	TUnivariateSample *p = new TUnivariateSample;
	p->SetName ("Cumulated " & GetName ());
	p->SetDescription (&s);
	p->SetData (GetDataSize	(), NewData, e.ReadBuf ());
	s.DeleteAll (FALSE);
	delete [] NewData;
    }
}

void TUnivariateSample::OrderSample (String filename, int Column) {
    TList s;
    s.Append (new TStringEl	("Data set containing ordered values of"));
    s.Append (new TStringEl	(GetName ()));
    TUnivariateSample *p = new TUnivariateSample;
    p->SetName ("Ordered" &	GetName	());
    p->SetDescription (&s);
    p->SetData (GetDataSize	(), DataPtr, filename);
    s.DeleteAll (FALSE);
}

void TUnivariateSample::OrderSample (TWindow *parent) {
    TEditField e (IDD_Name, ModifyName ('o') , MAXFILENAME);
    TDialog d (IDD_ConverseSample);
    d.SetHelpNumber (IDM_Order);
    d.SetTitel ("Order Sample");
    d << e;
    unsigned ch	= d.Handle (parent);
    if (ch == IDOK)
	OrderSample (e.ReadBuf (), 0);
}

void TUnivariateSample::TruncateSample (TWindow *parent) {
    TCalcDoubleField f (IDD_Threshold, GlobalData.TruncSampleThreshold);
    TEditField e (IDD_Name, ModifyName ('e'), MAXFILENAME);
    TDialog d (IDD_ExtremesDialog);
    d.SetTitel ("Save exceedances");
    d << e << f;
    unsigned ch	= d.Handle (parent);
    if (ch == IDOK) {
	GlobalData.TruncSampleThreshold = f.GetValue ();
	TruncateSample (e.ReadBuf (), GlobalData.TruncSampleThreshold, 0);
    }
}

void TUnivariateSample::ExtractMaxima (TWindow *parent)	{
    TIntField f	(IDD_GroupSize,	GlobalData.BlockMaximaSize, 1, INT_MAX);
    TEditField e (IDD_Name, ModifyName ('m'), MAXFILENAME);
    TDialog d (IDD_MaximaDialog);
    d << e << f;
    unsigned ch	= d.Handle (parent);
    if (ch == IDOK) {
	int GroupLength	= f.GetValue (),
	    NewDataSize	= GetDataSize () / GroupLength;
	if (!NewDataSize) {
            ErrorBox (parent, "No maxima found");
	    return;
	}
	double *NewData	= new double [NewDataSize];
        GlobalData.BlockMaximaSize = GroupLength;
	int i, j = 0, k;
	for (i = 0; i <	NewDataSize; i++) {
	    double maxv = GetRawData (j); j++;
	    for	(k = 1;	k < GroupLength; k++) {
		if (GetRawData (j) > maxv) maxv	= GetRawData (j);
		j++;
	    }
	    NewData [i]	= maxv;
	}
	TList s;
	s.Append (new TStringEl	("Blocks maxima "));
	s.Append (new TStringEl ("Group length:  " & form (20, "%d", GroupLength)));
	s.Append (new TStringEl ("Applied to:    " & GetName ()));
	TUnivariateSample *p = new TUnivariateSample;
	p->SetName ("Blocks Maxima of " & GetName ());
	p->SetDescription (&s);
	p->SetData (NewDataSize, NewData, e.ReadBuf ());
	s.DeleteAll (FALSE);
	delete [] NewData;
    }
}

void TUnivariateSample::SaveBlocksSum (TWindow *parent)	{
    TIntField f	(IDD_GroupSize,	GlobalData.BlockSumSize, 1, INT_MAX);
    TEditField e (IDD_Name, ModifyName ('m'), MAXFILENAME);
    TDialog d (IDD_MaximaDialog);
    d.SetHelpNumber (IDM_SaveBlocksSum);
    d.SetTitel ("Save blocks sums");
    d << e << f;
    unsigned ch	= d.Handle (parent);
    if (ch == IDOK) {
	int GroupLength	= f.GetValue (),
	    NewDataSize	= GetDataSize () / GroupLength;
	if (!NewDataSize) {
            ErrorBox (parent, "Block size is larger than sample size");
	    return;
	}
        GlobalData.BlockSumSize = GroupLength;
	double *NewData	= new double [NewDataSize];
	int i, j = 0, k;
	for (i = 0; i <	NewDataSize; i++) {
	    NewData [i] = 0.0;
	    for	(k = 0;	k < GroupLength; k++)
		NewData [i] += GetRawData (j++);
	}
	TList s;
	s.Append (new TStringEl	("Blocks sums"));
	s.Append (new TStringEl ("Group length:  " & form (20, "%d", GroupLength)));
	s.Append (new TStringEl ("Applied to:    " & GetName ()));
	TUnivariateSample *p = new TUnivariateSample;
	p->SetName ("Blocks sums of " & GetName ());
	p->SetDescription (&s);
	p->SetData (NewDataSize, NewData, e.ReadBuf ());
	s.DeleteAll (FALSE);
	delete [] NewData;
    }
}

void TUnivariateSample::SaveMovingSum (TWindow *parent) {
    TIntField f	(IDD_GroupSize,	GlobalData.MovingSumSize, 1, INT_MAX);
    TEditField e (IDD_Name, ModifyName ('m'), MAXFILENAME);
    TDialog d (IDD_MaximaDialog);
    d.SetHelpNumber (IDM_SaveMovingSums);
    d.SetTitel ("Save moving sums");
    d << e << f;
    unsigned ch	= d.Handle (parent);
    if (ch == IDOK) {
	int GroupLength	= f.GetValue (),
	    NewDataSize	= GetDataSize () - GroupLength + 1;
	if (NewDataSize <= 0) {
	    ErrorBox (parent, "Block size is larger than sample size");
	    return;
	}
	double *NewData	= new double [NewDataSize];
        GlobalData.MovingSumSize = GroupLength;
	int i;
	double val = 0.0;
	for (i = 0; i < GroupLength; i++) val += GetRawData (i);
	NewData [0] = val;
	for (i = 1; i <	NewDataSize; i++)
	    NewData [i] = NewData [i - 1] - GetRawData (i - 1) + GetRawData (i + GroupLength - 1);
	TList s;
	s.Append (new TStringEl	("Moving sums"));
	s.Append (new TStringEl ("Group length:  " & form (20, "%d", GroupLength)));
	s.Append (new TStringEl ("Applied to:    " & GetName ()));
	TUnivariateSample *p = new TUnivariateSample;
	p->SetName ("Moving sums of " & GetName ());
	p->SetDescription (&s);
	p->SetData (NewDataSize, NewData, e.ReadBuf ());
	s.DeleteAll (FALSE);
	delete [] NewData;
    }
}

void TUnivariateSample::SaveMovingMaxima (TWindow *parent) {
    TIntField f	(IDD_GroupSize,	GlobalData.MovingMaximaSize, 1, INT_MAX);
    TEditField e (IDD_Name, ModifyName ('m'), MAXFILENAME);
    TDialog d (IDD_MaximaDialog);
    d.SetHelpNumber (IDM_SaveMovingMaxima);
    d.SetTitel ("Save moving maxima");
    d << e << f;
    unsigned ch	= d.Handle (parent);
    if (ch == IDOK) {
	int GroupLength	= f.GetValue (),
	    NewDataSize	= GetDataSize () - GroupLength + 1;
	if (NewDataSize <= 0.0) {
	    ErrorBox (parent, "Block size is larger than sample size");
	    return;
	}
	double *NewData	= new double [NewDataSize];
        GlobalData.MovingMaximaSize = GroupLength;
	int i, k;
	for (i = 0; i <	NewDataSize; i++) {
	    NewData [i] = GetRawData (i);
	    for (k = i; k < i + GroupLength; k++)
		if (GetRawData (k) > NewData [i]) NewData [i] = GetRawData (k);
	}
	TList s;
	s.Append (new TStringEl	("Moving maxima"));
	s.Append (new TStringEl ("Group length:  " & form (20, "%d", GroupLength)));
	s.Append (new TStringEl ("Applied to:    " & GetName ()));
	TUnivariateSample *p = new TUnivariateSample;
	p->SetName ("Moving maxima of " & GetName ());
	p->SetDescription (&s);
	p->SetData (NewDataSize, NewData, e.ReadBuf ());
	s.DeleteAll (FALSE);
	delete [] NewData;
    }
}

void TUnivariateSample::UserTransformation (TWindow *parent) {
    TSample *q = new TUserTrafoSample (this);
    if (!q->Create (parent)) delete q;
}

unsigned TUnivariateSample::EditParameter (TWindow *parent) {
    #ifndef NDEBUG
	ErrorBox (parent, "TStandardSample::EditParameter called");
    #endif
    return IDOK;
}

BOOL TUnivariateSample::CreateData (TWindow *) {
    return FALSE;
}


#endif

void TUnivariateSample::SetData	(int n,	double *data, BOOL MakeCopy) {
    DataSize = n;
    GetMemory ();
    if (MakeCopy)
        for (int i = 0; i < DataSize; i++)
	    RawDataPtr [i] = data [i];
    else {
        delete [] RawDataPtr;
        RawDataPtr = data;
    }
    if (filename != String ("")) {
	SaveData (0);		// Hier	noch Handle auf	Frame-Window
	SampleList.InsertSample	(this);
    }
    SortData ();
}

void TUnivariateSample::SetData	(int n,	double *data, String fn) {
    filename = fn;
    SetData (n,	data);
}

void TUnivariateSample::SetSampleSize (int s, int k) {
    DataSize = s;
    BlockSize =	k;
}

#ifndef	TERMINAL

TSample	*TUnivariateSample::ConvertToTimeSeries	(TWindow *parent) {
    TEditField e (IDD_Name, ModifyName ('t'), MAXFILENAME);
    TDialog d (IDD_ConverseSample);
    d.SetTitel ("Convert to Time Series");
    d << e;
    TTimeSeriesSample *p = NULL;
    unsigned ch	= d.Handle (parent);
    if (ch == IDOK) {
	double *x = new	double [DataSize];
	p = new	TTimeSeriesSample;
	p->SetName ("Time Series " & GetName ());
	TList s;
	s.Append (new TStringEl	("Data set generated by Xtremes applying the Convert to Time Series option to"));
	s.Append (new TStringEl	(GetName ()));
	p->SetDescription (&s);
	s.DeleteAll (FALSE);
	for (int i = 0;	i < DataSize; i++) x[i]	= i + 1;
	p->SetData (DataSize, x, RawDataPtr, e.ReadBuf ());
	delete [] x;
    }
    return p;
}

TSample *TUnivariateSample::ConvertToFrequency (TWindow *parent) {
    const double EPS = 1e-6;
    int i;
    for (i = 0; i < DataSize; i++)
        if (fabs (DataPtr [i] - (int) DataPtr [i]) > EPS) {
            ErrorBox (parent, "Sample contains non-integer data");
            return NULL;
        }
    TEditField e (IDD_Name, ModifyName ('f'), MAXFILENAME);
    TDialog d (IDD_ConverseSample);
    d.SetTitel ("Convert to Frequency");
    d.SetHelpNumber (IDM_ConvertDiscrete);
    d << e;
    if (d.Handle (parent) == IDOK) {
        // count number of points
        int x0 = (int) DataPtr [i], x1,
            diff = 0;
        for (i = 1; i < DataSize; i++) {
            x1 = (int) DataPtr [i];
            if (x1 != x0) {
                x0 = x1;
                diff++;
            }
        }
        // create discrete data set
        int *t = (int *) malloc ((diff + 1) * sizeof (int)),
            *c = (int *) malloc ((diff + 1) * sizeof (int));
        diff = 0;
        t [diff] = (int) DataPtr [0];
        c [diff] = 1;
        for (i = 1; i < DataSize; i++)
            if (int (DataPtr [i]) != t [diff]) {
                t [++diff] = (int) DataPtr [i];
                c [diff] = 1;
            } else
                c [diff]++;
	TFrequencySample *p = new TFrequencySample;
        p->SetData (diff + 1, t, c, e.ReadBuf ());
    }
    return NULL;
}

TSample	*TUnivariateSample::ConvertTo (TSampleType st, TWindow *parent)	{
    if (st == UNIVARIATE_SAMPLE) {
	WarningBox (parent, "Data set is already univariate");
	return NULL;
    } else if (st == GROUPED_SAMPLE) {
	TSample	*q = new TGroupedSample	(this);
	if (!q->Create (parent)) {
            delete q;
            return NULL;
        }
	return q;
    } else if (st == TIMESERIES_SAMPLE)
	return ConvertToTimeSeries (parent);
    else if (st == FREQUENCY_SAMPLE)
    	return ConvertToFrequency (parent);
    else return TSample::ConvertTo (st, parent);
}

#endif


//
//

//
//

#ifndef	TERMINAL

TIIDSample::TIIDSample () {}
TIIDSample::~TIIDSample	() {}

BOOL TIIDSample::CreateData (TWindow *parent) {
    for	(int i = 0; i <	DataSize; i++) {
	RawDataPtr [i] = DrawRV	();
	if (BlockSize >	1)
	    for	(int j = 1; j <	BlockSize; j++)
		RawDataPtr [i] = max (RawDataPtr [i], DrawRV ());
    }
    return TRUE;
}

void TIIDSample::CreateInteractive (TWindow *parent) {
    new	TSampleVisualizeWindow (this, GetDF ());
}

#endif

/********************************************************************
			Frequency Data Sets
********************************************************************/

#ifndef	TERMINAL

class TFrequencyListWindow : public TSampleListWindow {
    typedef TSampleListWindow inherited;
    public:
	TFrequencyListWindow (TMDIFrame *, TFrequencySample *p, int breadth);
    protected:
	virtual	int GetSubMenuPos ();
	virtual	String GetLine (int i);
};

TFrequencyListWindow::TFrequencyListWindow (TMDIFrame *MDIFrame, TFrequencySample *p, int breadth)
 : inherited (p, MDIFrame, p->GetFileName ()) {
    SetHeight (p->GetDataSize () + 2);
    SetWidth (breadth);
    SetIcon (IDI_TextLister);
}

int TFrequencyListWindow::GetSubMenuPos	() {
    return 12;
}

String TFrequencyListWindow::GetLine (int i) {
    if (i >= s->GetDataSize ())	return "";
    return form	(60, "%-10d %ld", ((TFrequencySample *) s)->GetPoint (i), ((TFrequencySample *) s)->GetFrequency (i));
}

#endif

TFrequencySample::TFrequencySample () {
    PointData =	NULL;
    FrequencyData = NULL;
    DataSize = Allocated = 0;
}

TFrequencySample::~TFrequencySample () {
    if (PointData) free	(PointData);
    if (FrequencyData) free (FrequencyData);
}

int TFrequencySample::GetDimension () {
    return 2;
}

int TFrequencySample::GetDataSize () {
    return DataSize;
}

double TFrequencySample::GetData (int i, int j) {
    assert (0 <= i && i	< DataSize);
    if (j == 0) return FrequencyData [i];
    return PointData [i];
}

String TFrequencySample::GetColumnName (int j) {
    if (j == 0) return "Point";
    return "Frequency";
}

int TFrequencySample::GetPoint (int i) {
    assert (0 <= i && i	< DataSize);
    assert (PointData != NULL);
    return PointData [i];
}

int TFrequencySample::GetFrequency (int i) {
    assert (0 <= i && i	< DataSize);
    assert (FrequencyData != NULL);
    return FrequencyData [i];
}

void TFrequencySample::SetData (int n, int *t, int *c, String fn) {
    DataSize = n;
    PointData = t;
    FrequencyData = c;
    filename = fn;
    SaveData (0);		// Hier	noch Handle auf	Frame-Window
    SortData ();
    SampleList.InsertSample (this);
}

#ifndef	TERMINAL

void TFrequencySample::PlotHistogram (TWindow *parent) {
    PlotWindowManager.GetOpenWindow (DENSITY)->InsertHistogram (new TFrequencyHistogram	(this));
}

void TFrequencySample::ListData	(TMDIFrame *MDIFrame) {
    new	TFrequencyListWindow (MDIFrame, this, 20);
}

TSample	*TFrequencySample::ConvertToUnivariate (TWindow *parent) {
    TEditField e (IDD_Name, ModifyName ('u'), MAXFILENAME);
    TDialog d (IDD_ConverseSample);
    d.SetTitel ("Convert to Univariate");
    d << e;
    unsigned ch	= d.Handle (parent);
    if (ch == IDOK) {
	TUnivariateSample *p = new TUnivariateSample;
	p->SetName ("Univariate	" & GetName ());
	TList s;
	s.Append (new TStringEl	("Data set generated by	Xtremes	applying the Convert to	Univariate Option to"));
	s.Append (new TStringEl	(GetName ()));
	p->SetDescription (&s);
	s.DeleteAll (FALSE);
	int c = 0;
        int i;
	for (i = 0; i < DataSize; i++) c += FrequencyData [i];
        if (c > 100000L)
            if (!VerificationBox (parent, "The resulting data set will contain " & form (20, "%d", c) &
                                        "data points. Do you really want to perform	the operation?",
                                        "Warning")) return NULL;
        double *x =	new double [(int) c], *y = x;
        for	(i = 0;	i < DataSize; i++)
            for (int j = 0;	j < FrequencyData [i]; j++)
                *y++ = PointData [i];
        p->SetData ((int) c, x, e.ReadBuf ());
        delete [] x;
        return p;
    }
    ErrorBox (parent,	"The data set cannot be	converted because it contains more "
                            "than "	& form (20, "%d", MAXSAMPLESIZE) & " data points.");
    return NULL;
}

void TFrequencySample::FrequenciesToRelative (TWindow *parent) {
    TWriteFileField e (IDD_Name, ModifyName ('r'));
    TDialog d (IDD_ConverseSample);
    d.SetTitel ("Transform to Relative Frequencies");
    d.SetHelpNumber (IDM_FrequenciesRelative);
    d << e;
    unsigned ch	= d.Handle (parent);
    if (ch == IDOK) {
	TMultivariateSample *p = new TMultivariateSample;
        p->SetData (2, DataSize);
        double *x = new double [DataSize], sum = 0.0;
	int i;
        for (i = 0; i < DataSize; i++) {
            x [i] = PointData [i];
            sum += FrequencyData [i];
        }
        p->SetColumn (0, x, "Point");
        for (i = 0; i < DataSize; i++)
            x [i] = FrequencyData [i] / sum;
	p->SetColumn (1, x, "Rel. Freq.");
        delete [] x;
        p->SetFilename (e.ReadBuf (), "Relative Frequencies of " & Name ());
        p->SaveData (0);
        SampleList.InsertSample (p);
    }
}

TSample	*TFrequencySample::ConvertTo (TSampleType st, TWindow *parent) {
    if (st == UNIVARIATE_SAMPLE)
	return ConvertToUnivariate (parent);
    return TSample::ConvertTo (st, parent);
}

unsigned TFrequencySample::EditParameter (TWindow *parent) {
    return IDCANCEL;
}

TAnimatorWindow *TFrequencySample::GetAnimatorWindow () {
    return NULL;
}

BOOL TFrequencySample::CreateData (TWindow *parent) {
    return FALSE;
}

#endif

BOOL TFrequencySample::AddPoint	(int p)	{
    BOOL Found = FALSE;
    int	i;
    for	(i = 0;	i < DataSize; i++)
	if (p <= PointData [i])	{
	    Found = p == PointData [i];
	    break;
	}
    if (Found) FrequencyData [i]++;
    else {
	if (DataSize >=	Allocated) {
	    Allocated += 100;
	    PointData =	(int *)	realloc	(PointData, Allocated *	sizeof (int));
	    FrequencyData = (int *) realloc (FrequencyData, Allocated * sizeof	(int));
	}
	for (int j = DataSize; j > i; j--) {
	    PointData [j] = PointData [j - 1];
	    FrequencyData [j] =	FrequencyData [j - 1];
	}
	DataSize++;
	PointData [i] =	p;
	FrequencyData [i] = 1;
    }
    return TRUE;
}

String TFrequencySample::GetHeader () {
    return SampleHeaders [FREQUENCY_SAMPLE];
}

TSampleType TFrequencySample::GetSampleType () {
    return FREQUENCY_SAMPLE;
}

BOOL TFrequencySample::GetMemory () {
    PointData =	(int *)	malloc (DataSize * sizeof (int));
    FrequencyData = (int *) malloc (DataSize *	sizeof (int));
    return FrequencyData != NULL;
}

BOOL TFrequencySample::Append (int v, int w) {
    if (DataSize >= AllocatedSize) {
	AllocatedSize += 100;
	if (AllocatedSize > MAXSAMPLESIZE) return FALSE;
	PointData = (int *) realloc (PointData,	AllocatedSize *	sizeof (int));
	FrequencyData =	(int *) realloc (FrequencyData, AllocatedSize * sizeof	(int));
	if (!FrequencyData) return FALSE;
    }
    PointData [DataSize] = v;
    FrequencyData [DataSize++] = w;
    return TRUE;
}

BOOL TFrequencySample::ReadData	(TTextFileReader &f) {
    BOOL b = TRUE; 
    int RegisteredSize;
    if (!ReadSampleSize	(f, RegisteredSize)) return FALSE;
    DataSize = AllocatedSize = 0;
    do {
	int v =	f.GetInt ();
        if (f.GetError() == TTextFileReader::END_OF_FILE) continue;
	int w = -1;
	if (!f.IsError ()) w = f.GetLong ();
	if (!f.IsError ()) b = Append (v, w);
        if (w < 0) {
            ReadMessage = LOADERRNUM;
	    return FALSE;
        }
    } while (!f.IsError	() && b);
    if (f.GetError () == TTextFileReader::END_OF_FILE)
	return TRUE;
    else if (f.GetError	())
	ReadMessage = LOADERRNUM;
    else ReadMessage = MAXSAMPLESIZE == ACADEMICSIZE ? ACADEMIC : INSUFFMEM;
    return FALSE;
}

BOOL TFrequencySample::WriteData (FILE *f) {
    fprintf (f,	"SampleSize: %d\n", DataSize);
    for	(int i = 0; i <	DataSize; i++)
	fprintf	(f, "%d	%d\n", PointData [i], FrequencyData [i]);
    return TRUE;
}

#ifndef	TERMINAL
void TFrequencySample::ShowFunctionalParameters	(TWindow *parent) {
    TStaticField SSize (IDD_SampleSize),
		 SMinimum (IDD_SampleMin),
		 SMaximum (IDD_SampleMax),
		 SSkewness (IDD_SampleSkewness),
		 SMean (IDD_SampleMean),
		 SMode (IDD_Mode),
		 SVariance (IDD_SampleVariance);
    TDialog d (IDD_FunctionalParametersDiscrete);
    d << SSize << SMinimum << SMaximum << SSkewness
      << SMean << SMode	<< SVariance;
    int TotalSize = 0;
    double sum = 0;
    double sumsq = 0.0;
    int	MaxIndex = 0;
    int MaxFreq = 0;
    for	(int i = 0; i <	DataSize; i++) {
	TotalSize += FrequencyData [i];
	sum += ((double) PointData [i])	* FrequencyData	[i];
	sumsq += (((double) PointData [i]) * PointData [i]) * FrequencyData [i];
	if (FrequencyData [i] >	MaxFreq) {
	    MaxFreq = FrequencyData [i];
	    MaxIndex = i;
	}
    }

    char buf [20];
    sprintf (buf, "%ld", TotalSize);
    SSize.SetText (buf);

//    SSize.SetText (form (20, "%ld", TotalSize));
    SMinimum.SetText (form (20,	"%d", PointData	[0]));
    SMaximum.SetText (form (20,	"%d", PointData	[DataSize - 1]));
    SMode.SetText (form	(20, "%d", PointData [MaxIndex]));

    SMean.SetText (form	(20, "%g", (double) sum	/ TotalSize));
    SVariance.SetText (form (20, "%g", sqrt ((sumsq - sum * sum / TotalSize) / TotalSize)));
    d.Handle (parent);
}
#endif

#ifndef	TERMINAL
BOOL TFrequencyIIDSample::CreateData (TWindow *parent) {
    for	(int i = 0; i <	TotalSize; i++)
	AddPoint (DrawRV ());
    return TRUE;
}

void TFrequencyIIDSample::CreateInteractive (TWindow *parent) {
    new	TFrequencySampleVisualizeWindow	(this, GetDF (), TotalSize, GetAnimatorWindow ());
}

#endif

/*******************************************************************/
/*			Grouped Data				   */
/*******************************************************************/

#ifndef	TERMINAL

class TGroupedListWindow : public TSampleListWindow {
    typedef TSampleListWindow inherited;
    public:
	TGroupedListWindow (TMDIFrame *, TGroupedSample *p);
    protected:
	virtual	int GetSubMenuPos ();
	virtual	String GetLine (int i);
};

TGroupedListWindow::TGroupedListWindow (TMDIFrame *MDIFrame, TGroupedSample *p)
 : inherited (p, MDIFrame, p->GetFileName ()) {
    SetHeight (p->GetGroupCount ());
    SetIcon (IDI_TextLister);
}

int TGroupedListWindow::GetSubMenuPos () {
    return 9;
}

String TGroupedListWindow::GetLine (int	i) {
    if (i >= ((TGroupedSample *) s)->GetGroupCount ()) return "";
    return form	(60, "%-15g %g", ((TGroupedSample *) s)->GetPartition (i), ((TGroupedSample *) s)->GetFrequency (i));
}

#endif

double *TGroupedSample::Tab = NULL;
int TGroupedSample::TabCount = 0;

TGroupedSample::TGroupedSample (TUnivariateSample *q) {
    if (!Tab) {
        Tab = new double [11];
        double Tab0 [] = { 100, 200, 300, 400, 500, 1000, 2000, 3000, 4000, 5000, 10000 };
        for (int i = 0; i < 11; i++)
            Tab [i] = Tab0 [i];
        TabCount = 11;
    }
    Frequencies	= NULL;
    GroupCount = TabCount;
    Partition =	new double [11];
    for	(int i = 0; i <	TabCount; i++) Partition [i] = Tab [i];
    p =	q;
}

TGroupedSample::~TGroupedSample	() {
    if (Frequencies) delete [] Frequencies;
    if (Partition) delete [] Partition;
}

TSampleType TGroupedSample::GetSampleType () {
    return GROUPED_SAMPLE;
}

int TGroupedSample::GetDimension () {
    return 2;
}

int TGroupedSample::GetDataSize () {
    return GroupCount;
}

double TGroupedSample::GetData (int i, int j) {
    if (j == 0) return GetPartition (i);
    return GetFrequency (i);
}

String TGroupedSample::GetColumnName (int j) {
    if (j == 0) return "Partition";
    return "Frequency";
}

double TGroupedSample::GetFrequency (int i) {
    assert (Frequencies	!= NULL);
    assert (i >= 0 && i	< GroupCount);
    return Frequencies [i];
}

double TGroupedSample::GetPartition (int i) {
    assert (Partition != NULL);
    assert (i >= 0 && i	<= GroupCount);
    return i ==	GroupCount ?
		2 * Partition [GroupCount - 1] - Partition [GroupCount - 2] :
		Partition [i];
}

double TGroupedSample::EmpQF (double q)	{
    int	m = GetGroupCount ();
    double ntotal = 0.0;
    for	(int i = 0; i <	m; i++)	ntotal += GetFrequency (i);
    int	l = 0; double f	= 0.0;
    for	(;;) {
	f += GetFrequency (l) /	ntotal;
	if (f >= q - q * 1e-10 || l == m - 1) break;
	l++;
    }
    f -= GetFrequency (l) / ntotal;
    return (GetPartition (l + 1) - GetPartition	(l)) / GetFrequency (l)	* ntotal * (q -	f) + GetPartition (l);

}

double TGroupedSample::SampleHazard (double x, double beta) {
    return 0.0;
}

BOOL TGroupedSample::Append (double v, double w) {
    if (GroupCount >= AllocatedSize) {
	AllocatedSize += 100;
	if (AllocatedSize > MAXSAMPLESIZE) return FALSE;
	Frequencies = (double *) realloc (Frequencies, AllocatedSize * sizeof (double));
	Partition = (double *) realloc (Partition, AllocatedSize * sizeof (double));
	if (!Partition)	return FALSE;
    }
    Partition [GroupCount] = v;
    Frequencies	[GroupCount++] = w;
    return TRUE;
}

BOOL TGroupedSample::ReadData (TTextFileReader &f) {
    BOOL b = TRUE; 
    int	RegisteredSize;
    if (!ReadSampleSize	(f, RegisteredSize)) return FALSE;
    GroupCount = AllocatedSize = 0;
    double v0 = -DBL_MAX;
    do {
	double v = f.GetDouble (), w = -1.0;
        if (f.GetError() == TTextFileReader::END_OF_FILE) continue;
	if (!f.IsError ()) w = f.GetDouble ();
	if (!f.IsError ()) b = Append (v, w);
        if (!f.IsError () && (v <= v0 || w < 0)) {
            ReadMessage = LOADERRNUM;
            return FALSE;
        }
        v0 = v;
    } while (!f.IsError	() && b);
    if (f.GetError () == TTextFileReader::END_OF_FILE)
	return TRUE;
    else if (f.GetError	())
	ReadMessage = LOADERRNUM;
    else
	ReadMessage = MAXSAMPLESIZE == ACADEMICSIZE ? ACADEMIC : INSUFFMEM;
    return FALSE;
}

BOOL TGroupedSample::WriteData (FILE *f) {
    fprintf (f,	"SampleSize: %d\n", GroupCount);
    for	(int i = 0; i <	GroupCount; i++)
	fprintf	(f, "%g\t%g\n",	Partition [i], Frequencies [i]);
    return TRUE;
}

BOOL TGroupedSample::GetMemory () {
    return TRUE;
}

String TGroupedSample::GetHeader () {
    return SampleHeaders [GROUPED_SAMPLE];
}

#ifndef	TERMINAL

void TGroupedSample::ListData (TMDIFrame *MDIFrame) {
    new	TGroupedListWindow (MDIFrame, this);
}

void TGroupedSample::PlotHistogram (TWindow *parent) {
    PlotWindowManager.GetOpenWindow (DENSITY)->InsertFunction (new TGroupedHistogram (this));
}

void TGroupedSample::PlotSampleDF (TWindow *parent) {
    PlotWindowManager.GetOpenWindow (DISTRIBUTION)->InsertFunction (new	TEmpiricalGroupedDF (this));
}

void TGroupedSample::PlotSampleQF (TWindow *parent) {
    PlotWindowManager.GetOpenWindow (QUANTILE)->InsertFunction (new TEmpiricalGroupedQF	(this));
}

void TGroupedSample::PlotMeanExcessFunction (TWindow *parent) {
    PlotFunction (parent, new TEmpiricalGroupedMeanRLF (this), MEANEXCESS);
}

void TGroupedSample::PlotMedianExcessFunction (TWindow *parent)	{
    PlotWindowManager.GetOpenWindow (MEDIANEXCESS)->InsertFunction (new	TEmpiricalGroupedMedianRLF (this));
}

void TGroupedSample::PlotHazardFunction	(TWindow *parent) {
    PlotWindowManager.GetOpenWindow (HAZARD)->InsertFunction (new TEmpiricalGroupedHazardFunction (this));
}



class TGroupedSampleDialog : public TDialog {
    typedef TDialog inherited;
    public:
	TGroupedSampleDialog (double *Partition, double	GroupCount, String filename,
			      double fromvalue,	double tovalue,	double stepvalue);
	TDoubleEditor PartitionEdit;
	TEditField NameEdit;
	TDoubleField from, to, step;
    protected:
	virtual	void NotifyActionEvent (TDialogMember *m);
};

TGroupedSampleDialog::TGroupedSampleDialog (double *Partition, double GroupCount, String filename,
					    double fromvalue, double tovalue, double stepvalue)
 : TDialog (IDD_GroupedSample),
   PartitionEdit (IDD_Partition, Partition, GroupCount),
   NameEdit (IDD_Name, filename, MAXFILENAME),
   from	(IDD_From, fromvalue),
   to (IDD_To, tovalue),
   step	(IDD_Step, stepvalue) {
    *this << PartitionEdit << NameEdit << from << to <<	step;
}

void TGroupedSampleDialog::NotifyActionEvent (TDialogMember *m) {
    if (m->GetID () == IDD_Generate) {
	if (from.IsCorrect () && to.IsCorrect () && step.IsCorrect ()) {
	    BOOL Success = FALSE;
	    double fromvalue = from.GetValue (),
		   tovalue = to.GetValue (),
		   stepvalue = step.GetValue ();
	    if (stepvalue) {
		double d = (tovalue - fromvalue) / stepvalue + 1;
		if (d >	0.0 && d < MAXSAMPLESIZE) {
		    int	Count =	(int) d;
		    double *v =	new double [Count], *q = v;
		    for	(int i = 0; i <	Count; i++)
			*q++ = fromvalue + i * stepvalue;
		    PartitionEdit.SetValues (v,	Count);
		    delete [] v;
		    Success = TRUE;
		}
	    }
	    if (!Success) WarningBox (this, "Cannot generate partition");
	}
    } else inherited::NotifyActionEvent (m);
}

unsigned TGroupedSample::EditParameter (TWindow *parent) {
    TGroupedSampleDialog d (Tab, TabCount, p->ModifyName ('g'),
                            GlobalData.ConvertGroupStart, GlobalData.ConvertGroupStop,
                            GlobalData.ConvertGroupStep);
    if (d.Handle (parent) == IDOK) {
	TabCount = GroupCount = d.PartitionEdit.GetCount ();
	Partition = d.PartitionEdit.GetValues ();
        if (Tab) delete [] Tab;
        Tab = d.PartitionEdit.GetValues ();
        GlobalData.ConvertGroupStart = d.from.GetValue ();
        GlobalData.ConvertGroupStop = d.to.GetValue ();
        GlobalData.ConvertGroupStep = d.step.GetValue ();
	qsort (Partition, GroupCount, sizeof (double), realcomp);
	filename = d.NameEdit.ReadBuf ();
	if (GroupCount < 2) return IDCANCEL;
	name = "Grouped " & p->Name ();
	Description.Append (new	TStringEl ("Data set generated by Xtremes applying the Convert to Grouped data option to"));
	Description.Append (new	TStringEl (p->Name ()));
	return IDOK;
    }
    return IDCANCEL;
}

BOOL TGroupedSample::CreateData	(TWindow *parent) {
    int	i, q, group;
    if (Frequencies) delete Frequencies;
    Frequencies	= new double [GroupCount];
    for	(i = 0;	i < GroupCount;	i++) Frequencies [i] = 0.0;
    for	(i = 0,	q = 0, group = -1; i < p->GetDataSize (); i++) {
	while (group < GroupCount - 1 && (*p) [q] >= Partition [group +	1] && (*p) [q] != DBL_MAX) group++;
	if (group >= 0)	Frequencies [group]++;
	q++;
    }
    return TRUE;
}

TSample	*TGroupedSample::ConvertTo (TSampleType	st, TWindow *parent) {
    if (st == UNIVARIATE_SAMPLE) {
	TSmoothedSample	*p = new TSmoothedSample (this);
	if (!p->Create (parent)) {
            delete p;
            return NULL;
        }
	return p;
    }
    return TSample::ConvertTo (st, parent);
}

void TGroupedSample::ConverseSample (TWindow *parent) {
    if (GroupCount < 2) {
        ErrorBox (parent, "Need at least two groups to convert sign");
        return;
    }
    TEditField e (IDD_Name, ModifyName ('c') , MAXFILENAME);
    TDialog d (IDD_ConverseSample);
    d << e;
    unsigned ch	= d.Handle (parent);
    if (ch == IDOK) {
	double *NewPartition = new double [GroupCount],
               *NewFrequencies = new double [GroupCount];
	for (int i = 1;	i < GroupCount; i++) {
            NewPartition [i] = -Partition [GroupCount - i];
            NewFrequencies [i] = Frequencies [GroupCount - 1 - i];
        }
        NewPartition [0] = NewPartition [1] - (Partition [GroupCount - 1] - Partition [GroupCount - 2]);
        NewFrequencies [0] = Frequencies [GroupCount - 1];
	TList s;
	s.Append (new TStringEl	("Data set generated by	Xtremes	applying the Converse Sign option to"));
	s.Append (new TStringEl	(GetName ()));
	TGroupedSample *p = new TGroupedSample;
	p->SetName ("Conversed " & GetName ());
	p->SetDescription (&s);
	p->SetData (GroupCount, NewPartition, NewFrequencies, e.ReadBuf ());
	s.DeleteAll (FALSE);
	delete [] NewPartition;
        delete [] NewFrequencies;
    }
}

#endif

void TGroupedSample::SetData (int n, double *par, double *count, String	fn) {
    if (Frequencies) delete [] Frequencies;
    if (Partition) delete [] Partition;
    GroupCount = n;
    Frequencies	= new double [n];
    Partition =	new double [n];
    if (Partition) {
	int Missing = 0;
	for (int i = 0;	i < n; i++)
	    if (count [i] == NOTAVAILABLE || par [i] ==	NOTAVAILABLE)
		Missing++;
	    else {
		Frequencies [i - Missing] = count [i];
		Partition [i - Missing]	= par [i];
	    }
    }
    filename = fn;
    SaveData (0);		// Hier	noch Handle auf	Frame-Window
    SortData ();
    SampleList.InsertSample (this);
}


#ifndef	TERMINAL

TSmoothedSample::TSmoothedSample (TGroupedSample *p0) {
    p =	p0;
}

unsigned TSmoothedSample::EditParameter	(TWindow *parent) {
    TEditField e (IDD_Name, p->ModifyName ('s'), MAXFILENAME);
    TDialog d (IDD_ConverseSample);
    d.SetTitel ("Convert to Univariate");
    d << e;
    unsigned ch	= d.Handle (parent);
    if (ch == IDOK) {
	filename = e.ReadBuf ();
	name = "Univariate " & p->GetName ();
	Description.Append (new	TStringEl ("Data set generated by Xtremes applying the Convert to Univariate option to"));
	Description.Append (new	TStringEl (p->GetName ()));
    }
    return ch;
}

BOOL TSmoothedSample::CreateData (TWindow *parent) {
    int	GroupCount = p->GetGroupCount ();
    DataSize = 0;
    int	index =	GroupCount;
    while (index > 0)
	DataSize += (int) p->GetFrequency (--index);
    if (!GetMemory ()) return FALSE;
    double *q =	RawDataPtr;
    for	(int j = index;	j < GroupCount - 1; j++)
	if (p->GetFrequency (j)	> 0) {
	    double step	= (p->GetPartition (j +	1) - p->GetPartition (j)) / p->GetFrequency (j);
	    double v = p->GetPartition (j) + step / 2.0;
	    for	(int i = 0; i <	p->GetFrequency	(j); i++) {
		*q++ = v;
		v += step;
	    }
	}
    for	(int i = 0; i <	p->GetFrequency	(GroupCount - 1); i++)
	*q++ = p->GetPartition (GroupCount - 1);
    return TRUE;
}


#endif


