/*  samplist.cpp

    List of loaded samples.

    This file is part of Xtremes for Windows,
    Copyright (C) 1991, 1999 Xtremes Group, Siegen.
*/

#include "samplist.hpp"
#include "sample.hpp"

#ifndef TERMINAL
#include "protocol.hpp"
#include "samplwin.hpp"
#include "wintypes.hpp"
#include "censor.hpp"
#include "filedlg.hpp"
#include "ufoglob.hpp"
#include "frame.hpp"

#pragma hdrstop
#include "resource.h"
#include "privmsg.hpp"
#endif

#ifndef TERMINAL
TSampleList::TSampleList () :
  TDialog (IDD_ChooseSampleBox),
  Samples (IDD_SampleList, NULL),
  DeleteButton (IDD_DeleteSample),
  Description (IDD_Description, NULL) {
  Samples.SetTabStops (2, 60, 120);
  operator << (Samples) << Description << DeleteButton;
}
#endif

TSampleList::~TSampleList () {
    DeleteAll (TRUE);
}

#ifdef TERMINAL

TSample *TSampleList::GetActiveSample () {
    return (TSample *) GetFirst ();
}

#else

TSample *TSampleList::GetActiveSample (TWindow *parent) {
    TSample *p = (TSample *) GetFirst ();
    if (!p) {
        LoadSample (parent);
        return (TSample *) GetFirst ();
    }
    return p;
}

BOOL TSampleList::CheckActiveSample (TWindow *parent, TSampleType Required) {
    String Accept ("The selected operation cannot be applied to "),
	   Choose ("Please choose another active data set using the Data/Choose Data or Data/Read Data option."),
           Msg;
    TSample *p = (TSample *) GetFirst ();
    if (p) {
        TSampleType Current = p->GetSampleType ();
        switch (Required) {
            case UNIVARIATE_GROUPED_REQUIRED:
                if (Current == TIMESERIES_SAMPLE)
		    Msg = Accept & "data sets containing time series." & Choose;
		break;
	    case UNIVARIATE_REQUIRED :
		if (Current == GROUPED_SAMPLE)
		    Msg = Accept & "grouped data sets. " & Choose;
		if (Current == TIMESERIES_SAMPLE)
		    Msg = Accept & "data sets containing time series."& Choose;
		break;
	    case BIVARIATE_BASE_REQUIRED:
		if (Current != TIMESERIES_SAMPLE)
		    Msg = "The selected operation can only be applied to data sets containing time series. " & Choose;
		break;
	    case GROUPED_SAMPLE :
		if (Current == UNIVARIATE_SAMPLE)
		    Msg = Accept & "univariate data sets.  " & Choose;
		if (Current == TIMESERIES_SAMPLE)
		    Msg = Accept &  "data sets containing time series." & Choose;
                break;
            case TIMESERIES_SAMPLE:
                if (Current != TIMESERIES_SAMPLE)
		    Msg = "The selected operation can only be applied to data sets containing time series. " & Choose;
		break;
        }
        if (Msg.Len ()) {
            ErrorBox (parent, Msg);
            return FALSE;
        }
        return TRUE;
    }
    return FALSE;
}


BOOL TSampleList::LoadSample (TWindow *parent) {
    String fn;
    if (GetOpenFileName (parent, fn, MAXFILENAME, "Data files (*.dat)\0*.dat\0All files (*.*)\0*.*\0", "Load data set"))
        return LoadSample (fn);
    else return FALSE;
}

void TSampleList::NotifyInitDialog () {
    inherited::NotifyInitDialog ();
    ChangeLists ();
}

void TSampleList::ChangeLists () {
    Samples.SetList (&SampleCopy);
    if (SampleCopy.Entries ()) {
        TSample *p = (TSample *) SampleCopy.GetEntry (Samples.Selection ());
        if (p) Description.SetList (p->GetDescription ());
        DeleteButton.Enable ();
    } else {
        Description.SetList (NULL);
        DeleteButton.Disable ();
    }
}

void TSampleList::NotifyActionEvent (TDialogMember *m) {
    int id = m->GetID ();
    TSample *p = (TSample *) SampleCopy.GetEntry (Samples.Selection ());
    if (p && id == IDD_SampleList)	// ???? && codeNotify == LBN_SELCHANGE)
        Description.SetList (p->GetDescription ());
    else if (p && id == IDD_DeleteSample) {
        p->SetDelete (TRUE);
        SampleCopy.DeleteActive ();
        Deleted = TRUE;
        ChangeLists ();
    } else inherited::NotifyActionEvent (m);
}

void TSampleList::ChooseActiveSample (TWindow *parent) {
    TSample *p;
    for (p = (TSample *) GetFirst (); p; p = (TSample *) GetNext ()) {
        SampleCopy.Append (p);
        p->SetDelete (FALSE);
    }
    Deleted = FALSE;
    if (Handle (parent) == IDOK) {
        if (Deleted) {
            ((TExtremesMDIFrame *) PlotWindowManager.GetFramePtr ())->ValidateAllFunctions ();
            p = (TSample *) GetFirst ();
            while (p)
                if (p->IsDeleted ()) {
                    DeleteActive (TRUE);
                    p = (TSample *) GetActive ();
                } else p = (TSample *) GetNext ();
        }
        p = (TSample *) GetEntry (Samples.Selection ());
        if (p) {
            DeleteActive ();
            InsertSample (p);
        } else if (Deleted) {
            TMDIChildWindow *w = TSampleWindow::GetOpenWindow (PlotWindowManager.GetFramePtr ());
            w->Close ();
        }
        ((TExtremesMDIFrame *) PlotWindowManager.GetFramePtr ())->HandleSampleChanged ();
        if (p) Protocol << ("New active sample: " & p->Name());
        else Protocol << "New active sample: None";
        ActualSample = p;
    }
    SampleCopy.DeleteAll ();
}

BOOL TSampleList::DeleteDuplicates () {
    return FALSE;
}

#endif

void TSampleList::InsertSample (TSample *p) {
    String t = p->GetFileName ();
    for (TSample *q = (TSample *) GetFirst (); q; q = (TSample *) GetNext ())
        if (t == q->GetFileName ()) {
            q->SetDelete (TRUE);
#ifndef TERMINAL
            ((TExtremesMDIFrame *) PlotWindowManager.GetFramePtr ())->ValidateAllFunctions ();
#endif
            DeleteActive (TRUE);
            break;
        }

    GetFirst ();
    InsertBeforeActive (p);
#ifndef TERMINAL
    TSampleWindow::GetOpenWindow (PlotWindowManager.GetFramePtr ())->SetSample (p);
    ((TExtremesMDIFrame *) PlotWindowManager.GetFramePtr ())->HandleSampleChanged ();
    Protocol << ("New active sample: " & p->Name ());
    ActualSample = p;
#endif
}


BOOL TSampleList::LoadSample (String fn) {
    BOOL Result = FALSE;
    TSample *p = NULL;
    TTextFileReader f (fn);

    char ch = f.GetNextChar ();
    f.UngetChar (ch);
    if (ch == 'X') {
        String buf = f.GetLine ();
        if (buf == String (SampleHeaders [UNIVARIATE_SAMPLE]))
            p = new TUnivariateSample;
        else if (buf == String (SampleHeaders [GROUPED_SAMPLE]))
            p = new TGroupedSample;
        else if (buf == String (SampleHeaders [TIMESERIES_SAMPLE]))
            p = new TTimeSeriesSample;
    #ifndef TERMINAL
        else if (buf == String (SampleHeaders [CENSORED_SAMPLE]))
            p = new TCensoredSample;
    #endif
        else if (buf == String (SampleHeaders [MULTIVARIATE_SAMPLE]))
            p = new TMultivariateSample;
        else if (buf == String (SampleHeaders [FREQUENCY_SAMPLE]))
            p = new TFrequencySample;
        if (!p) {
            #ifdef TERMINAL
                puts ("Unknown sample type");
            #else
                ErrorBox (PlotWindowManager.GetFramePtr (), fn & " does not contain a valid sample type");
            #endif
        } else
            Result = p->LoadData (f);
    } else {
        p = new TMultivariateSample ();
        Result = ((TMultivariateSample *) p)->LoadUntypedData (f);
    }
    if (!Result) {
        String t = p->GetReadMessage ();
#ifndef TERMINAL
        ErrorBox (PlotWindowManager.GetFramePtr (), t & form (50, " in line %d", f.GetLineNumber ()));
#else
        printf (t & form (50, " in line %d", f.GetLineNumber ()));
#endif
        delete p;
    } else {
        p->SetFileName (fn);
        InsertSample (p);
    }
    return Result;
}


// Declaration of global sample list


TSampleList SampleList;

