/*  scanner.hpp
 *
 *  Declaration of XPL scanner.
 *
 *  This file is part of XPL.
 *  Copyright (C) 1998 Xtremes Group, Siegen.
 */

#ifndef SCANNER_HPP
#define SCANNER_HPP

#include "boolean.hpp"
#include "string.hpp"

#include "maxstat.hpp"

enum TToken {

    // program symbols
    BEGIN, END, IF, THEN, ELSE, FOR, TO, DOWNTO, DO, _CONST, TYPE, VAR, WHILE,
    REPEAT, UNTIL, RECORD, ARRAY, SET, VECTOR, MATRIX, OF, PROCEDURE,
    FUNCTION, RETURN, CASE, RANGE, NIL, FORWARD, UNIT, INTERFACE,
    IMPLEMENTATION, USES, LABEL, GOTO,

    // extensions for estimators
    ESTIMATOR, RETURNS, PROGRAM, _COMPONENT, INPORTS, OUTPORTS,

    // operators
    _IN, AND, OR, XOR, NOT, DIVIDE, MODULO, ADD, SUB, MUL, DIV, POW, FAC,

    EQUAL, GREATERTHAN, LESSTHAN,               // Note: order must not
    GREATEREQUAL, LESSEQUAL, NOTEQUAL,          //       be changed

    // other symbols
    BRACKETOPEN, BRACKETCLOSE, SQUAREBRACKETOPEN, SQUAREBRACKETCLOSE,
    COMMA, SEMICOLON, COLON, ENDPOINT, DEFINE, TWOPOINT, DEREF,
    IDENTIFIER, CHARCONST, INTCONST, DBLCONST, STRCONST, TERMINATOR,

    // unrecognized token
    ERROR_TOKEN, 

    // end entry for panic recovery lists
    END_OF_LIST
};

class TScanner {
    public:
	TScanner ();
	TScanner (String fn);
        virtual ~TScanner ();

	BOOL IsValid ()			{ return validFlag; }
        void SetProgram (const char *, String FileName = "");
        void Restart ();

	void GetNextToken ();
        TToken GetToken ()              { return cToken; }
	BOOL Try (TToken c);

        double GetDouble ()             { return dValue; }
	int GetInteger ()               { return iValue; }
	String GetString ()             { return sValue; }
	int GetLine ()                  { return Line; }
        String GetFile ()		{ return fn; }		// file currently scanned 
    private:
	BOOL SkipWhiteSpace ();
	BOOL SkipComment ();
	void ParseNumber ();
	void ParseIdentifier ();
	void ParseString (char TermChar);
	void ParseOperator ();
	void SetTokenInc (TToken c);
	TToken cToken;
	double dValue;
	String sValue;
	int iValue;
	int Line;
	static char *Names [];
	const char *fpos, *startpos;

	// scanning files
	BOOL validFlag;
        String fn;
	FILE *f;
	char *buffer;
};

#endif
