/*  slope.cpp

    Implemenation of slope (GP) estimator.

    This file is part of Xtremes for Windows,
    Copyright (C) 1991, 1997 Xtremes Group, Siegen.
*/

#include <math.h>
#include "allest.hpp"
#include "mathfunc.hpp"
#include "stddef.hpp"

#ifndef TERMINAL
#include "resource.h"
#endif

TSlopeEstimator::TSlopeEstimator (BOOL AddMenu)
#ifdef TERMINAL
  : TUnivariateGPDEstimator ("Slope(GP)") {
#else
  : TUnivariateGPDEstimator ("Slope(GP)", IDM_SlopeEstimator, IDD_Estimator, AddMenu) {
#endif
    Data.SetMode (BETA_MODE);
}

TEstMode TSlopeEstimator::GetEstimatorMode () {
    return BETA_MODE;
}

TEstResult TSlopeEstimator::Estimate (int k, TSample *q) {
    TUnivariateSample *p = (TUnivariateSample *) q;
    int n = p->GetDataSize ();
    if (k < 4) return EST_EXT;
    if (k > n) return EST_PAR;

    double *x = new double [k],
	   *y = new double [k],
	   sum = 0.0;

    for (int i = 1; i < k; i++) {
	sum += (*p) [n - i];
	x [k - i - 1] = (*p) [n - i - 1];
	y [k - i - 1] = sum / i - (*p) [n - i - 1];
    }
    double *v = PolynomialRegression (1, k - 1, x, y);
    double s = v [1];
    delete [] x;
    delete [] y;
    delete [] v;

    if (s == -1.0) return EST_FAILURE;
    RawParameters [0] = s / (s + 1.0);

    TransformModels (k, p);
    return EST_OK;
}

TEstimatorData *TSlopeEstimator::GetParameter () {
    return &Data;
}

TEstimator *TSlopeEstimator::Duplicate () {
    TSlopeEstimator *p = new TSlopeEstimator;
    p->Data = Data;
    return p;
}

int TSlopeEstimator::GetMinK () {
    return 4;
}

void TSlopeEstimator::CalcEstimator (Array<double> &alpha, int &kmax, TSample *q) {
    TUnivariateSample &p = *(TUnivariateSample *) q;
    int k = GetMinK ();
    for (; k <= p.GetDataSize (); k++) {
	Estimate (k, q);
	alpha [k] = RawParameters [0];
    }
    kmax = p.GetDataSize ();
}
