/*  stddef.hpp

    Inline-Deklarationen haeufig benoetiger Funktionen
*/

#ifndef STDDEF_HPP
#define STDDEF_HPP

#include <assert.h>
#include "string.hpp"

#if !(defined (__BORLANDC__) && __BORLANDC__ < 0x0540)
  #include <algorithm>
#endif

#ifdef __GNUG__
using std::min;
using std::max;
#endif

#if defined (_MSC_VER) || (defined (__BORLANDC__) && __BORLANDC__ < 0x0540 && !defined (__MINMAX_DEFINED))

#ifdef max
#undef max
#endif

#ifdef min
#undef min
#endif

template <class T>
inline T max (T a, T b) {
    return a > b ? a : b;
}

template <class T>
inline T min (T a, T b) {
    return a < b ? a : b;
}

#define __MINMAX_DEFINED

#endif

template <class T>
inline T max (T a, T b, T c) {
    return max (max (a, b), c);
}

template <class T>
inline T min (T a, T b, T c) {
    return min (min (a, b), c);
}


template <class T>
inline T sqr (T a) {
    return a * a;
}

template <class T>
inline void Adjust (T &a, T amin, T amax) {
    if (a > amax) a = amax;
    if (a < amin) a = amin;
}

template <class T>
inline T adjust (T a, T amin, T amax) {
    if (a > amax) return amax;
    if (a < amin) return amin;
    return a;
}

template <class T>
inline void Swap (T &a, T &b) {
    T c = a;
    a = b;
    b = c;
}

template <class T>
inline T square (T x) {
    return x * x;
}

// Comparison function for quicksort

#ifndef _USERENTRY
#define _USERENTRY
#endif

int _USERENTRY realcomp (const void *a, const void *b);

// Sort concomitant

template <class T>
void sort (int l, int r, double *first, T *second){
  int i, j;
  double x; T y;
  i=l; j=r; x=first[(l+r)/2]; y = second [(l+r)/2];
  do {
	 while (first[i]<x || (first[i]==x && second[i]>y)) i++;
	 while (x<first[j] || (first[j]==x && second[j]<y)) j--;
	 if (i<=j) {

		Swap (first[i], first[j]);
		Swap (second[i], second[j]);
		i++; j--;
	 }
	 } while (i<=j);
  if (l<j) sort(l,j,first,second);
  if (i<r) sort(i,r,first,second);
}



// Array class

template <class T> class Array {
    public:
	Array (int minindex, int maxindex);
	~Array ();
	T & operator [] (int a);
    private:
	int minind, maxind;
	T *p;
};

template <class T> Array<T>::Array (int minindex, int maxindex) {
    minind = minindex; maxind = maxindex;
    p = new T [maxind - minind + 1];
}

template <class T> Array<T>::~Array () {
    delete [] p;
}

template <class T> T & Array<T>::operator [] (int a) {
    assert (minind <= a && a <= maxind);
    return p [a - minind];
}

#define MYMAXDOUBLE 1e300
#define MYMINDOUBLE 1e-300
#define MY_PI	    3.1415926

BOOL ParseReal (const char *fpos, double &value);

#endif
