#include <assert.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>

#ifndef TERMINAL
#ifdef _WINDOWS
#include "globals.hpp"
#include <windows.h>
#endif
#endif

#include "string.hpp"

String::String (const char *s, BOOL MakeCopy) {
    assert (s != NULL);
    p = new StringData;
    p->s = MakeCopy ? mystrdup (s) : s;
    p->RefCount++;
    p->Len = strlen (s);
}

String::String (const String &s) {
    p = s.p;
    p->RefCount++;
}

String::String (const char c) {
    char buf [2];
    buf [0] = c;
    buf [1] = '\0';
    p = new StringData;
    p->Len = 1;
    p->s = mystrdup (buf);
    p->RefCount++;
}

String::~String () {
    DecRefCount ();
}

void String::DecRefCount () {
    if (!--p->RefCount) delete p;
}

String String::operator = (const String s) {
    DecRefCount ();
    p = s.p;
    p->RefCount++;
    return *this;
}

String String::operator () (int pos, int len) {
    char *buf = new char [len + 1];
    *buf = '\0';
    if (pos <= Len ()) {
        strncpy (buf, p->s + pos, len);
        buf [len] = '\0';
    }
    String s (buf);
    delete [] buf;
    return s;
}

int String::Len () const {
    return p->Len;
}

//char *String::operator const char * () {
//    return p->s;
//}

String operator & (const String s, const String t) {
    char *buf = new char [s.Len () + t.Len () + 1];
    strcpy (buf, s);
    strcpy (buf + s.Len (), t);
    String u (buf, FALSE);
    return u;
}

BOOL operator == (const String s, const String t) {
    return strcmp (s.p->s, t.p->s) == 0;
}

BOOL operator != (const String s, const String t) {
    return strcmp (s.p->s, t.p->s) != 0;
}

BOOL operator < (const String s, const String t) {
    return strcmp (s.p->s, t.p->s) < 0;
}

BOOL operator > (const String s, const String t) {
    return strcmp (s.p->s, t.p->s) > 0;
}

BOOL operator <= (const String s, const String t) {
    return strcmp (s.p->s, t.p->s) <= 0;
}

BOOL operator >= (const String s, const String t) {
    return strcmp (s.p->s, t.p->s) >= 0;
}

String form (int MaxLen, const char *fmt, ...) {
    char *buf = new char [MaxLen];
    va_list argptr;
    va_start (argptr, fmt);
    vsprintf (buf, fmt, argptr);
    String s (buf);
    va_end (argptr);
    delete [] buf;
    return s;
}

char *mystrdup (const char *s) {
    int n = strlen (s) + 1;
    char *t = new char [n];
    memcpy (t, s, n);
    return t;
}

void StringToFile (FILE *f, String s) {
    int size = s.Len ();
    fwrite (&size, sizeof (size), 1, f);
    fwrite ((const char *) s, sizeof (char), (int) size, f);
}

String StringFromFile (FILE *f) {
    int size;
    fread (&size, sizeof (size), 1, f);
    char *buf = new char [(int) size + 1];
    fread (buf, sizeof (char), (int) size, f);
    buf [(int) size] = '\0';
    return String (buf, FALSE);         // grabs buffer
}

String::StringData::~StringData () {
    // Borland C++ 4.52 does not allow delete on const pointers.
#if __BORLANDC__ < 0x0500
    char *t;
    memcpy (&t, &s, sizeof (t));
    if (t) delete [] t;
#else
     if (s) delete [] s;
#endif
}

