/*  textfile.cpp

    Class to access text file.

    This file is part of Xtremes for Windows,
    Copyright (C) 1994 Xtremes Group, Siegen.
*/

#include <ctype.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>

#include "textfile.hpp"

#define TERMINAL

// weg mit diesem Unsinn!!!!

TTextFileReader::TTextFileReader (String fn)
 : BufSize (512) {
    buf = new char [BufSize];
    filename = fn;
#ifdef TERMINAL
    hFile = fopen (fn, "rt");
    if (!hFile)
#else
    OFSTRUCT ofstruct;
    hFile = OpenFile (fn, &ofstruct, OF_READ | OF_SHARE_DENY_WRITE);
    if (hFile == -1)
#endif
	Error = FILE_NOT_FOUND;
    else {
	FillBuf ();
	LineCount = 1;
	charunget = -2;
	Error = ERR_NO_ERROR;
    }
}

TTextFileReader::~TTextFileReader () {
#ifdef TERMINAL
    if (hFile) fclose (hFile);
#else
    if (hFile) _lclose (hFile);
#endif
    delete [] buf;
}

void TTextFileReader::Reset () {
#ifdef TERMINAL
    fseek (hFile, 0, SEEK_SET);
#else
    _llseek (hFile, 0, FILE_BEGIN);
#endif
    FillBuf ();
}

void TTextFileReader::FillBuf () {
#ifdef TERMINAL
    if (hFile) read = fread (buf, sizeof (char), BufSize, hFile);
    else read = 0;
#else
    if (hFile != -1) read = _lread (hFile, buf, BufSize);
    else read = 0;
#endif
    bufptr = buf;
}

int TTextFileReader::GetNextChar () {
    if (charunget != -2) {
	int h = charunget;
	charunget = -2;
	if (h == EOF) Error = END_OF_FILE;
	return h;
    }
    if (bufptr >= buf + read)
	if (read) {
	    FillBuf ();
	    return GetNextChar ();
	} else {
	    Error = END_OF_FILE;
	    return EOF;
	}
    if (*bufptr == '\r') {
	bufptr++;
	return GetNextChar ();
    }
    if (*bufptr == '\n') {
	LineCount++; bufptr++;
	return 0;
    }
    if (*bufptr == '\x1A') {
	Error = END_OF_FILE;
	return EOF;
    }
    if (CommentMode && *bufptr == '%') {
        bufptr++;
        int ch;
        do
            ch = GetNextChar ();
        while (ch != 0 && ch != EOF);
        return ch;
    }
    return *bufptr++;
}

String TTextFileReader::GetLine () {
    char s [100], *u = s;
    int c;
    String t ("");
    while ((c = GetNextChar ()) != EOF && c != '\0') {
	*u++ = (char) c;
	if (u - s > 98) {
	    *u = '\0';
            t = t & s;
	    u = s;
	}
    }
    *u = '\0';
    t = t & s;
    return t;
}

String TTextFileReader::GetString (BOOL Lower) {
    char s [100], *u = s;
    String t ("");
    int c = GetNextChar ();
    while (c == ' ' || !c) c = GetNextChar ();
    while (c >= 'A' && c <= 'Z' || c >= 'a' && c <= 'z') {
	if (Lower) *u++ = (char) (c | 32);
	else *u++ = (char) c;
	if (u - s > 98) {
	    *u = '\0';
	    t = t & u;
	    u = s;
	}
	c = GetNextChar ();
    }
    UngetChar (c);
    *u = '\0';
    t = t & s;
    return t;
}

String TTextFileReader::GetQuotedString (BOOL Lower) {
    char s [100], *u = s;
    String t ("");
    int c = GetNextChar ();
    while (c == ' ' || c == '\t' || !c) c = GetNextChar ();
    if (c == '\"') {
	c = GetNextChar ();
	while (c && c != EOF && c != '\"') {
	    if (Lower) *u++ = (char) (c | 32);
	    else *u++ = (char) c;
	    if (u - s > 98) {
		*u = '\0';
		t = t & s;
		u = s;
	    }
	    c = GetNextChar ();
	}
	*u = '\0';
	t = t & s;
	if (c == '\"') return t;
    }
    Error = PARSE_ERROR;
    return "";
}

BOOL TTextFileReader::UngetChar (int c) {
    if (charunget != -2) return FALSE;
    if (c == EOF && Error == END_OF_FILE) Error = ERR_NO_ERROR;
    charunget = c;
    return TRUE;
}

TTextFileReader::TErrorCode TTextFileReader::GetError () {
    TErrorCode c = Error;
//    Error = ERR_NO_ERROR;
    return c;
}

long TTextFileReader::GetLong () {
    char s [100], *u = s;
    int c = GetNextChar ();
    while (c == ' ' || c == '\t' || !c) c = GetNextChar ();
    if (c == EOF) return 0;
    while (isdigit (c) && u - s < 98) {
	*u++ = (char) c;
	c = GetNextChar ();
    }
    UngetChar (c);
    if (u > s && u - s < 98) {
	*u = '\0';
	long Result = strtol (s, &u, 10);
	if (!*u) return Result;
    }
    if (!IsError ()) Error = PARSE_ERROR;
    return 0;
}

int TTextFileReader::GetInt () {
    long Result = GetLong ();
    if ( Result <= INT_MAX && Result >= -INT_MAX) return (int) Result;
    if (!IsError ()) Error = NUMBER_TOO_BIG;
    return 0;
}

double TTextFileReader::GetDouble () {
    char s [100], *u = s;
    int c = GetNextChar ();
    while (c == ' ' || c == '\t' || !c) c = GetNextChar ();
    if (c == EOF) return 0.0;
    while (c && ((c >= '0' && c <= '9') || strchr ("+-eE.,", c)) && u - s < 98) {
	*u++ = c == ',' ? '.' : (char) c;
	c = GetNextChar ();
    }
    UngetChar (c);
    if (u > s && u - s < 98) {
	*u = '\0';
	double Result = strtod (s, &u);
	if (!*u && Result != HUGE_VAL && Result != -HUGE_VAL) return Result;
    }
    if (!IsError ()) Error = PARSE_ERROR;
    return 0;
}

double TTextFileReader::GetDoubleNA () {        // !!!! mit GetDouble vereinheitlichen
    char s [100], *u = s;
    int c = GetNextChar ();
    while (c == ' ' || c == '\t' || !c) c = GetNextChar ();
    if (c == EOF) return 0.0;
    while (c && ((c >= '0' && c <= '9') || strchr ("+-eE.,", c)) && u - s < 98) {
	*u++ = c == ',' ? '.' : (char) c;
	c = GetNextChar ();
    }
    UngetChar (c);
    if (u > s && u - s < 98) {
	*u = '\0';
	if (u == s + 1 && *s == '.') return NOTAVAILABLE;
	double Result = strtod (s, &u);
	if (!*u && Result != HUGE_VAL && Result != -HUGE_VAL) return Result;
    }
    if (!IsError ()) Error = PARSE_ERROR;
    return 0;
}

