/*  textfile.hpp

    Class to access text file.

    This file is part of Xtremes for Windows,
    Copyright (C) 1991, 2000 Xtremes Group, Siegen.
*/

#ifndef TEXTFILE_HPP
#define TEXTFILE_HPP

#include "string.hpp"

#define NOTAVAILABLE    -1.2345e305

class TTextFileReader {
    public:
	enum TErrorCode { ERR_NO_ERROR, PARSE_ERROR, NUMBER_TOO_BIG, END_OF_FILE, FILE_NOT_FOUND };
	TTextFileReader (String fn);
	~TTextFileReader ();
        void Reset ();                  // reopen file and start reading from beginning
	String GetLine ();
	int GetNextChar ();         	// returns 0 at end of line, -1 at end of file
	BOOL UngetChar (int c);
	int GetInt ();
	long GetLong ();
	double GetDouble ();
	double GetDoubleNA ();		// return NOTAVAILABLE if point (".") is read
	String GetString (BOOL Lower = FALSE);
	String GetQuotedString (BOOL Lower = FALSE);
	int GetLineNumber ()        	{ return LineCount; }
	TErrorCode GetError ();
	BOOL IsError ()			{ return Error != ERR_NO_ERROR; }
        void SetCommentMode (BOOL f)    { CommentMode = f; }
    private:
	const int BufSize;
	char *buf, *bufptr;
	int read;
	FILE *hFile;
	int LineCount;
	int charunget;
	void FillBuf ();
	TErrorCode Error;
        BOOL CommentMode;
        String filename;
};

#endif

