/*  xpl.hpp

    Adds Xtremes specific routines (access to data sets,
    distribution, estimators) to the StatPascal compiler.

    This file is part of StatPascal.
    Copyright (C) 1998, 2000 Xtremes Group, Siegen.
*/

#ifndef XPL_HPP
#define XPL_HPP

#include "compiler.hpp"
#include "samplist.hpp"
#include "allest.hpp"

#ifdef COMPONENT
  #ifdef ACETAO
    #include "xgplS.h"          // ACE+TAO
  #else
    #include "xgpl.h"           // MICO
  #endif
#endif

#ifndef TERMINAL
#include "terminal.hpp"
#endif

enum { CTS = SPHALT + 1, CS, CCS, CMS, MCMS, MSC, MSF,
       LDT, LDS, LDD, LDA, LDU, CNAME, CDATA, RDATA,
       GP1HILL, GP1HILLA, GP1ME, GP1MEA, GPMLE, GPMLEA,
       GPMOMENT, GPMOMENTA, GPDREES, GPDREESA, 
       EV1MLE, EV1MLEA, EVMLE, EVMLEA, EVMDE, EVMDEA, EVRDE, EVRDEA,
       GP0MLE, GP0MLEA, EV3MLE, EV3MLEA,
       GP1HILLCEN, GPMLECEN,
       TMLE, TMLEA, NCTMLE, NCTMLEA,
       GAUSSMLE, GAUSSMLEA, GAUSSLSE, GAUSSLSEA, GAUSSMDE, GAUSSMDEA,
       GAUSSMHDE, GAUSSMHDEA,
       SDF, SQF, SDEN, SMCS, SREAD, PLOT, BOXPLOT, PLOT3D, CONT3D, CLWIN,
       MBOX, VBOX, CBOX,
       EXEC, PLAY,

       FPARETORV, FGAUSSRV, FNORMALRV, FCAUCHYRV, FFRECHETRV,
       FBETARV, FWEIBULLRV, FGUMBELRV, FEXPONENTIALRV,
       FGPDRV, FEVDRV, FGAMMARV, FCHIRV, FTRV,
       VFPARETORV, VFGAUSSRV, VFNORMALRV, VFCAUCHYRV, VFFRECHETRV,
       VFBETARV, VFWEIBULLRV, VFGUMBELRV, VFEXPONENTIALRV,
       VFGPDRV, VFEVDRV, VFGAMMARV, VFCHIRV, VFTRV,

       FPARETODEN, FPARETOQF, FPARETODF,
       FGAUSSDEN, FGAUSSQF, FGAUSSDF,
       FGAUSSDENV, FGAUSSQFV, FGAUSSDFV,
       FCAUCHYDEN,
       FFRECHETDEN, FFRECHETQF, FFRECHETDF,
       FBETADEN, FBETAQF, FBETADF,
       FWEIBULLDEN, FWEIBULLQF, FWEIBULLDF,
       FEXPONENTIALDEN, FEXPONENTIALQF, FEXPONENTIALDF,
       FGUMBELDEN, FGUMBELQF, FGUMBELDF,
       FGPDDEN, FGPDQF, FGPDDF,
       FEVDDEN, FEVDQF, FEVDDF,
       FCHIDEN, FCHIQF, FCHIDF,
       FGAMMADEN, FGAMMADF,
       FTDEN, FTDF, FTQF,
       FNCTDEN, FNCTDF, FNCTQF,
       FHRDEN, FHRDF, FHRSF, FHRGPDEN, FHRGPDF, FHRGPSF,
       FGMFDEN, FGMFDF, FGMFSF, FGMFGPDEN, FGMFGPDF, FGMFGPSF,
       FGAMMA, FRUINSIM, FRUINSIMB,

       FUNIFORM,

       TST, TSWELCH, TSWILCOXON, 

       FMEAN, FVAR, FMOM, FMEDIAN, FSMOOTH, POLY,
       REG, SCA, SETCOL, SETLIN, SETSTY, SETOPT, SEPS,
       SETCS, SETCT, SETCM, SETCF, SETLF, SETLB,

       SGV, GGV, SGS, GGS,					// globals

       EXTENSION_FIRST };

class TXPLPCodeInterpreter : public TPCodeInterpreter {
    public:
	TXPLPCodeInterpreter ();
	virtual ~TXPLPCodeInterpreter ();
	void SetActiveSample (TSample *s)       { ActSample = s; }
	String GetRunTimeErrorMessage (TRunTimeErrorType);
    protected:
        virtual TRunTimeErrorType ExtendedCode (TPCode &c);     // handles PCodes listed above
        TRunTimeErrorType CallPOTEstimator (TEstimator *, BOOL VectorGiven, int NrOfParameters = 3);
        TRunTimeErrorType CallOtherEstimator (TEstimator *, BOOL VectorGiven, int NrOfParameters = 3);
	TSample *CreateTempSample ();
	void GetStringPartition (String a, int *&p, int &Nr);

        TSample *ActSample;
        TMultivariateSample *cms;
#ifndef TERMINAL

        void ExecuteProcedure ();
        void ChiSquareStatistic ();

        TTerminalWindow *GetWindowPointer ();
        TTerminalWindow *w;

        // plot options
        int PlotColor, LineStyle, LineThickness, LineLine, LineGap, PlotMode;
	TFont PlotFont;

#endif
};

class TXPLCompiler : public TCompiler {
    typedef TCompiler inherited;
    public:
	TXPLCompiler (TPCodeInterpreter *p, int ProgramSize, String PathName);
	virtual void Compile ();
#ifdef COMPONENT
	virtual BOOL IsComponent ();
	String GetComponentName ();
	int GetNumberOfInports ();
	XGPLNodes::TPortType *GetInportTypes ();
	int GetNumberOfOutports ();
	XGPLNodes::TPortType *GetOutportTypes ();
#endif
    protected:
	virtual void BuildUserSymbols ();
	virtual void BuildPredefinedFunctions ();


    private:
	void AddGPEstimator (String Name, int CodeFull, int CodeActiveSample);
	void AddOtherEstimator (String Name, int CodeFull, int CodeActiveSample, int NrOfParameter = 3);
#ifdef COMPONENT
	BOOL ComponentFlag;
	void Component ();
	String ComponentName;
	TRoutine *InputParameters;
	XGPLNodes::TPortType ConvertType (TType *);
#endif

};

#endif



